/***********************************************************************
 * File: BinSemaphoreTest.java
 * 	(Extracted from
 *	   "$Id: BinSemaphoreTest.java,v 1.2 2004/02/22 12:12:55 maoyam Exp maoyam $")
 *
 * Abstracts:
 *   This Java source code implements the test program of
 * the binary-semaphore utility class `my_util.BinSemaphore'.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import	my_util.BinSemaphore;

public class BinSemaphoreTest extends Thread {
	private BinSemaphore	semaphore = null;
	private BinSemaphore	semaphore2 = null; // for randezvous

	static final long SLEEP_MSECS = 2345;
	private long sleep_msecs = 0;	// --create-taken test

	static final int	TAKER	= 1, POLLER = 2;
	private int	taker = 0; // --shared_object test : threads kind

	static int	x	= 0; // --simple test : shared class field
	private boolean	even_actor;
	private boolean sem_enable;

	// --simple test
	public BinSemaphoreTest(BinSemaphore sem
						, boolean even_actor
						, boolean sem_enable
						) {
		System.out.print("BinSemaphoreTest:--simple test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		semaphore	= sem;
		this.even_actor	= even_actor;
		this.sem_enable	= sem_enable;
	}

	// --shared-object test
	public BinSemaphoreTest(BinSemaphore [] sem, int taker) {
		System.out.print("BinSemaphoreTest:--shared-object test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		semaphore	= sem[0];
		semaphore2	= sem[1];
		this.taker	= taker;
	}

	// --create-taken test
	public BinSemaphoreTest(long sleep_msecs, BinSemaphore sem) {
		System.out.print("BinSemaphoreTest:--create-taken test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		this.sleep_msecs	= sleep_msecs;
		if (sleep_msecs < 0) semaphore	= sem;
	}

	public void run() {
		boolean	blocking	= true;
		long	sleep_interval;
		int		loop_cnt;

		System.out.print("BinSemaphoreTest.run():: this.hashCode() = "
						+ this.hashCode()
						+ "\n"
						);
		if (sleep_msecs > 0) {
			// --create-taken test
			BinSemaphoreTest child;

			System.out.print("CREATE-TAKER: starts...\n");
			StringBuffer msg = new StringBuffer("How are you?");
			semaphore	= new BinSemaphore(this, msg); {
				if (!semaphore.take(this, true)) {
					System.out.print("CREATE-TAKER: Twice "
									+ "semaphore.take("
									+ this + ", true) returns false."
									+ "\n"
									);
					// For Test
				}

				System.out.print("CREATE-TAKER: make child to start\n");
				child = new BinSemaphoreTest(-sleep_msecs, semaphore);
				child.start();

				System.out.print("CREATE-TAKER: sleeps...\n");
				try { this.sleep(sleep_msecs);
				} catch (Exception e) { ; }

				if (!semaphore.leave(this)) {
					System.out.print("CREATE-TAKER: semaphore.leave("
									+ this + ") returns false."
									+ "\n"
									);
					return;	// FATAL
				}
				if (!semaphore.leave(this)) {
					System.out.print("CREATE-TAKER: Twice "
									+ "semaphore.leave("
									+ this + ") returns false."
									+ "\n"
									);
					// For test
				}
			}

			try { child.join(); } catch (Exception e) { ; }

			System.out.print("CREATE-TAKER: child answer is \""
							+ msg + "\"."
							+ "\n"
							);

			System.out.print("CREATE-TAKER: exits.\n");
			return;
		}
		else if (sleep_msecs < 0) {
			// --create-taken test
			StringBuffer msg;

			System.out.print("CREATE-CHILD: starts...\n");
			if (semaphore.take(this, true)) {
				try { msg = (StringBuffer)
							semaphore.get_exclusive_object(this);
				} catch (Exception e) {
					System.out.print("CREATE-CHILD: semaphore."
									+ "get_exclusive_object(" + this
									+ ") returns false."
									+ "\n"
									);
					return;	// FATAL
				}

				System.out.print("CREATE-CHILD: parent msg is \""
								+ msg + "\""
								+ ".\n"
								);

				System.out.print("CREATE-CHILD: sleeps...\n");
				try { this.sleep(-sleep_msecs);
				} catch (Exception e) { ; }

				msg.append(" --- I am Fine!");
				System.out.print("CREATE-CHILD: edited msg is \""
								+ msg + "\""
								+ ".\n"
								);

				if (!semaphore.leave(this)) {
					System.out.print("CREATE-CHILD: semaphore."
									+ "leave(" + this + ") "
									+ "returns false."
									+ "\n"
									);
					return;	// FATAL
				}
			}

			System.out.print("CREATE-CHILD: exits.\n");
			return;
		}
		else if (taker == BinSemaphoreTest.TAKER) {
			// --shared-object test
			// Tester for taking shared object (taker thread)
			System.out.print("Taker: starts...\n");
			blocking	= true;
			sleep_interval	= 270;

			if (!semaphore.take(this, blocking)) {
				System.out.print("Taker blocking = " + blocking
								+ ": semaphore.take(" + this
								+ ", " + blocking + ") returns false."
								+ "\n"
								);
			} else {
				StringBuffer	shared_obj;
				try {
					shared_obj = (StringBuffer)
								semaphore.get_exclusive_object(this);

					shared_obj.append("Hi, ");
					System.out.print("Taker: edited (#1) shared_obj "
									+ "is \"" + shared_obj + "\"."
									+ "\n"
									);
				} catch (Exception e) {
					System.out.print("Taker: "
									+ "semaphore.get_exclusive_object("
									+ this + ") throws Exception " + e
									+ "\n"
									);
					return; // FATAL
				}

				System.out.print("Taker: sleeps... \n");
				try { this.sleep(1100); } catch (Exception e) { ; }

				shared_obj.append("Poller!");
				System.out.print("Taker: waked up! Taker edited (#2) "
								+ "shared_obj is \""
								+ shared_obj + "\""
								+ "\n"
								);

				if (!semaphore.leave(this)) {
					System.out.print("Taker: semaphore.leave( "
									+ this + ") returns false."
									+ " And then, taker waits responce "
									+ "from poller, via the shared "
									+ "object."
									+ "\n"
									);
					return;	// FATAL
				}
			}

			while (!semaphore2.take(this, false)) {
				System.out.print("Taker: Poller had taken semaphore2."
								+ "\n"
								);
				try { this.sleep(170); } catch (Exception e) { ; }
			}
			if (!semaphore2.leave(this)) {
				System.out.print("Taker: semaphore2.leave(" + this
								+ ") returns flase."
								+ "\n"
								);
				return;	// FATAL
			}

			if (!semaphore.take(this, blocking)) {
				System.out.print("Taker blocking = " + blocking
								+ ": Can't take semaphore "
								+ "for waiting responce."
								+ "\n"
								);
			} else {
				StringBuffer	shared_obj;

				try {
					shared_obj	= (StringBuffer)
								semaphore.get_exclusive_object(this);
					System.out.print("Taker: Responce from poller is \""
									+ shared_obj + "\"."
									+ "\n"
									);
				} catch (Exception e) {
					System.out.print("Taker: semaphore."
									+			"get_exclusive_object("
									+ this
									+ ") throws Exception " + e
									+ "(responce)"
									+ "\n"
									);
					return; // FATAL
				}

				if (!semaphore.leave(this)) {
					System.out.print("Taker: semaphore.leave("
									+ this + ") returns false, "
									+ "after responce-wait."
									+ "\n"
									);
				}
			}

			System.out.print("Taker: ... returns (thread exits)\n");
			return;
		}
		else if (taker == BinSemaphoreTest.POLLER) {
			// --shared-object test
			// Tester for polling shared object (poller thread)
			System.out.print("Poller: starts...\n");
			blocking	= false;
			sleep_interval	= 230;

			System.out.print("Poller: semaphore2.take(" + this
							+ ", true)..."
							+ "\n"
							);
			if (!semaphore2.take(this, true)) {
				System.out.print("Poller: semaphore2.take(" + this
								+ ") return false."
								+ "\n"
								);
				return;	// FATAL
			} else {
				while (true) {
					StringBuffer shared_obj;

					while (!semaphore.take(this, blocking)) {
						System.out.print("Poller blocking = " + blocking
										+ ": Taker had taken semaphore."
										+ "\n"
										);
						try { this.sleep(sleep_interval);
						} catch (Exception e) { ; }
					}

					try {
						shared_obj = (StringBuffer)
								semaphore.get_exclusive_object(this);
					} catch (Exception e) {
						System.out.print("Poller: semaphore."
										+ "get_exclusive_object(" + this
										+ ") throws Exception " + e
										+ "."
										+ "\n"
										);
						return;	// FATAL
					}

					if (!(shared_obj.toString().equals(""))) {
						shared_obj.reverse();
						System.out.print("Poller: reversed shared_obj "
										+ "is \"" + shared_obj + "\"."
										+ "\n"
										);

						if (!semaphore.leave(this)) {
							System.out.print("Poller: semaphore."
											+ "leave( " + this
											+ ") returns false."
											+ "\n"
											);
							return;	// FATAL
						}

						break;
					}

					System.out.print("Poller: Taker hasn't taken "
									+ "semaphore."
									+ "\n"
									);
					if (!semaphore.leave(this)) {
						System.out.print("Poller: smaphore.leave("
										+ this + ") returns false."
										+ "\n"
										);
						return;	// FATAL
					}
					try { this.sleep(170); } catch(Exception e) { ; }
				}

				System.out.print("Poller: sleeps...\n");
				try { this.sleep(1300); } catch (Exception e) { ; }

				if (!semaphore2.leave(this)) {
					System.out.print("Poller: semaphore2.leave(" + this
									+ ") return false."
									+ "\n"
									);
					return;	// FATAL
				}
				System.out.print("Poller: semaphore2.leave(" + this
								+ ") is done."
								+ "\n"
								);
			}

			System.out.print("Poller: ... returns (thread exits)\n");
			return;
		}

		// --simple test
		// 2 Threads without/within semaphore
		System.out.print("BinSemaphoreTest.run(): "
				+ ((even_actor)? "even": "odd") + " starts...\n"
				);
		if ((sem_enable == true) && !semaphore.take(this, blocking)) {
			System.out.print("even_actor = " + even_actor
							+ "sem_enable = " + sem_enable
							+ " semaphore.take(" + this
							+ ", " + blocking
							+ ") returns false.\n"
							);
		}
		else {
			if (even_actor) {
				x	= 2;
				sleep_interval	= 110;
				loop_cnt	= 30;
			}
			else {
				x	= 1;
				sleep_interval	= 230;
				loop_cnt	= 15;
			}

			for (int i = 0; i < loop_cnt; ++i) {
				System.out.print("even_actor = " + even_actor
								+ ": x = " + x + "\n"
								);
				try {
					this.sleep(sleep_interval);
				}
				catch (Exception e) {
					System.out.print("even_actor = " + even_actor
									+ "sem_enable = " + sem_enable
									+ " i = " + i
									+ ": run() caught exception "
									+ "from this.sleep()"
									);
				}
				x	+= 2;
			}

			if ((sem_enable == true)
			&& !semaphore.leave(this)) {
				System.out.print("even_actor = " + even_actor
								+ "sem_enable = " + sem_enable
								+ " semaphore.leave(" + this
								+ ") returns false.\n"
								);
			}
		}

		return;
	}
		
	static public void main(String[] args) {
		BinSemaphore	sem = new BinSemaphore();
		boolean	within_sem = false;
		String	within_msg = "without";

		if (args.length == 0 || args[0].equals("-simple")) {
			for (int i = 0; i < 2; ++i) {
				BinSemaphoreTest even = new BinSemaphoreTest(sem
														, true
														, within_sem
														);
				BinSemaphoreTest odd = new BinSemaphoreTest(sem
														, false
														, within_sem
														);

				System.out.print("BinSemaphoreTest.main(): "
								+ "2 Threads " + within_msg
								+ " BinSemaphore\n"
								);
				even.start();
				odd.start();

				try { odd.join(); } catch (Exception e) {
					System.out.print("main() i = " + i
									+ ": Exception in odd.join()\n");
				}
				try { even.join(); } catch (Exception e) {
					System.out.print("main() i = " + i
									+ ": Exception in even.join()\n");
				}

				within_sem	= true;
				within_msg	= "within";
				System.out.print("\n");
			}
		}

		if (args.length == 0 || args[0].equals("--shared-object")) {
			for (int i = 0; i < 2; ++i) {
				StringBuffer	shared_obj = new StringBuffer("");
				BinSemaphore [] sem2 = {
								  new BinSemaphore((Object)shared_obj)
								, new BinSemaphore(null)
								};
				BinSemaphoreTest	taker =
										new BinSemaphoreTest(sem2
											, BinSemaphoreTest.TAKER
											);
				BinSemaphoreTest	poller =
										new BinSemaphoreTest(sem2
											, BinSemaphoreTest.POLLER
											);

				System.out.print("main():#" + i + " Semaphore taker "
								+ "and Polling thread "
								+ "(these shares an object)\n"
								);

				taker.start(); poller.start();
				try { poller.join(); taker.join();
				} catch (Exception e) { ; }

				System.out.print("\n");
			}
		}

		if (args.length == 0 || args[0].equals("--create-taken")) {
			System.out.print("main(): Create-Taken test\n");
			BinSemaphoreTest parent = new BinSemaphoreTest(SLEEP_MSECS
														, null
														);
			parent.start();
			try { parent.join(); } catch (Exception e) { ; }

			System.out.print("\n");
		}

		return;
	}
}
