/***********************************************************************
 * File: FacilityAgentTest.java
 * 	(Extracted from
 *	   "$Id: FacilityAgentTest.java,v 1.4 2004/06/20 10:56:39 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements an agent that recieved the
 * "FacilityCommandTest" request, request to 
 * "FacilityAgentTest_prim_sifter" for making the prime numbers,
 * and responces the numbers to "FacilityCommandTest" request-sender.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import my_util.Log;
import my_util.FacilityAgent;
import my_util.FacilityCommand;

public class FacilityAgentTest extends FacilityAgent {
	static final public int FAILED_RESPONCE = 1;

	public boolean init() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest.init() run.");

		return	true;
	}

	public int process(FacilityCommand req) throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest.process(" + req
														+ ") run.");

		FacilityCommandTest	buf = (FacilityCommandTest)req;
		long[] primes	= buf.getPrimes();
		int	primes_cnt	= 0;
		FacilityCommandTest_prime_sifter	cmd =
								new FacilityCommandTest_prime_sifter();

		primes[primes_cnt++]	= 2;
		for (long i = 3; i < Long.MAX_VALUE; i += 2) {
			cmd.setFac_name(getFacname());
			cmd.setAid(getAid());
			cmd.setResp_to_agent(true);
			cmd.setResp_queue_name(getAgname());
			cmd.setPrimes_max_cnt(primes.length);
			cmd.setNumber(i);
			cmd = (FacilityCommandTest_prime_sifter)
						request("FacilityAgentTest_prime_sifter"
								, (Object)cmd 
								, 0
								);
			if (cmd.getIs_prime()) {
				primes[primes_cnt++]	= cmd.getNumber();
				log(Log.INFO, 0, "Get the " + primes_cnt + "-th prime "
						+ ". The prime number is " + cmd.getNumber()
						+ "."
						);
				if (primes_cnt == primes.length) {
					break;
				}
			}
		}

		boolean	result	= responce(buf, 0);

		log(Log.DEBUG, 0
			, "FacilistyAgentTest.process(" + req + ") responce() "
			+ "returns " + result + "."
			);

		return	(result)? NORMALEND: FAILED_RESPONCE;
	}

	public boolean suspend() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest.suspend() run.");

		return	true;
	}

	public boolean resume() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest.resume() run.");

		return	true;
	}

	public boolean terminate() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest.terminate() run.");

		return	true;
	}
}
