/***********************************************************************
 * File: FacilityAgentTest_prime_sifter.java
 * 	(Extracted from
 *	   "$Id: FacilityAgentTest_prime_sifter.java,v 1.3 2004/06/14 11:43:27 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements an agent that is a prime sifter.
 *  This agent saves and restore the prime numbers (those this agent
 * already sifted) to "FacilityAgentTest_prime_sifter.jobj" java-object
 * file.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import	java.util.Arrays;
import	java.io.*;
import my_util.Log;
import my_util.FacilityAgent;
import my_util.FacilityCommand;

public class FacilityAgentTest_prime_sifter extends FacilityAgent {
	// process() error codes
	static final public int FAILED_RESPONCE = 1;

	static final private String save_file_path =
							"FacilityAgentTest_prime_sifter.jobj";
	static private boolean saved = false;

	private long max_number = 0;
	private int primes_cnt = 0;
	private long[] primes = null;
	static final private int prime_mask
						= (1 <<  1) | (1 <<  7) | (1 << 11) | (1 << 13)
						| (1 << 17) | (1 << 19) | (1 << 23) | (1 << 29);
	static final private int[] PRESET_PRIMES	= { 2, 3, 5 };
	static final private int PRESET_PRIMES_LCM	= (2 * 3 * 5);

	public boolean init() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest_prime_sifter.init() run.");

		try {
			ObjectInputStream	ois = new ObjectInputStream(
											new FileInputStream(
												new File(save_file_path)
											));
			max_number = ois.readLong();
			primes_cnt = ois.readInt();
			if (primes_cnt > 0) {
				primes = (long[])ois.readObject();
			}

			ois.close();
		} catch (Exception e) {
			log(Log.WARN, 999
				,"init(): Occured exception! {" + e.getMessage() + "} "
								+ "Initialized primes[]. "
								+ "This agent continue to work... "
								);
			e.printStackTrace();

			primes_cnt	= PRESET_PRIMES.length;
			primes	= new long[primes_cnt];
			for (int i = 0; i < primes_cnt; ++i) {
				primes[i]	= PRESET_PRIMES[i];
			}
		}

		log(Log.DEBUG, 222
			, "init(): prime_mask = 0x"
				+ Integer.toString(prime_mask, 16)
				+ ", PRESET_PRIMES_LCM = " + PRESET_PRIMES_LCM);
		log(Log.DEBUG, 111
			, "init(): max_number = " + max_number + ", primes_cnt = "
				+ primes_cnt + ", primes[0] = " + primes[0]
				+ ((primes_cnt > 1)?
						(",primes[1] = " + primes[1]
								+ ", primes[2] = " + primes[2]):
						""
					)
			);

		return	true;
	}

	private void expand_primes_array(int length) {
		if (length > primes_cnt) {
			long[] prs = new long[length];
			System.arraycopy(primes, 0, prs, 0, primes_cnt);
			for (int i = primes_cnt; i < length; ++i) {
				prs[i]	= Long.MAX_VALUE;
			}
			primes	= prs;
		}
	}

	public int process(FacilityCommand req) throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest_prime_sifter.process("
						+ req + ") run.");

		FacilityCommandTest_prime_sifter	buf =
								(FacilityCommandTest_prime_sifter)req;

		if (primes.length < buf.getPrimes_max_cnt()) {
			expand_primes_array(buf.getPrimes_max_cnt());
		}

		long	number = buf.getNumber();
		if (number == 2 || number == 3 || number == 5) {
			buf.setIs_prime(true);
		}
		else if ((prime_mask & (1 << (number % PRESET_PRIMES_LCM)))
																== 0) {
	
			buf.setIs_prime(false);
		}
		else if (max_number >= number) {
			buf.setIs_prime((Arrays.binarySearch(primes, number) >= 0));
		}
		else {
			buf.setIs_prime(true);
			for (int i = PRESET_PRIMES.length; i < primes_cnt; ++i) {
	
				if (number % primes[i] == 0) {
					buf.setIs_prime(false);
					break;
				}
			}
			if (buf.getIs_prime()) {
				if (primes.length == primes_cnt) {
					expand_primes_array(primes_cnt + 1);
				}
				primes[primes_cnt++]	= number;
			}
			max_number	= number;
		}

		if (buf.getIs_prime()) {
			log(Log.INFO, 0, "Find the " + primes_cnt
							+ "-th prime number " + buf.getNumber()
							+ "."
							);
		}

		boolean	result	= responce(buf, 0);

		log(Log.DEBUG, 0
			, "FacilistyAgentTest_prime_sifter.process(" + req
			+ ") responce() " + "returns " + result + "."
			);

		return	(result)? NORMALEND: FAILED_RESPONCE;
	}

	public boolean suspend() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest_prime_sifter.suspend() "
							+ "run."
							);

		return	true;
	}

	public boolean resume() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest_prime_sifter.resume() "
							+ "run."
							);

		return	true;
	}

	public boolean terminate() throws Exception {
		log(Log.DEBUG, 0, "FacilityAgentTest_prime_sifter.terminate() "
							+ "run."
							);
		log(Log.DEBUG,111
			, "terminate(): max_number = " + max_number
				+ ", primes_cnt = " + primes_cnt
				+ ", primes[0] = " + primes[0]
				+ ((primes_cnt > 1)?
						(",primes[1] = " + primes[1]
								+ ", primes[2] = " + primes[2]):
						""
					)
			);

		try {
			ObjectOutputStream	oos = new ObjectOutputStream(
											new FileOutputStream(
												new File(save_file_path)
											));
			oos.writeLong(max_number);
			oos.writeInt(primes_cnt);
			if (primes_cnt > 0) {
				oos.writeObject(primes);;
			}

			oos.close();
		} catch (Exception e) {
			log(Log.WARN, 999
				, "terminate(): Occured exception! {" + e.getMessage()
					+ "} Leave \"" + save_file_path + "\" "
					+ "java-object file.\n"
				);
			e.printStackTrace();
		}

		return	true;
	}
}
