/***********************************************************************
 * File: FacilityTest.java
 * 	(Extracted from
 *	   "$Id: FacilityTest.java,v 1.6 2004/07/04 12:39:00 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a program that tests Facility,
 * FacilityAgent, FacilityAgentControlBlock and FacilityCommand classes
 * in my_util package.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import my_util.Message;
import my_util.Facility;
import my_util.FacilityCommand;
import my_util.LogConf;
import my_util.Log;
import my_util.FailedProcessingException;

public class FacilityTest {

	static private FacilityCommand request_to_facility(Facility fac
												, FacilityCommand cmdbuf
												, String app_name
												, String resp_queue_name
												, Message resq
												) throws Exception {
		// set the command-header for Application
		cmdbuf.setFac_name(app_name);
		cmdbuf.setAid(0);
		cmdbuf.setResp_to_agent(false);
		cmdbuf.setResp_queue_name(app_name);

		// try to send the request command to the destiny facility...
		if (fac.put_request(cmdbuf, 0, false) == false) {
			// Failed to send the request to this facility
			throw new Exception("FacilityTest.request_to_facility("
					+ cmdbuf + ", " + app_name + ", "
					+ resp_queue_name + "): "
					+ "fac.put_request(" + cmdbuf + ", "
					+ "0, false) returns false."
					);
		}

		// try to get the responce of the request command
		// check the request command processing phase
		int	req_status	= cmdbuf.getStatus();
		if (req_status >= cmdbuf.OCCURED_EXCEPTION) {
			// The reuest processing is failed!
			throw new FailedProcessingException(
					"FacilityTest.request_to_facility(" + cmdbuf + ", "
					+ app_name + ", " + resp_queue_name + "): "
					+ cmdbuf.getProcessing_agent()
					+ "\" agent failed processing ["
					+ "status = " + req_status
					+ ", " + "halt_cause = "
					+ cmdbuf.getHalt_cause() + ", "
					+ "exception_class_name = "
					+ cmdbuf.getException_class_name()
					+ "]"
					);
		}

		// try to send the responce toward the invoker
		return (FacilityCommand)(resq.get(true, true, 0));
	}

	static public void main(String[] args) {
		Facility fac = null, fac_prime_sifter = null;
		int	max_concurrency	= 1, concurrency;
		
		try {
			System.out.println(
				"FailityTest.main(): Start to Facility.create()!!!!");
			String app_name = "FacilityTest Application"
					, fac_name	= "FacilityAgentTest";
			concurrency	= 0;
			LogConf log_conf = new LogConf(null
											, Log.STDOUT
											, null
											, Log.DEFAULT_DELIMIT
											, Log.DEBUG
											);
			fac = Facility.create(fac_name
									, fac_name
									, max_concurrency
									, concurrency
									, log_conf
									, 1
									);
			System.out.println("FailityTest.main(): created " + fac_name
								+ " facility."
								);

			fac_prime_sifter =
					Facility.create("FacilityAgentTest_prime_sifter"
									, "FacilityAgentTest_prime_sifter"
									, 1
									, 1
									, log_conf
									, 1
									);
			System.out.println("FailityTest.main(): created "
							+ "FacilityAgentTest_prime_sifter facility."
							);
			System.out.println(
				"FailityTest.main(): Finished Facility.create()!!!!");

			System.out.println(
				"FailityTest.main(): Start to fac.activate()!!!!");
			if (fac.activate()) {
				System.out.println("FacilityTest.main(): "
								+ "fac.activate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
								+ "fac.activate() returns false.");
			}
			System.out.println(
				"FailityTest.main(): Finished fac.activate()!!!!");

			System.out.println("FailityTest.main(): "
				+ "Start to fac_prime_sifter.activate()!!!!");
			if (fac_prime_sifter.activate()) {
				System.out.println("FacilityTest.main(): "
						+ "fac_prime_sifter.activate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
						+ "fac_prime_sifter.activate() returns false.");
			}
			System.out.println("FailityTest.main(): "
				+ "Finished fac_prime_sifter.activate()!!!!");

			System.out.println("FailityTest.main(): "
				+ "Start to fac.change_concurrency()!!!!");
			for (concurrency = 0
			; concurrency <= max_concurrency
			; ++concurrency) {
				if (fac.change_concurrency(concurrency)) {
					System.out.println("FacilityTest.main(): "
								+ "fac.change_concurrency("
								+ concurrency +") returns true"
								+ "(max_concurrency = "
								+ max_concurrency
								+ ", concurrency = "
								+ concurrency + ")."
								);
				}
				else {
					System.out.println("FacilityTest.main(): "
								+ "fac.change_concurrency("
								+ concurrency +") returns false"
								+ "(max_concurrency = "
								+ max_concurrency + "."
								);
				}
			}
			System.out.println("FailityTest.main(): "
				+ "Finished fac.change_concurrency()!!!!");

			System.out.println(
				"FailityTest.main(): "
				+ "Start to request_to_facility()!!!"
				);
			Message resq = Message.create(Thread.currentThread()
										, app_name
										, 1
										);
			if (resq == null) {
				System.out.println("FailityTest.main(): "
					+ "Message.create() returns null!!!!");
			}
			else {
				FacilityCommandTest	cmdbuf = new FacilityCommandTest();
				long[] primes = new long[50];

				for (int j = 0; j < 2; ++j) {
					// make the request command
					cmdbuf.setPrimes(primes);

					// try to reuest the command to the destiny facility
					long t0 = System.currentTimeMillis();
					cmdbuf = (FacilityCommandTest)(request_to_facility(
											  fac
											, (FacilityCommand)cmdbuf
											, app_name
											, app_name
											, resq
											));
					long t1 = System.currentTimeMillis();

					System.out.println(
						"FailityTest.main(): primes[] are...");
					primes = cmdbuf.getPrimes();
					for (int i = 0; i < primes.length; ++i) {
						System.out.println("\tprimes[" + i + "] = "
										+ primes[i]
										);
					}
					System.out.println("FacilityTest.main(): "
									+ "t0 = " + t0 + ", t1 = " + t1
									+ ", t1 - t0 = " + (t1 - t0)
									+ ", primes.length = "
									+ primes.length
									);

					System.out.println("FacilityTest.main(): "
											+ "Start to change log!!!");
					if (fac.change_log(new LogConf(null
											, Log.STDOUT
											, null
											, Log.DEFAULT_DELIMIT
											, ((j == 0)?
													Log.INFO: Log.DEBUG)
											))) {
						System.out.println("FacilityTest.main(): "
								+ "fac.change_log() returns true.");
					}
					else {
						System.out.println("FacilityTest.main(): "
								+ "fac.change_log() returns false.");
					}

					if (fac_prime_sifter.change_log(
								new LogConf(null
									, Log.STDOUT
									, null
									, Log.DEFAULT_DELIMIT
									, ((j == 0)? Log.INFO: Log.DEBUG)
									))) {
						System.out.println("FacilityTest.main(): "
								+ "fac_prime_sifter.change_log() "
								+ "returns true.");
					}
					else {
						System.out.println("FacilityTest.main(): "
								+ "fac_prime_sifter.change_log() "
								+ "returns false.");
					}
					System.out.println("FacilityTest.main(): "
											+ "Finished change log!!!");

					if (j == 0) {
						primes	= new long[10];
						System.out.println("FacilityTest.main(): "
											+ "Invoke "
											+ "request_to_facility()"
											+ ", second time starts... "
											+ "primes.length = "
											+ primes.length
											);
					}
				}
			}
			System.out.println(
				"FailityTest.main(): "
				+ "Finished request_to_facility()!!!"
				);

			System.out.println("FailityTest.main(): "
								+ "Start to fac.deactivate()!!!!");
			if (fac.deactivate()) {
				System.out.println("FacilityTest.main(): "
								+ "fac.deactivate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
								+ "fac.deactivate() returns false.");
			}
			System.out.println(
				"FailityTest.main(): Finished fac.deactivate()!!!!");

			System.out.println("FailityTest.main(): "
				+ "Start to fac_prime_sifter.deactivate()!!!!");
			if (fac_prime_sifter.deactivate()) {
				System.out.println("FacilityTest.main(): "
					+ "fac_prime_sifter.deactivate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
					+ "fac_prime_sifter.deactivate() returns false.");
			}
			System.out.println("FailityTest.main(): "
				+ "Finished fac_prime_sifter.deactivate()!!!!");

			System.out.println(
				"FailityTest.main(): Start to fac.activate(), "
														+ "SECOND!!!!");
			if (fac.activate()) {
				System.out.println("FacilityTest.main(): "
								+ "fac.activate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
								+ "fac.activate() returns false.");
			}
			System.out.println(
				"FailityTest.main(): Finished fac.activate(), "
														+ "SECOND!!!!");

			System.out.println("FailityTest.main(): "
				+ "Start to fac_prime_sifter.activate(), SECOND!!!!");
			if (fac_prime_sifter.activate()) {
				System.out.println("FacilityTest.main(): "
						+ "fac_prime_sifter.activate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
						+ "fac_prime_sifter.activate() returns false.");
			}
			System.out.println("FailityTest.main(): "
				+ "Finished fac_prime_sifter.activate(), SECOND!!!!");

			System.out.println("FacilityTest.main(): Start to get "
												+ "1000 primes!!!");
			FacilityCommandTest	cmdbuf = new FacilityCommandTest();
			long[] primes = new long[1000];

			cmdbuf.setPrimes(primes);
			long t0 = System.currentTimeMillis();
			cmdbuf	= (FacilityCommandTest)(request_to_facility(
											  fac
											, (FacilityCommand)cmdbuf
											, app_name
											, app_name
											, resq
											));
			long t1 = System.currentTimeMillis();
			System.out.println(
				"FailityTest.main(): primes[] are...");
			primes = cmdbuf.getPrimes();
			for (int i = 0; i < primes.length; ++i) {
				System.out.println("\tprimes[" + i + "] = "
								+ primes[i]
								);
			}
			System.out.println("FacilityTest.main(): "
							+ "t0 = " + t0 + ", t1 = " + t1
							+ ", t1 - t0 = " + (t1 - t0)
							+ ", primes.length = "
							+ primes.length
							);
			System.out.println("FacilityTest.main(): Finished to get "
												+ "1000 primes!!!");

			System.out.println("FailityTest.main(): "
							+ "Start to fac.deactivate(), SECOND!!!!");
			if (fac.deactivate()) {
				System.out.println("FacilityTest.main(): "
								+ "fac.deactivate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
								+ "fac.deactivate() returns false.");
			}
			System.out.println(
			"FailityTest.main(): Finished fac.deactivate(), SECOND!!!");

			System.out.println("FailityTest.main(): "
				+ "Start to fac_prime_sifter.deactivate(), SECOND!!!!");
			if (fac_prime_sifter.deactivate()) {
				System.out.println("FacilityTest.main(): "
					+ "fac_prime_sifter.deactivate() returns true.");
			}
			else {
				System.out.println("FacilityTest.main(): "
					+ "fac_prime_sifter.deactivate() returns false.");
			}
			System.out.println("FailityTest.main(): "
				+ "Finished fac_prime_sifter.deactivate(), SECOND!!!!");

		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
}
