/***********************************************************************
 * File: G.java
 * 	(Extracted from
 *	   "$Id: G.java,v 1.5 2004/06/26 17:27:12 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements an example for using java.nio
 * socket TCP/IP networking.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import java.io.*;
import java.nio.*;
import java.nio.charset.*;
import java.nio.channels.*;
import java.nio.channels.spi.*;
import java.net.*;
import java.util.*;

public class G {
	static public void main(String[] args) {
		try {
			Selector sel = SelectorProvider.provider().openSelector();
			ServerSocketChannel ssc = ServerSocketChannel.open();
			ssc.configureBlocking(false);
			InetAddress lh = InetAddress.getLocalHost();
			InetSocketAddress isa = new InetSocketAddress(lh, 8900);
			ssc.socket().bind(isa);
			SelectionKey accept_sk =
								ssc.register(sel
											, SelectionKey.OP_ACCEPT
											);
			int n = 0;
			while ((n = sel.select()) >= 0) {
				System.out.println("n = " + n);
				if (n == 0) {
					continue;
				}
				Set readies = sel.selectedKeys();
				Iterator ite = readies.iterator();
				while (ite.hasNext()) {
					SelectionKey sk = (SelectionKey)(ite.next());
					ite.remove();

					System.out.println("accept_sk == sk is \""
									+ (accept_sk == sk) + "\""
									);

					if (sk.isAcceptable()) {
						ServerSocketChannel ach =
								(ServerSocketChannel)(sk.channel());
						Socket so = ach.accept().socket();
						SocketChannel sc = so.getChannel();
						sc.configureBlocking(false);
						sc.register(sel, SelectionKey.OP_READ);

						System.out.println("registered the new conn.");
						continue;
					}

					SocketChannel sc = (SocketChannel)(sk.channel());

					reached_EOF = false;
					boolean is_HTTP_request =
											print_inputs_from_conn(sc);
					if (!reached_EOF) {
						if (is_HTTP_request) {
							responce_HTTP(sc);
						}
						else {
							responce_non_HTTP(sc);
						}
					}

					if (reached_EOF) {
						sk.cancel();
						sc.socket().close();
						sc.close();
						System.out.println("Closed connection.");
					}
				}
			}

			System.out.println("n = " + n);
			ssc.socket().close();
			ssc.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static boolean reached_EOF = false;
	private static CharsetDecoder http_cd =
							Charset.forName("UTF-8").newDecoder();
	private static int hex2int(byte b) {
		int d = (new Byte(b)).intValue();
		return	('0' <= d && d <= '9')?	d - '0':
				('a' <= d && d <= 'f')?	d - 'a' + 10:
				('A' <= d && d <= 'F')?	d - 'A' + 10:
				/** Not hexa-dicimal code **/	-1;
	}
	private static void decode_URI(ByteBuffer bb
									, boolean replace_LF_CR) {
		int limit = bb.position();
		byte[]	ba = bb.array();
		int	p = 0;
		for (int i = 0; i < limit;) {
			if (i + 1 < ba.length
			&& ba[i] == '\r'
			&& ba[i + 1] == '\n') {
				ba[p]	= ba[i + 1];
				++p; i += 2;
			}
			else if (ba[i] == '+') {
				ba[p]	= ' ';
				++p; ++i;
			}
			else if (i + 2 < ba.length && ba[i] == '%') {
				ba[p]	= (byte)((hex2int(ba[i + 1]) << 4)
										| hex2int(ba[i + 2]));
				++p; i += 3;
			}
			else {
				ba[p++]	= ba[i++];
			}
		}
		bb.clear();
		bb.put(ba, 0, p);
	}
	private static boolean print_inputs_from_conn(SocketChannel sc)
													throws Exception {
		boolean result = false, checked_1st_line = false;
		ByteBuffer	bb = ByteBuffer.allocate(4096);
		reached_EOF	= false;
		while (true) {
			int	n = -1;
			try { n = sc.read(bb); } catch (IOException ie) { ; }
			if (n < 0) { reached_EOF = true; break; }

			if (!checked_1st_line && bb.position() < 4) {
				continue;
			}
			for (int i = 0; i <= bb.position() - 4;) {
				int	c0 = (new Byte(bb.get(i))).intValue()
					, c1 = (new Byte(bb.get(i + 1))).intValue()
					, c2 = (new Byte(bb.get(i + 2))).intValue()
					, c3 = (new Byte(bb.get(i + 3))).intValue();
				if (!checked_1st_line
				&& ((c0 == 'G' && c1 == 'E' && c2 == 'T'
										&& (c3 == ' ' || c3 == '\t'))
					|| (c0 == 'P' && c1 == 'O' && c2 == 'S'
														&& c3 == 'T')
					)) {
					result	= true; // this request in HTTP
				}
				checked_1st_line = true;

				// seaech the end of reauests...
				if (c0 != '\r'
				|| c1 != '\n') {
					++i; continue;
				}
				if (result && (c2 != '\r' || c3 != '\n')) {
					i += 3; continue;
				}

				if (result) { // this is the HTTP reuest
					decode_URI(bb, true);
					bb.flip();
					CharBuffer cb = null;
					try {
						cb = http_cd.decode(bb);
					} catch (CharacterCodingException cce) {
						reached_EOF = true;
						return result;
					}
					System.out.println("{" + cb.toString() + "}");
				}
				else { // this is the non-HTTP reuest
					String req = new String(bb.array()
											, 0
											, bb.position()
											);
					System.out.print("Recieved non-HTTP: {" + req
																+ "}");
				}

				return	result;
			}
			if (bb.remaining() == 0) {
				ByteBuffer bbb =
							ByteBuffer.allocate(bb.capacity() + 4096);
				bbb.put(bb.array());
				bb	= bbb;
			}
		}
		reached_EOF = true;
		return	result;
	}

	private static CharsetEncoder ce =
							Charset.forName("UTF-8").newEncoder();
	private static String encoding			= "UTF8";
	private static String charset_attr		= "charset=utf-8";
	private static void responce_HTTP(SocketChannel sc)
										throws Exception {
		// HTTP ̉͏ UTF-8 ZbgŃGR[hĂ
		String html = "<HTML><HEAD><!META HTTP-EQUIV=\"Content-Type\" "
					+ "CONTENT=\"text/html; " + charset_attr + "\">"
					+ "<TITLE>Responce</TITLE></HEAD>\r\n"
					+ "<BODY><H1>Responce</H1>\r\n"
					+ "Responce from Inspector 1.0<br>\r\n"
					+ "<em>CXyN^ 1.0</em>̉<br>\r\n"
					+ new Date().toString()
					+ "<FORM name=f "
						+ "method=\"POST\" action=\"/form_action\">\r\n"
					+ "<INPUT maxLength=256 size=55 name=q value=\"\">"
						+ "<INPUT type=hidden name=sid value=Inspector>"
						+ "<INPUT type=submit name=b "
									+ "value=\"Inspector \">\r\n"
					+ "</FORM>\r\n"
					+ "</BODY></HTML>\r\n";
		CharBuffer cb = CharBuffer.allocate(html.length());
		ByteBuffer bb = ce.encode((CharBuffer)(cb.put(html).flip()));
		System.out.println("html.length() = " + html.length());
		System.out.println("bb.limit() = " + bb.limit());
		String head =
				"HTTP/1.1 200 OK\r\n"
				+ "Server: Inspector 1.0\r\n"
				+ "Content-length: " + bb.limit() + "\r\n"
				+ "Content-type: text/html; " + charset_attr + "\r\n"
				+ "Connection: close\r\n"
				+ "\r\n";
		cb	= CharBuffer.allocate(head.length());
		ByteBuffer bb_head =
						ce.encode((CharBuffer)(cb.put(head).flip()));
		try { sc.write(bb_head); sc.write(bb);
		} catch (IOException ie) { ; }
		reached_EOF = true;
	}

	private static void responce_non_HTTP(SocketChannel sc)
										throws Exception {
		// TeraTerm ȂꍇA[Setup]>>[Terminal] 
		// [Transmit]  CR+LF ɂA[Local echo]  ON ɂ邱
		byte[] resp = "CXyN^ 1.0 ̉\r\n".getBytes();
		ByteBuffer bb = ByteBuffer.allocate(resp.length);
		try { sc.write((ByteBuffer)(bb.put(resp).flip()));
		} catch (IOException ie) { reached_EOF = true; }
	}
}
