/***********************************************************************
 * File: J.java
 * 	(Extracted from
 *	   "$Id: J.java,v 1.1 2004/07/19 11:26:36 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements an example code for non-blocking
 * connecting, send and recieve via java.nio.SockectChannel.
 *  This is a client program for the TimerTest.class program.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import java.util.*;
import java.io.*;
import java.net.*;
import java.nio.*;
import java.nio.channels.*;

import my_util.Utillity;
import my_util.TimeoutException;

public class J implements Runnable {
	public J() throws Exception {}

	private int wait_channel_event(Selector sel, long timeout)
													throws Exception {
		long	start = System.currentTimeMillis(), to = timeout;
		int	result = -1;

		while (true) {
			if ((timeout > 0)
			&& (to = Utillity.check_timeout(start, timeout)) < 1) {
				System.out.println("***Already, time-outed (sel = "
									+ sel + ", timeout = " + timeout
									+ ", start = " + start + ")."
									);
				throw new Exception("Already, time-outed");
			}

			if ((timeout > 0) && to > 100) {
				to	= 100;
			}

			if (((timeout == 0) || to < 1)
			&& (((result = sel.select(to)) < 1)
					&& timeout > 0) && to < 100) {
				throw new TimeoutException("wait_channel_event(" + sel
												+ ", " + timeout + ")"
											, timeout
											);
			}

			if (result > 0) {
				break; /* occures events */
			}
		}

		return	result; /* returns numbers of occured events */
	}

	public SocketChannel connect(InetAddress addr
								, int port
								, long timeout
								) throws Exception {
		SocketChannel result = null;
		Selector sel = null;

		try {
			sel = Selector.open();
			result = SocketChannel.open();
			result.configureBlocking(false);
			result.connect(new InetSocketAddress(addr, port));
			SelectionKey consk = result.register(sel
											, SelectionKey.OP_CONNECT
											, null
											);
			int n = wait_channel_event(sel, timeout);
			if (n >= 1) {
				for (Iterator ite = sel.selectedKeys().iterator()
				; ite.hasNext()
				; ) {
					SelectionKey sk = (SelectionKey)ite.next();
					ite.remove();
					SocketChannel sc = (SocketChannel)sk.channel();
					if (sc == result) {
						if (result.finishConnect()) {
							consk.cancel();
						}
						else {
							System.out.println("***Failed in "
								+ "result.finishConnect() that returns "
								+ "false."
								);
							throw new Exception(
									"Failed in result.finishConnec()");
						}
					}
					else {
						System.out.println("***FATAL!! Not-registered "
							+ "SelectionKey is selected via sel. "
							+ "(result = " + result + ", sc = " + sc
							+ ", sk = " + sk + ", sel = " + sel + ")."
							);
						sk.cancel();
						sc.close();
					}
				}
			}
			else {
				System.out.println("***Timeout to connect. (sel = "
					+ sel + ", result = " + result
					+ ", timeout = " + timeout + ", addr = " + addr
					+ ", port = " + port + ")."
					);

				consk.cancel();
				result.close();
				result	= null;
			}
		} catch (Exception e) {
			if (result != null) {
				result.close();
				result	= null;
			}
			throw e;
		} finally {
			if (sel != null) {
				sel.close();
			}
		}

		return	result;	/* returns the SocketChannel establiched */
	}

	public int send(SocketChannel sc, ByteBuffer bb, long timeout)
													throws Exception {
		int	result = -1, n;
		Selector sel = null;

		try {
			sel = Selector.open();
			sc.register(sel, SelectionKey.OP_WRITE, null);
			if ((n = wait_channel_event(sel, timeout)) > 0) {
				for (Iterator ite = sel.selectedKeys().iterator()
				; ite.hasNext()
				; ) {
					SelectionKey sk = (SelectionKey)ite.next();
					ite.remove();
					SocketChannel sc_writable =
											(SocketChannel)sk.channel();
					if (sc_writable != sc) {
						System.out.println("***FATAL!! Not-registered "
							+ "SelectionKey is selected via sel. "
							+ "(sc_writable = " + sc_writable
							+ ", sc = " + sc + ", sk = " + sk
							+ ", sel = " + sel + ")."
							);
						sk.cancel();
						sc.close();
					}
					else {
						int sz = bb.remaining();
						result	= 0;
						while ((result += sc_writable.write(bb)) < sz) ;
					}
				}
			}
			else {
				System.out.println("***Time-outed (sel = " + sel
									+ ", timeout = " + timeout + ")."
									);
				throw new TimeoutException("Time-outed in send()"
											, timeout
											);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			if (sel != null) {
				sel.close();
			}
		}

		return	result; /* returns sent bytes from the ByteBuffer */
	}

	public int recieve(SocketChannel sc, ByteBuffer bb, long timeout)
													throws Exception {
		int result = -1, n;
		Selector sel = null;

		try {
			sel = Selector.open();
			sc.register(sel, SelectionKey.OP_READ, null);
			if ((n = wait_channel_event(sel, timeout)) > 0) {
				for (Iterator ite = sel.selectedKeys().iterator()
				; ite.hasNext()
				; ) {
					SelectionKey sk = (SelectionKey)ite.next();
					ite.remove();
					SocketChannel sc_readable =
											(SocketChannel)sk.channel();
					if (sc_readable != sc) {
						System.out.println("***FATAL!! Not-registered "
							+ "SelectionKey is selected via sel. "
							+ "(sc_readable = " + sc_readable
							+ ", sc = " + sc + ", sk = " + sk
							+ ", sel = " + sel + ")."
							);
						sk.cancel();
						sc.close();
					}
					else {
						result	= sc_readable.read(bb);
					}
				}
			}
			else {
				System.out.println("***Time-outed (sel = " + sel
									+ ", timeout = " + timeout + ")."
									);
				throw new TimeoutException("Time-outed in recieve()"
											, timeout
											);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			if (sel != null) {
				sel.close();
			}
		}

		return	result; /* returns read bytes to ByteBuffer */
	}

	public void run() {
		SocketChannel	sc = null;
		try {
			sc = connect(InetAddress.getLocalHost()
										, 8900
										, 0 /*??? 10000 */
										);

			if (sc != null) {
				for (int i = 0; i < 30; ++i) {
					byte[] buf = "Full\r\n\r\n".getBytes();
					ByteBuffer	bb = ByteBuffer.allocate(buf.length);

					int sz_write = -1, sz_read = -1;
					if ((sz_write = send(sc
									, (ByteBuffer)bb.put(buf).flip()
									, 0 /*??? 10000 */
									)) < 1) {
						/* connection is closed. */
						System.out.println("*** the connection is "
								+ "closed from remote (send() returns "
								+ sz_write + ")."
								);
						break;
					}

					bb = ByteBuffer.allocate(100 * 1024);
					bb.clear();
					if ((sz_read = recieve(sc
											, bb
											, 0 /*??? 10000 */
											)) < 0) {
						/* reached EOF*/
						System.out.println("*** the coonection is "
									+ "closed from remote "
									+ "(recieve() returns "
									+ sz_read + ")."
									);
						break;
					}

					bb.flip();
					buf = new byte[bb.remaining()];
					bb.get(buf);
					System.out.println("{" + new String(buf) + "}\n");

					try { Thread.sleep(1000); } catch (Exception e) {;}
				}
			}
		} catch (Exception e) {
			System.out.println("*** Occured Exception: "
											+ e.getMessage());
			e.printStackTrace();
		} finally {
			if (sc != null) {
				System.out.println("*** close this connection.");
				try { sc.close(); } catch (Exception e) {
					System.out.println("*** Occured Exception: "
												+ e.getMessage());
					e.printStackTrace();
				}
			}
		}

		return;
	}

	public static void main(String[] args) {
		try {
			System.out.println("*** J starts...");
			J j = new J();
			Thread thr = new Thread(j);
			thr.start();
			thr.join();
			System.out.println("*** J finished!");
		} catch (Exception e) {
			System.out.print("Occures Exception in main(): "
													+ e.getMessage());
			e.printStackTrace();
		}

		return;
	}
}
