/***********************************************************************
 * File: MessageTest.java
 * 	(Extracted from
 *	   "$Id: MessageTest.java,v 1.4 2004/05/24 12:34:09 maoyam Exp maoyam $")
 *
 * Abstracts:
 *   This Java source code implements the test program of
 * the message queue utility class `my_util.Message'.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import my_util.Message;
import	my_util.Semaphore;

public class MessageTest extends Thread {
	boolean		is_producer	= false;
	String		queue_key = null;
	public Message		queue = null;
	Semaphore	sem = null;
	int			serial	= -1;

	public MessageTest(Semaphore sem
						, boolean is_producer
						, String key
						, int serial
						, int consumer_cnt
						) {
		this.is_producer	= is_producer;
		this.sem	= sem;
		this.serial	= serial;
		queue_key	= key;
		if (is_producer) {
			queue	= Message.attach(key);
			System.out.print("MessageTest(" + sem + ", " + is_producer
							+ ", \"" + key + "\"): Message.attach("
							+ key + ") returns "
							+ queue
							+ "\n"
							);
		}
		else {
			queue	= Message.create(this, key, consumer_cnt);
			System.out.print("MessageTest(" + sem + ", " + is_producer
							+ ", \"" + key + "\"): Message.create("
							+ this + ", " + key + ", 10) returns "
							+ queue
							+ "\n"
							);
		}
	}

	public void run() {
		int	i;
		boolean	result = false;
		String	packet;

		if (is_producer) {
			try {
				result	= sem.take(this, true, true, 0);
			} catch (Exception e) {
				System.out.print("producers[" + serial + "] (" + this
								+ "): sem.take(" + this
								+ ", true, true, 1000) returns "
								+ result + "...FATAL"
								+ "\n"
								);
				return;	// FATAL
			} {
				for (i = 0; i < 100; ++i) {
					packet	= "Packet " + i + " from producers["
									+ serial + "] (" + this + ")";
					try {
						result	= queue.put((Object)packet
											, (serial % 2 == 0)
											, true, 0
											);
						System.out.print("producers[" + serial
										+ "] (" + this + "): Send \""
										+ packet + "\""
										+ ((result == false)?
													"...FATAL"
													: ""
													)
										+ "\n"
										);
					} catch (Exception e) {
						System.out.print("producers[" + serial + "] ("
										+ this + "): ");
						print_exception(e);
						return;	// FATAL
					}
				}
				if (!sem.leave(this)) {
					System.out.print("producers[" + serial + "] ("
									+ this + "): sem.leave(" + this
									+ ") returns false"
									+ "\n"
									);
					return; // FATAL
				}
			}
			System.out.print("producers[" + serial + "] (" + this
							+ "): exits."
							+ "\n"
							);
		}
		else {
			while (true) {
				try {
					packet	= (String)(queue.get((serial % 2 == 0)
												, true
												, 0
												));
					System.out.print("consumers[" + serial + "] ("
									+ this + "): recieved \"" + packet
									+ "\""
									+ "\n"
									);
					if (packet == null) {
						break; // `queue' is destroyed
					}
				} catch (Exception e) {
					System.out.print("consumers[" + serial + "] ("
									+ this + "): ");
					print_exception(e);
					break;
				}
			}

			result	= Message.destroy(this, queue_key);
			System.out.print("consumers[" + serial + "] (" + this
							+ "): Message.destroy(" + queue_key
							+ ") returns " + result
							+ "\n"
							);

			System.out.print("consumer[" + serial + "] (" + this + "): "
							+ "exits."
							+ "\n"
							);
		}
		return; // NORMAL END
	}

	static private void print_exception(Exception e) {
		System.out.print("Exception \"" + e + "\" is caught.\n"
						+ "Exception message is " + e.getMessage()
						+ "\n");
		e.printStackTrace();
		return;
	}

	public static void main(String[] args) {
		String	key	= null;
		Thread main_thread = Thread.currentThread();

		if (args.length == 0) {
			// Test1: create(), attach(), destroy()
			System.out.print("*** Test1 Starts...\n");
			key	= "Test1";
			boolean	result;

			Message	msg = Message.create(main_thread, key, 1);
			System.out.print("Message.create(" + main_thread
							+ ", " + "\"" + key + "\"" + ", " + 1 + ") "
							+ "returns " + msg
							+ "\n"
							);
			if (msg == null) {
				System.out.print("*** FATAL\n");
				return;	// FATAL
			}

			Message msg1 = Message.attach(key);
			System.out.print("Message.attach(" + key + ") "
							+ "(#1) returns " + msg1
							+ "\n"
							);
			if (msg1 == null) {
				System.out.print("*** FATAL\n");
				return;	// FATAL
			}

			String packet = "This test is XbhԒʐMJjY.";
			String packet1;
			boolean	exception_is_FATAL = false;
			try {
				packet1	= (String)(msg1.get(true, false, 0));
				System.out.print("msg1.get(true, false, 0) returns "
								+ packet1
								+ "\n"
								);
				packet1	= (String)(msg1.get(true, true, 100));
				System.out.print("msg1.get(true, true, 100) returns "
								+ packet1
								+ "\n"
								);
				System.out.print("*** FATAL\n");
				return;	// FATAL
			} catch (Exception e) {
				print_exception(e);
			}

			try {
				exception_is_FATAL	= true;
				result	= msg.put((Object)packet, false, true, 100);
				System.out.print("msg.put(\"" + packet
								+ "\", false, true, 100) returns "
								+ result
								+ "\n"
								);
				try {
					result	= msg.put((Object)packet, true, true, 100);
					System.out.print("msg.put(\"" + packet
									+ "\", true, true, 100) returns "
									+ result
									+ "\n"
									);
				} catch (Exception e) {
					print_exception(e);
				}
				
				packet1	= (String)(msg1.get(true, true, 0));
				System.out.print("msg1.get(true, true, 0) "
								+ "retuns \"" + packet1 + "\""
								+ "\n"
								);
				exception_is_FATAL	= false;
				packet1	= (String)(msg1.get(true, true, 100));
				System.out.print("msg1.get(true, true, 100) "
								+ "retuns \"" + packet1 + "\""
								+ "\n"
								);
				System.out.print("*** FATAL\n");
			} catch (Exception e) {
				print_exception(e);
				if (exception_is_FATAL) {
					System.out.print("*** FATAL\n");
					return;	// FATAL
				}
			}

			result	= Message.destroy(main_thread, key);
			System.out.print("Message.destroy(" + main_thread
							+ ", \"" + key + "\") returns " + result
							+ "\n"
							);
			if (result == false) {
				System.out.print("*** FATAL\n");
				return;	// FATAL
			}

			Message msg2 = Message.attach(key);
			System.out.print("Message.attach(" + key + ") "
							+ "(#2) returns " + msg2
							+ "\n"
							);
			if (msg2 != null) {
				System.out.print("*** FATAL\n");
				return;	// FATAL
			}

			System.out.print("*** Test 1 finished.\n\n");

			// Test 2: ring-buffer
			System.out.print("*** Test 2 Starts...\n");
			key	= "Test2";
			int	length	= 4;
			boolean	head;

			for (int i = 0; i < 4; ++i) {
				for (int j = 0; j < 4; ++j) {
					int	k;
					msg	= Message.create(main_thread, key, length);
					try {
						for (k = 0; k < length; ++k) {
							msg.put("Packet " + k, false, true, 100);
						}
						for (int l = 0; l < j; ++l) {
							packet	= (String)(msg.get(true
														, true
														, 100
														));
						}
						for (int l = 0; l < j; ++l, ++k) {
							msg.put("Packet " + k, false, true, 100);
						}
						msg.put("Packet Blocked", true, true, 100);
					} catch (Exception e) {
						print_exception(e);
					}

					msg1	= Message.attach(key);
					try {
						switch (i) {
						case 0:
							for (k = 0; k < length + 1; ++k) {
								packet1	= (String)(msg1.get(true
															, true
															, 100
															));
								System.out.print("i = " + i
												+ ", j = " + j
												+ ", k = " + k
												+ ", packet1 is "
												+ "\"" + packet1 + "\""
												+ "\n"
												);
							}
							break;

						case 1:
							for (k = 0; k < length + 1; ++k) {
								packet1	= (String)(msg1.get(false
															, true
															, 100
															));
								System.out.print("i = " + i
												+ ", j = " + j
												+ ", k = " + k
												+ ", packet1 is "
												+ "\"" + packet1 + "\""
												+ "\n"
												);
							}
							break;

						case 2:
							head	= true;
							for (k = 0; k < length + 1; ++k) {
								packet1	= (String)(msg1.get(head
															, true
															, 100
															));
								System.out.print("i = " + i
												+ ", j = " + j
												+ ", k = " + k
												+ ", packet1 is "
												+ "\"" + packet1 + "\""
												+ "\n"
												);
								head	= (!head);
							}
							break;

						case 3:
							head	= false;
							for (k = 0; k < length + 1; ++k) {
								packet1	= (String)(msg1.get(head
															, true
															, 100
															));
								System.out.print("i = " + i
												+ ", j = " + j
												+ ", k = " + k
												+ ", packet1 is "
												+ "\"" + packet1 + "\""
												+ "\n"
												);
								head	= (!head);
							}
							break;

						default:
							break;
						}
					} catch (Exception e) {
						print_exception(e);
					}

					msg.destroy(main_thread, key);
				}
			}

			System.out.print("*** Test 2 finished.\n\n");

		}
		else if(args.length == 1) {
			// Test 3: Inter-thread communicatins
			System.out.print("*** Test 3 Starts...\n");
			long	t0 = System.currentTimeMillis(), t1;
			key	= "Test3";
			Semaphore	sem1;
			int			p_thread_cnt = Integer.parseInt(args[0])
						, c_thread_cnt = Integer.parseInt(args[0])
						, i;
			MessageTest[]	consumers = new MessageTest[c_thread_cnt];
			MessageTest[]	producers = new MessageTest[p_thread_cnt];

			System.out.print("args[0] = " + args[0]
							+ "p_thread_cnt = " + p_thread_cnt
							+ ", c_thread_cnt = " + c_thread_cnt
							);
			for (i = 0; i < c_thread_cnt; ++i) {
				consumers[i]	= new MessageTest(null
												, false
												, key
												, i
												,c_thread_cnt
												);
				try {
					consumers[i].start();
				} catch (Error e) {
					System.out.print(key + ":FATAL: consumers[" + i
									+ "].start() throws " + e + "\n");
					e.printStackTrace();
					return;	// FATAL
				}
				System.out.print(key + ": consumers[" + i
								+ "] starts..."
								+ "\n"
								);
			}

			sem1	= new Semaphore(main_thread
									, false
									, null
									, producers.length
									);
			System.out.print(key + ": sem1 is " + sem1 + "\n");
			for (i = 0; i < p_thread_cnt; ++i) {
				producers[i]	= new MessageTest(sem1
												, true
												, key
												, i
												, p_thread_cnt
												);
				try { 
					producers[i].start();
				} catch (Error e) {
					System.out.print(key + ":FATAL: producers[" + i
									+ "].start() " + "throws " + e
									+ "\n"
									);
					e.printStackTrace();
					return;	// FATAL
				}
				System.out.print(key + ": producers[" + i
								+ "] starts..."
								+ "\n"
								);
			}

			sem1.leave(main_thread);
			try { Thread.sleep(1000); } catch (Exception e) { ; }
			sem1.take(main_thread, true);
			System.out.print(key + ": producers sent all packets.\n");

			sem1.leave(main_thread);
			for (i = 0; i < p_thread_cnt; ++i) {
				try { producers[i].join(); } catch (Exception e) { ; }
			}
			System.out.print(key + ": All producers are exited.\n");

			do {
				i = consumers[0].queue.get_message_cnt();
				System.out.print(key + ": "
								+ "consumers[0].queue.get_message_cnt()"
								+ " returns " + i
								+ "\n"
								);
				try { Thread.sleep(200); } catch (Exception e) { ; }
			} while (i != 0);
			Message.destroy(consumers[0], key);

			for (i = 0; i < c_thread_cnt; ++i) {
				try { consumers[i].join(); } catch (Exception e) { ; }
			}
			System.out.print(key + ": All consumers are exited.\n");

			t1	= System.currentTimeMillis();
			System.out.print(key + ": Elapsed time t0 = " + t0
							+ ", t1 = " + t1 + ", t1 - t0 = "
							+ (t1 - t0) + ", thread cnt = "
							+ c_thread_cnt
							+ "\n"
							);
			System.out.print("*** Test 3 finished.\n\n");
		}
		else {
			System.out.print("Usage: java MessageTest <thread cnt>\n");
			return;	// Usage Error
		}

		System.out.print("*** All tests are OK!\n");
		return;	// NORMAL END
	}
}
