/***********************************************************************
 * File: SemaphoreTest.java
 * 	(Extracted from
 *	   "$Id: SemaphoreTest.java,v 1.5 2004/06/12 17:47:26 maoyam Exp maoyam $")
 *
 * Abstracts:
 *   This Java source code implements the test program of
 * the semaphore utility class `my_util.Semaphore'.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import	my_util.Semaphore;

public class SemaphoreTest extends Thread {
	private Semaphore	semaphore = null;
	private Semaphore	semaphore2 = null; // for randezvous

	private String	test_desc	= null;

	// --simple test
	static int	x	= 0; // --simple test : shared class field
	private boolean	even_actor;
	private boolean sem_enable;
	public SemaphoreTest(Semaphore sem
						, boolean even_actor
						, boolean sem_enable
						) {
		System.out.print("SemaphoreTest:--simple test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		semaphore	= sem;
		this.even_actor	= even_actor;
		this.sem_enable	= sem_enable;
	}

	// --shared-object test
	static final int	TAKER	= 1, POLLER = 2;
	private int	taker = 0; // --shared_object test : threads kind
	public SemaphoreTest(Semaphore [] sem, int taker) {
		System.out.print("SemaphoreTest:--shared-object test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		semaphore	= sem[0];
		semaphore2	= sem[1];
		this.taker	= taker;
	}

	// --create-taken test
	static final long SLEEP_MSECS = 2345;
	private long sleep_msecs = 0;	// --create-taken test
	public SemaphoreTest(long sleep_msecs, Semaphore sem) {
		System.out.print("SemaphoreTest:--create-taken test: "
						+ "this.hashCode() = " + this.hashCode()
						+ "\n"
						);
		this.sleep_msecs	= sleep_msecs;
		if (sleep_msecs < 0) semaphore	= sem;
	}

	// --monopolize-sharing-mixed test
	public int		param_cnt		= 0;
	public int		calculated_cnt	= 0;
	public int[]	param_A	= new int[40];
	public int[]	calculator	= new int[param_A.length];
	public int[]	param_B	= new int[param_A.length];
	private int	calculator_id	= -1;
	public SemaphoreTest(String test_desc) {
		this.test_desc	= test_desc;
	}
	public SemaphoreTest(String test_desc, int id, Semaphore sem) {
		calculator_id	= id;
		this.test_desc	= test_desc;
		semaphore		= sem;
	}

	// "--timeout-take" test
	private	String test_name	= null;
	public SemaphoreTest(String test_name, Semaphore sem) {
		if (sem != null) {
			semaphore	= sem;
		}
		this.test_name	= test_name;
	}

	// Never Used!
	public SemaphoreTest() {
	}

	// "--timeout-take" test (child-thread main loop)
	private void timeout_take_child_run() {
		boolean	sharing;

		// Child start...
		System.out.print("Child-thread(" + this + "): starts...\n");

		// Get sharing flag (exclusive object via `semaphore')
		if (!semaphore.take(this, true)) {
			System.out.print("Child-thread(" + this
							+ "): take(#1) failed."
							+ "\n"
							);
			return;	// FATAL
		}
		else {
			Boolean	sharingo;

			try {
				sharingo	= (Boolean)(
									semaphore.get_exclusive_object(this)
									);
			} catch (Exception e) {
				System.out.print("Child-thread(" + this
								+ "): get_exclusive_object() failed."
								+ e
								+ "\n"
								);
				e.printStackTrace();
				return;	// FATAL
			}

			sharing	= sharingo.booleanValue();

			if (!semaphore.leave(this)) {
				System.out.print("Child-thread(" + this
								+ "): leave(#1) failed."
								+ "\n"
								);
				return;	// FATAL
			}
		}

		// Re-take `semaphore' according to `sharing'
		System.out.print("Child-thread(" + this +"): take()"
						+ " sharing = " + sharing 
						+ ".\n"
						);
		if (!semaphore.take(this, sharing, true)) {
			System.out.print("Child-thread(" + this
							+ "): take(#2) failed."
							+ "\n"
							);
			return;	// FATAL
		}
		else {

			System.out.print("Child-thread(" + this +"): sleeps...\n");
			try { this.sleep(5 * 1000); } catch (Exception e) { ; }
			System.out.print("Child-thread(" + this
							+ "): ...Wakes up!"
							+ "\n"
							);

			if (!semaphore.leave(this)) {
				System.out.print("Child-thread(" + this
								+ "): leave(#2) failed."
								+ "\n"
								);
				return;	// FATAL
			}
		}

		// Child exits.
		System.out.print("Child-thread(" + this + "): ...Exit\n");
		return;	// Normal end
	}

	// "--timeout-take" test (parent-thread main loop)
	private void timeout_take_initial_run() {
		boolean	child_sharing = true, sharing = true;
		// Parent start...
		System.out.print("Parent-thread(" + this + "): starts...\n");

		do {
			do {
				semaphore	= new Semaphore(new Boolean(child_sharing));
				SemaphoreTest child	= new SemaphoreTest(
												"--timeout-take-child"
												, semaphore
												);

				child.start();

				try { this.sleep(200); } catch (Exception e) { ; }

				for (int t = 10; t > 0; --t) {
					boolean	r;
					long	timeout	= t * 100;
					try {
						r = semaphore.take(this
											, sharing
											, true
											, timeout
											);
					} catch (Exception e) {
						System.out.print("Parent-thread(" + this
										+ "): semaphore.take(sharing = "
										+ sharing + ") throws " + e
										+ " '" + e.getMessage() + "'"
										+ "\n"
										);
						e.printStackTrace();
						continue; // Exception will be timeout.
					}
					System.out.print("Parent-thread(" + this
									+ "): timeout = " + timeout
									+ ", take(sharing = " + sharing
									+ ") returns " + r
									+ "\n"
									);
					if ((r) && !semaphore.leave(this)) {
						System.out.print("Parent-thread(" + this
										+ "): semaphore.leave() failed"
										+ "\n"
										);
						return;	// FATAL
					}
				}

				try { child.join(); } catch (Exception e) { ; }

				child_sharing	= !child_sharing;
			} while (child_sharing == false);

			sharing	= !sharing;
		} while (sharing == false);

		// Child exits.
		System.out.print("Parent-thread(" + this + "): ...Exit\n");
		return;	// Normal end
	}

	// main loop
	public void run() {
		boolean	blocking	= true;
		long	sleep_interval;
		int		loop_cnt;

		System.out.print("SemaphoreTest.run():: this.hashCode() = "
						+ this.hashCode()
						+ "\n"
						);

		if (test_name != null) {
			if (test_name.compareTo("--timeout-take") == 0) {
				timeout_take_initial_run();
			}
			else if (test_name.compareTo("--timeout-take-child") == 0) {
				timeout_take_child_run();
			}
			return;
		}

		if (test_desc != null) {
			if (calculator_id == -1) {
				int[]	max_owners =
							{ Semaphore.MIN_CNT_OF_OWNERS
							, Semaphore.DEFAULT_CNT_OF_SHARING_OWNERS
							, param_A.length
							};

				for (int j = 0; j < max_owners.length; ++j) {
					SemaphoreTest[]	calculators	=
									new SemaphoreTest[param_A.length];
					int	i;
					long	t0, t1;

					// Initial start...
					System.out.print("Initial Thread: starts...\n");
					t0	= System.currentTimeMillis();

					calculated_cnt	= 0;
					semaphore = new Semaphore(this
											, this
											, max_owners[j]
											); {
						for (	
						; param_cnt < param_A.length / 2
						; ++param_cnt
						) {
							param_A[param_cnt]	= param_cnt;
						}
						for (i = 0; i < param_A.length; ++i) {
							calculator[i]	= -1;
							param_B[i]	= 0;
						}

						for (i = 0; i < param_A.length / 2; ++i) {
							calculators[i]	= new SemaphoreTest(
															test_desc
															, i
															, semaphore
															);
							System.out.print("Initial Thread: starts "
											+ "Calculators[" + i + "] ("
											+ calculators[i].hashCode()
											+ ") semaphore = "
											+ semaphore
											+ ")\n"
											);
							calculators[i].start();
						}

						if (!semaphore.leave(this)) {
							System.out.print("Inital thread"
											+": leaves semaphore failed"
											+ "\n"
											);
							return;	// FATAL
						}
					}

					semaphore2 = new Semaphore(this, null); {
						if (!semaphore.take(this, false, true)) {
							System.out.print("Initial Thread"
											+ ": takes "
											+ "semaphore failed"
											+ "\n"
											);
							return; // FATAL
						}
						else {
							System.out.print("Initial Thread"
											+ ": sets the rest of "
											+ "param_A[], param_cnt = "
											+ param_cnt
											+ "\n"
											);
							for (
							; param_cnt < param_A.length
							; ++param_cnt) {
								param_A[param_cnt]	= param_cnt;
								try { this.sleep(200);
								} catch (Exception e) { ; }
							}

							if (!semaphore.leave(this)) {
								System.out.print("Initial Thread"
												+": leaves semaphore "
												+ "failed"
												+ "\n"
												);
								return;	// FATAL
							}
						}

						if (!semaphore2.leave(this)) {
							System.out.print("Initial Thread"
											+": smepahore2.leave() "
											+ "return false"
											+ "\n"
											);
							return;	// FATAL
						}
					}

					for (i = 0; i < calculators.length; ++i) {
						try { calculators[i].join();
						} catch (Exception e) { ; }
					}

					t1	= System.currentTimeMillis();

					System.out.print("Elapsed time: t0 = " + t0
									+ ", t1 = " + t1
									+ ", t1 - t0 = " + (t1 - t0)
									+ "\n"
									);
					System.out.print(" i : param_A[] : calculator[] "
													+ ": param_B[]\n");
					for (i = 0; i < param_A.length; ++i) {
						System.out.print(
						" " + i + " :  " + param_A[i]
						+ " : " + calculator[i]
						+ " : " + param_B[i]
						+ "\n"
						);
					}
				}
			}
			else {
				SemaphoreTest	ts;
				boolean			must_calc, must_idoling;

				// calculators starts...
				System.out.print("Calculator (" + this.hashCode()
								+ "): id = " + this.calculator_id
								+ ", semaphore = " + semaphore
								+ ": starts...\n"
								);
				while (true) {
					int	i;

					if (!semaphore.take(this, true, true)) {
						System.out.print("Calculator ("
										+ this.hashCode() + "): "
										+ "semaphore.take(" + this
										+ ", " + true + ", " + true
										+ ") returns false\n"
										);
						return;	// FATAL
					}
					else {
						try {
							ts	= (SemaphoreTest)
								semaphore.get_exclusive_object(this);
						} catch (Exception e) {
							System.out.print("Calculator ("
											+ this.hashCode() + "): "
											+ " semaphore."
											+ "get_exclusive_object("
											+ this + ") throws " + e
											+ "\n"
											);
							e.printStackTrace();
							return;	// FATAL
						}

						must_idoling	= (ts.calculated_cnt
											== ts.param_A.length / 2);
						must_calc = (ts.calculated_cnt < ts.param_cnt);

						if (!semaphore.leave(this)) {
							System.out.print("Calculator ("
											+ this.hashCode() + "): "
											+ " semaphore.leave("
											+ this + ") returns false"
											+ "\n"
											);
							return;	// FATAL
						}
					}

					if (must_idoling) {
						System.out.print("Calculator ("
										+ this.hashCode() + "): "
										+ "Wait for initial thread "
										+ "leaves ts.semaphore2..."
										+ "\n"
										);

						if (!ts.semaphore2.take(this, true)) {
							System.out.print("Calculator ("
											+ this.hashCode() + "): "
											+ " semaphore2.take("
											+ this + ", " + true
											+ ") returns false"
											+ "\n"
											);
							return;	// FATAL
						}
						else {
							if (!ts.semaphore2.leave(this)) {
								System.out.print("Calculator ("
												+ this.hashCode()
												+ "): semaphore2.leave("
												+ this
												+ ") returns false"
												+ "\n"
												);
								return;	// FATAL
							}
						}

					}
					else if (!must_calc) {
						System.out.print("Calculator ( "
										+ this.hashCode()
										+ "): ts.calculator_cnt reached"
										+ " " + ts.param_A.length
										+ ".  This thread exits!\n"
										);
						break;
					}

					if (!semaphore.take(this, false, true)) {
						System.out.print("Calculator ("
										+ this.hashCode() + "): "
										+ " semaphore.take("
										+ this + ", " + false + ", "
										+ true + ") returns false"
										+ "\n"
										);
						return;	// FATAL
					}
					else {
						for (i = 0; i < ts.param_cnt; ++i) {
							if (ts.calculator[i] < 0) {
								ts.calculator[i]	= this.hashCode();
								++ts.calculated_cnt;
								break;
							}
						}

						if (!semaphore.leave(this)) {
							System.out.print("Calculator ("
											+ this.hashCode() + "): "
											+ " semaphore.leave("
											+ this + ") returns false"
											+ "\n"
											);
							return;	// FATAL
						}
					}

					if (i == ts.param_A.length) {
						System.out.print("Calculator ( "
										+ this.hashCode()
										+ "): All entry of ts."
										+ "calculator[] had hashCodes."
										+ "  This thread exits!\n"
										);
						break;
					}

					if (i < ts.param_A.length / 2) {
						ts.param_B[i]	= ts.param_A[i] + ts.param_A[i];
					}
					else {
						ts.param_B[i]	= ts.param_A[i] * ts.param_A[i];
					}
					try { this.sleep(100); } catch (Exception e) { ; }
				}
			}

			return;
		}

		if (sleep_msecs > 0) {
			// --create-taken test
			SemaphoreTest child;

			System.out.print("CREATE-TAKER: starts...\n");
			StringBuffer msg = new StringBuffer("How are you?");
			semaphore	= new Semaphore(this, msg); {
				if (!semaphore.take(this, true)) {
					System.out.print("CREATE-TAKER: Twice "
									+ "semaphore.take("
									+ this + ", true) returns false."
									+ "\n"
									);
					// For Test
				}

				System.out.print("CREATE-TAKER: make child to start\n");
				child = new SemaphoreTest(-sleep_msecs, semaphore);
				child.start();

				System.out.print("CREATE-TAKER: sleeps...\n");
				try { this.sleep(sleep_msecs);
				} catch (Exception e) { ; }

				if (!semaphore.leave(this)) {
					System.out.print("CREATE-TAKER: semaphore.leave("
									+ this + ") returns false."
									+ "\n"
									);
					return;	// FATAL
				}
				if (!semaphore.leave(this)) {
					System.out.print("CREATE-TAKER: Twice "
									+ "semaphore.leave("
									+ this + ") returns false."
									+ "\n"
									);
					// For test
				}
			}

			try { child.join(); } catch (Exception e) { ; }

			System.out.print("CREATE-TAKER: child answer is \""
							+ msg + "\"."
							+ "\n"
							);

			System.out.print("CREATE-TAKER: exits.\n");
			return;
		}
		else if (sleep_msecs < 0) {
			// --create-taken test
			StringBuffer msg;

			System.out.print("CREATE-CHILD: starts...\n");
			if (semaphore.take(this, true)) {
				try { msg = (StringBuffer)
							semaphore.get_exclusive_object(this);
				} catch (Exception e) {
					System.out.print("CREATE-CHILD: semaphore."
									+ "get_exclusive_object(" + this
									+ ") returns false."
									+ "\n"
									);
					e.printStackTrace();
					return;	// FATAL
				}

				System.out.print("CREATE-CHILD: parent msg is \""
								+ msg + "\""
								+ ".\n"
								);

				System.out.print("CREATE-CHILD: sleeps...\n");
				try { this.sleep(-sleep_msecs);
				} catch (Exception e) { ; }

				msg.append(" --- I am Fine!");
				System.out.print("CREATE-CHILD: edited msg is \""
								+ msg + "\""
								+ ".\n"
								);

				if (!semaphore.leave(this)) {
					System.out.print("CREATE-CHILD: semaphore."
									+ "leave(" + this + ") "
									+ "returns false."
									+ "\n"
									);
					return;	// FATAL
				}
			}

			System.out.print("CREATE-CHILD: exits.\n");
			return;
		}
		else if (taker == SemaphoreTest.TAKER) {
			// --shared-object test
			// Tester for taking shared object (taker thread)
			System.out.print("Taker: starts...\n");
			blocking	= true;
			sleep_interval	= 270;

			if (!semaphore.take(this, blocking)) {
				System.out.print("Taker blocking = " + blocking
								+ ": semaphore.take(" + this
								+ ", " + blocking + ") returns false."
								+ "\n"
								);
			} else {
				StringBuffer	shared_obj;
				try {
					shared_obj = (StringBuffer)
								semaphore.get_exclusive_object(this);

					shared_obj.append("Hi, ");
					System.out.print("Taker: edited (#1) shared_obj "
									+ "is \"" + shared_obj + "\"."
									+ "\n"
									);
				} catch (Exception e) {
					System.out.print("Taker: "
									+ "semaphore.get_exclusive_object("
									+ this + ") throws Exception " + e
									+ "\n"
									);
					e.printStackTrace();
					return; // FATAL
				}

				System.out.print("Taker: sleeps... \n");
				try { this.sleep(1100); } catch (Exception e) { ; }

				shared_obj.append("Poller!");
				System.out.print("Taker: waked up! Taker edited (#2) "
								+ "shared_obj is \""
								+ shared_obj + "\""
								+ "\n"
								);

				if (!semaphore.leave(this)) {
					System.out.print("Taker: semaphore.leave( "
									+ this + ") returns false."
									+ " And then, taker waits responce "
									+ "from poller, via the shared "
									+ "object."
									+ "\n"
									);
					return;	// FATAL
				}
			}

			while (!semaphore2.take(this, false)) {
				System.out.print("Taker: Poller had taken semaphore2."
								+ "\n"
								);
				try { this.sleep(170); } catch (Exception e) { ; }
			}
			if (!semaphore2.leave(this)) {
				System.out.print("Taker: semaphore2.leave(" + this
								+ ") returns flase."
								+ "\n"
								);
				return;	// FATAL
			}

			if (!semaphore.take(this, blocking)) {
				System.out.print("Taker blocking = " + blocking
								+ ": Can't take semaphore "
								+ "for waiting responce."
								+ "\n"
								);
			} else {
				StringBuffer	shared_obj;

				try {
					shared_obj	= (StringBuffer)
								semaphore.get_exclusive_object(this);
					System.out.print("Taker: Responce from poller is \""
									+ shared_obj + "\"."
									+ "\n"
									);
				} catch (Exception e) {
					System.out.print("Taker: semaphore."
									+			"get_exclusive_object("
									+ this
									+ ") throws Exception " + e
									+ "(responce)"
									+ "\n"
									);
					e.printStackTrace();
					return; // FATAL
				}

				if (!semaphore.leave(this)) {
					System.out.print("Taker: semaphore.leave("
									+ this + ") returns false, "
									+ "after responce-wait."
									+ "\n"
									);
				}
			}

			System.out.print("Taker: ... returns (thread exits)\n");
			return;
		}
		else if (taker == SemaphoreTest.POLLER) {
			// --shared-object test
			// Tester for polling shared object (poller thread)
			System.out.print("Poller: starts...\n");
			blocking	= false;
			sleep_interval	= 230;

			System.out.print("Poller: semaphore2.take(" + this
							+ ", true)..."
							+ "\n"
							);
			if (!semaphore2.take(this, true)) {
				System.out.print("Poller: semaphore2.take(" + this
								+ ") return false."
								+ "\n"
								);
				return;	// FATAL
			} else {
				while (true) {
					StringBuffer shared_obj;

					while (!semaphore.take(this, blocking)) {
						System.out.print("Poller blocking = " + blocking
										+ ": Taker had taken semaphore."
										+ "\n"
										);
						try { this.sleep(sleep_interval);
						} catch (Exception e) { ; }
					}

					try {
						shared_obj = (StringBuffer)
								semaphore.get_exclusive_object(this);
					} catch (Exception e) {
						System.out.print("Poller: semaphore."
										+ "get_exclusive_object(" + this
										+ ") throws Exception " + e
										+ "."
										+ "\n"
										);
						e.printStackTrace();
						return;	// FATAL
					}

					if (!(shared_obj.toString().equals(""))) {
						shared_obj.reverse();
						System.out.print("Poller: reversed shared_obj "
										+ "is \"" + shared_obj + "\"."
										+ "\n"
										);

						if (!semaphore.leave(this)) {
							System.out.print("Poller: semaphore."
											+ "leave( " + this
											+ ") returns false."
											+ "\n"
											);
							return;	// FATAL
						}

						break;
					}

					System.out.print("Poller: Taker hasn't taken "
									+ "semaphore."
									+ "\n"
									);
					if (!semaphore.leave(this)) {
						System.out.print("Poller: smaphore.leave("
										+ this + ") returns false."
										+ "\n"
										);
						return;	// FATAL
					}
					try { this.sleep(170); } catch(Exception e) { ; }
				}

				System.out.print("Poller: sleeps...\n");
				try { this.sleep(1300); } catch (Exception e) { ; }

				if (!semaphore2.leave(this)) {
					System.out.print("Poller: semaphore2.leave(" + this
									+ ") return false."
									+ "\n"
									);
					return;	// FATAL
				}
				System.out.print("Poller: semaphore2.leave(" + this
								+ ") is done."
								+ "\n"
								);
			}

			System.out.print("Poller: ... returns (thread exits)\n");
			return;
		}

		// --simple test
		// 2 Threads without/within semaphore
		if ((sem_enable == true) && !semaphore.take(this, blocking)) {
			System.out.print("even_actor = " + even_actor
							+ "sem_enable = " + sem_enable
							+ " semaphore.take(" + this
							+ ", " + blocking
							+ ") returns false.\n"
							);
		}
		else {
			System.out.print("SemaphoreTest.run(): "
							+ ((even_actor)? "even": "odd")
							+ " starts...\n"
							);
			if (even_actor) {
				x	= 2;
				sleep_interval	= 110;
				loop_cnt	= 30;
			}
			else {
				x	= 1;
				sleep_interval	= 230;
				loop_cnt	= 15;
			}

			for (int i = 0; i < loop_cnt; ++i) {
				System.out.print("even_actor = " + even_actor
								+ ": x = " + x + "\n"
								);
				try {
					this.sleep(sleep_interval);
				}
				catch (Exception e) {
					System.out.print("even_actor = " + even_actor
									+ "sem_enable = " + sem_enable
									+ " i = " + i
									+ ": run() caught exception "
									+ "from this.sleep()"
									);
					e.printStackTrace();
				}
				x	+= 2;
			}

			if ((sem_enable == true)
			&& !semaphore.leave(this)) {
				System.out.print("even_actor = " + even_actor
								+ "sem_enable = " + sem_enable
								+ " semaphore.leave(" + this
								+ ") returns false.\n"
								);
			}
		}

		return;
	}
		
	static public void main(String[] args) {
		Semaphore	sem = new Semaphore();
		boolean	within_sem = false;
		String	within_msg = "without";

		if (args.length != 0
		&& (args[0].equals("--help") || args[0].equals("-h"))) {
			System.out.print("java SemaphoreTest [--help "
						+ "| --simple "
						+ "| --shared-object "
						+ "| --create-taken"
						+ "]\n"
						+	 "  --help (or -h)  : shows this message\n"
						+	 "  --simple        : simple effect of "
						+							"semaphore\n"
						+	 "  --shared-object : sharing an object "
						+							"and randexbous\n"
						+	 "  --create-object : create an taken a"
						+					"monopolized semaphore\n"
						+	 "  * If nothig is specified, this test "
						+							"runs all tests.\n"
						);
			return;
		}

		if (args.length == 0 || args[0].equals("-simple")) {
			for (int i = 0; i < 2; ++i) {
				SemaphoreTest even = new SemaphoreTest(sem
														, true
														, within_sem
														);
				SemaphoreTest odd = new SemaphoreTest(sem
														, false
														, within_sem
														);

				System.out.print("SemaphoreTest.main(): "
								+ "2 Threads " + within_msg
								+ " Semaphore\n"
								);
				even.start();
				odd.start();

				try { odd.join(); } catch (Exception e) {
					System.out.print("main() i = " + i
									+ ": Exception in odd.join()\n");
					e.printStackTrace();
				}
				try { even.join(); } catch (Exception e) {
					System.out.print("main() i = " + i
									+ ": Exception in even.join()\n");
					e.printStackTrace();
				}

				within_sem	= true;
				within_msg	= "within";
				System.out.print("\n");
			}
		}

		if (args.length == 0 || args[0].equals("--shared-object")) {
			for (int i = 0; i < 2; ++i) {
				StringBuffer	shared_obj = new StringBuffer("");
				Semaphore [] sem2 = {
								  new Semaphore((Object)shared_obj)
								, new Semaphore(null)
								};
				SemaphoreTest	taker =
										new SemaphoreTest(sem2
											, SemaphoreTest.TAKER
											);
				SemaphoreTest	poller =
										new SemaphoreTest(sem2
											, SemaphoreTest.POLLER
											);

				System.out.print("main():#" + i + " Semaphore taker "
								+ "and Polling thread "
								+ "(these shares an object)\n"
								);

				taker.start(); poller.start();
				try { poller.join(); taker.join();
				} catch (Exception e) { ; }

				System.out.print("\n");
			}
		}

		if (args.length == 0 || args[0].equals("--create-taken")) {
			System.out.print("main(): Create-Taken test\n");
			SemaphoreTest parent = new SemaphoreTest(SLEEP_MSECS
														, null
														);
			parent.start();
			try { parent.join(); } catch (Exception e) { ; }

			System.out.print("\n");
		}

		if (args.length == 0 || args[0].equals("--timeout-take")) {
			System.out.print("main(): Timeout test\n");
			SemaphoreTest parent = new SemaphoreTest(
											((args.length == 0)?
												"--timeout-take"
												: args[0]
												)
											, null
											);
			parent.start();
			try { parent.join(); } catch (Exception e) { ; }

			System.out.print("\n");
		}

		if (args.length == 0
		|| args[0].equals("--monopolize-sharing-mixed")) {
			System.out.print("main(): Monopolize and Sharing mixed test"
							+ "\n"
							);

			SemaphoreTest	initial_thread =
									new SemaphoreTest(
										((args.length == 0)?
											"--monopolize-sharing-mized"
											: args[0])
										);
			initial_thread.start();
			try { initial_thread.join(); } catch (Exception e) { ; }

			System.out.print("\n");
		}

		return;
	}
}
