/***********************************************************************
 * File: SharedObjectTest.java
 * 	(Extracted from
 *	   "$Id: SharedObjectTest.java,v 1.1 2004/05/24 12:31:49 maoyam Exp maoyam $")
 *
 * Abstracts:
 *   This Java source code implements the test program of
 * the shared-object utility class `my_util.SharedObject'.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import	java.io.*;
import	my_util.SharedObject;
import	my_util.Sharable;

public class SharedObjectTest implements Runnable {
	private SharedObject	so = null;

	public SharedObjectTest(String key) {
		so	= SharedObject.attach(key);
		if (so	== null) {
			System.out.print("***FAILED::SharedObjectTest:constructor: "
							+ "Failed in  SharedObject.attach(" + key
							+ ")\n"
							);
		}
	}

	static public void print_exception(Exception e) {
		System.out.print(e.getMessage());
		e.printStackTrace();
	}

	static private String[] read_init_val_file(String path) {
		String[]	vals = null;

		try {
			File	f = new File(path);
			if (f.isFile() && f.canRead()) {
				BufferedReader	br = new BufferedReader(
										new InputStreamReader(
											new FileInputStream(f)
										));

				vals	= new String[3];
				int	i;
				for (i = 0; i < vals.length; ++i) {
					if ((vals[i] = br.readLine()) == null) {
						break;
					}
				}
				while (++i < vals.length) {
					vals[i]	= null;
				}

				br.close();
			}
		} catch (Exception e) {
			print_exception(e);
		}

		return	vals;
	}

	static private void configure_shared_object(String key
							, String sharable_class_name
							, int max_sharers
							, String[] init_vals
							) throws Exception {
		SharedObject so	= SharedObject.create(key
								, sharable_class_name
								, max_sharers
								);
		if (so.take(false, true, 0)) {
			Table	tbl	= (Table)(so.get_object());
			if (tbl != null) {
				if (!tbl.initialize(init_vals)) {
					System.out.print("***FATAL::SharedObjectTest:"
									+ "configure_shared_object(" + key
									+ ", " +	max_sharers
									+ ", " +	init_vals
									+ "): failed in tbl.initialize("
									+ "init_vals)"
									+ "\n"
									);
				}
				else {
					System.out.print("SharedObjectTest:"
									+ "configure_shared_object(" + key
									+ ", " +	max_sharers
									+ ", " +	init_vals
									+ "): tbl.initialize(init_vals) "
									+ "returns true."
									+ "\n"
									);
					tbl.print_vals();
				}
			}
			else {
				System.out.print("***FATAL::SharedObjectTest:"
								+ "configure_shared_object(" + key
								+ ", " +	max_sharers
								+ ", " +	init_vals
								+ "): so.get_object() returns null!"
								+ "\n"
								);
			}

			if (!so.leave()) {
				System.out.print("***FATAL::SharedObjectTest:"
								+ "configure_shared_object(" + key
								+ ", " +	max_sharers
								+ ", " +	init_vals
								+ "): so.leave() returns false!"
								+ "\n"
								);
			}
		}

		return;
	}

	public void run() {
		System.out.print("SharedObjectTest [" + this + "] starts...\n");

		try {
			for (int i = 0, limit = 10; i < limit; ++i) {
				if (so.take(false, true, 0)) {
					Table	tbl	= (Table)(so.get_object());
					if (tbl != null) {
						tbl.update();

						System.out.print("SharedObjectTest [" + this
										+ "] updated Table object ["
										+ tbl + "] (i = " + i + ")"
										+ "\n"
										);
						tbl.print_vals();
					}
					else {
						System.out.print("***FAILED::SharedObjectTest:"
										+ "run(): so.get_object() "
										+ "returns null. "
										+ "(i = " + i + ")"
										+ "\n"
										);
						i	= limit;
					}
					if (!so.leave()) {
						System.out.print("***FAILED::SharedObjectTest:"
										+ "run(): so.leave() returns "
										+ "false."
										+ "(i = " + i + ")"
										+ "\n"
										);
						break;
					}
				}
				Thread.currentThread().sleep(10); // sleep in 10 msec
			}
		} catch (Exception e) {
			System.out.print("***FAILED::SharedObjectTest:run(): "
							+ "Caught an exception!"
							+ "\n"
							);
			print_exception(e);
		}

		System.out.print("SharedObjectTest [" + this + "] finished!\n");

		return;
	}

	static public void main(String[] args) {
		System.out.print("SharedObjectTest:main() starts...\n");
		if (args.length > 2) {
			System.out.print("Usage: "
							+ "java SharedObjectTest "
							+ 		"[max_sharers [init_val_file]]"
							+ "\n"
							);
			return;
		}

		try {
			String[]	init_vals = { "0" // update_count
									, "A" // name
									, "1" // value
									};
			if (args.length > 1) {
				init_vals	= read_init_val_file(args[1]);
				if (init_vals == null) {
					System.out.print("Can't open \"" + args[1]
									+ "\" file."
									+ "\n"
									);
					return;
				}
			}

			int	max_sharers	= 2;
			if (args.length > 0) {
				max_sharers	= Integer.parseInt(args[0]);
			}

			String	key	= "SharedObjectTest";
			configure_shared_object(key
									, "Table"	// sharable_class_name
									, max_sharers
									, init_vals
									);

			Thread[]	ts = new Thread[max_sharers];
			for (int i = 0; i < max_sharers; ++i) {
				SharedObjectTest	sot = new SharedObjectTest(key);
				ts[i]	= new Thread(sot);
				ts[i].start();
				System.out.print("SharedObjectTest:main(): start " + i
								+ "-th thread [" + ts[i] + "] "
								+ "running within [" + sot + "]"
								+ "\n"
								);
			}

			for (int i = 0; i < max_sharers; ++i) {
				ts[i].join();
				System.out.print("SharedObjectTest:main(): joined " + i
								+ "-th thread [" + ts[i] + "]"
								+ "\n"
								);
			}
		} catch (Exception e) {
			System.out.print("***FAILED::SharedObjectTest:main(): "
							+ "caught an exception!"
							+ "\n"
							);
			print_exception(e);
		}

		System.out.print("SharedObjectTest:main() finished!\n");
		return;
	}
}
