/***********************************************************************
 * File: TimerUserTest_once.java
 * 	(Extracted from
 *	   "$Id: TimerUserTest_once.java,v 1.3 2004/07/04 12:45:20 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements an agent uses the "once" Timer.
 *  TimerTest facility class runs this instance for testing the Timer
 * mechanism.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

import my_util.FacilityAgent;
import my_util.TimerUser;
import my_util.FacilityCommand;
import my_util.TimerCommand;
import my_util.Log;

// ^C}@\PIɎgp@\̗B
// @\̂ "ONCE TIMER USER"B
// Pb`Tb̊Ԋuuă^CAEg Timer  init() ŐA
// timer_expired() ł̒ʒm TimerCommand 󂯕tB
// ^CAEgʒm邽тɁAC^[obPbÂ₵
// BAC^[oTbԂɓBꍇARbԂ̏x~
// ^C}ELZsǍ̓^C}𐶐邱Ƃ͂ȂB
// timer_halt() Ăяoꂽꍇ́AAvP[V̏IɓB
// ƈӖ̂ŁA Timer LZB
// AvP[V̓^C}@\~ĂA@\ terminate() 
// ĂяoB
// Timer ̃LZƊJńAꂼAsuspend()  resume() ł
// sĂB
// ̋@\̓^C}@\gp̂ŁAȊO FacilityCommand
//  process() ĂяoŒʒmꂽꍇُ͈IB
//
public class TimerUserTest_once extends FacilityAgent
								implements TimerUser {

	// ̋@\̏IR[h
	static final public int UNKNOWN_COMMAND_RECIEVED = 1;
											// s process() Ăяo
	static final public int FAILED_SET_TIMER = 2;
											// ^C}Ɏs
	static final public int FAILED_CANCEL_TIMER = 3;
											// ^C}ELZɎs

	// ^C}̃C^[ob
	static final long INITIAL_INTERVAL = 1; // 1sec
	private long interval_sec = INITIAL_INTERVAL;

	// ̋@\Őgp Timer CX^X
	private TimerCommand tc = null;

	// ^C}ELZstO (true ALZς)
	private boolean timer_canceled = false;

	// ^C}
	private void make_once_timer_command() {
		tc	= new TimerCommand();
		tc.setIntervals(interval_sec);
		tc.setPeriodic(false); // once timer
	}

	// ^C}_vo͂
	private void dump_tc() {
		log(Log.INFO, 0
			, "+++ TimerCommand: " + tc + " fields are...\n"
			+ "\texpiredTime          = " + tc.getExpiredTime() + "\n"
			+ "\tlastExpireTimeMillis = " + tc.getLastExpireTimeMillis()
																+ "\n"
			+ "\texpiredTime - lastExpireTimeMillis = "
			+ (tc.getExpiredTime() - tc.getLastExpireTimeMillis())
																+ "\n"
			+ "\tintervals       = " + tc.getIntervals() + "\n"
			+ "\tperiodic        = " + tc.getPeriodic() + "\n"
			+ "\tlogger          = " + tc.getLogger() + "\n"
			+ "\thalt_this_timer = " + tc.getHalt_this_timer() + "\n"
			+ "\texpireTimeMillis  = " + tc.getExpireTimeMillis()
																+ "\n"
			+ "\t+++ FacilirtCommand fields are...\n"
			+ "\t\tfac_name = " + tc.getFac_name() + "\n"
			+ "\t\taid = " + tc.getAid() + "\n"
			+ "\t\tresp_to_agent = " + tc.getResp_to_agent() + "\n"
			+ "\t\treap_queue_name = " + tc.getResp_queue_name() + "\n"
			+ "\t\tstatus = " + tc.getStatus() + "\n"
			+ "\t\tprocessing_agent = " + tc.getProcessing_agent()
																+ "\n"
			+ "\t\thalt_cause = " + tc.getHalt_cause() + "\n"
			+ "\t\texception_class_name = "
								+ tc.getException_class_name() + "\n"
			+ "\t+++++++++++++++++++++++++++++++++\n"
			+ "+++++++++++++++++++++++++++++++++\n"
			);
	}

	// ^C}Jn
	private boolean start_timer() throws Exception {
		boolean result = false;

		if ((result = timer_start(tc)) == false) {
			log(Log.ERROR, FAILED_SET_TIMER
				, "start_timer() failed to set_timer(" + tc + ")");
		}
		else {
			log(Log.INFO, 0
				, "start_timer() starts the timer (" + tc
					+") interval_sec = " + interval_sec
				);
			result	= true;
		}

		return result;	// returns true: start the timer
						// returns false: failed to start timer
	}

	// ^C}ELZ
	private boolean cancel_timer() throws Exception {
		boolean result = false;

		if ((timer_canceled == false)
		&& (result = timer_cancel(tc)) == false) {	
			log(Log.ERROR, FAILED_CANCEL_TIMER
				, "cancel_timer() failed to cancel_timer(" + tc + ")");
		}
		else {
			log(Log.INFO, 0
				, "cancel_timer() cancels the timer (" + tc + ")");
			dump_tc();

			timer_canceled	= true;
			result	= true;
		}

		return	result;	// returns true: cancel the timer
						// returns false: failed to cancel timer
	}

	// @\Jn
	// PI^C}C^[oPbԂŐ
	// (FacilityAgent ۃNXv܂B)
	public boolean init() throws Exception {
		boolean	result	= false;
		log(Log.DEBUG, 0, "TimerUserTest_once.init() run.");

		make_once_timer_command();

		return	start_timer();	// returns true: the timer starts
								// returns false: failed to start timer
	}

	// ^C}C^[oԂo߂ƂʒmB
	// ʒmꂽ^C}_vo͂AC^[oԂPb
	// PI^C}𐶐B
	// (TimerUser C^[tF[Xv܂)
	public int timer_expired(TimerCommand cmd) throws Exception {
		log(Log.DEBUG, 0
			, "TimerUserTest_once.timer_expired(" + cmd + ") run.");

		dump_tc();

		interval_sec += INITIAL_INTERVAL;
		make_once_timer_command();
		start_timer();

		if (interval_sec == 5) {
			try { Thread.sleep(3000); } catch (Exception e) { ; }
			log(Log.DEBUG, 0
				, "TimerUsetTest_once.timer_expired(" + cmd
				+ ") cancel_timer() returns " + cancel_timer()
				);
		}

		return	this.NORMALEND;
	}

	// ^C}@\~ۂɌĂяo܂B
	// (TimerUser C^[tF[Xv܂)
	public int timer_halt(TimerCommand cmd) throws Exception {
		log(Log.DEBUG, 0
			, "TimerUserTest_once.timer_halt(" + cmd + ") run.");

		dump_tc();

		return	this.NORMALEND;
	}

	// ^C}ȊO FacilityCommand MꂽꍇɌĂяoB
	// vR}hM邱Ƃ͂Ȃ̂ŁA̋@\ُ͈I
	// (FacilityAgent ۃNXv܂B)
	public int process(FacilityCommand req) throws Exception {
		log(Log.DEBUG, 0, "TimerUserTest_once.process(" + req
			+ "<class: " + req.getClass().getName() + ">) run.");

		return	UNKNOWN_COMMAND_RECIEVED;
	}

	// ̋@\G[WFgTXyhۂɌĂяoB
	// ^C}LZ
	// (FacilityAgent ۃNXv܂B)
	public boolean suspend() throws Exception {
		log(Log.DEBUG, 0, "TimerUserTest_once.suspend() run.");

		return	cancel_timer();
	}

	// ̋@\G[WFgĊJۂɌĂяoB
	// ^C}𐶐
	// (FacilityAgent ۃNXv܂B)
	public boolean resume() throws Exception {
		log(Log.DEBUG, 0, "TimerUserTest_once.resume() run.");

		make_once_timer_command();

		return	start_timer();
	}

	// ^C}@\̒~ timer_halt() ŒʒmꂽA邢́A
	// ُ폈NꍇȂǂŁA@\~ۂɌĂяoB
	// (FacilityAgent ۃNXv܂B)
	public boolean terminate() throws Exception {
		log(Log.DEBUG, 0, "TimerUserTest_once.terminate() run.");

		return	cancel_timer();
	}
}
