/***********************************************************************
 * File: Conv_encoding.java
 * 	(Extracted from
 *	   "$Id: Conv_encoding.java,v 1.6 2004/06/12 17:36:47 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a Conv_encoding utility tool.
 *  This tool converts files from a encoding to another, and appends
 * the extension to the converted file paths.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

import	java.io.*;
import	my_util.Utillity;

/**
 * <p>t@C̕GR[fBOϊc[łB<br
 * >ϊ̏o̓t@C̃pX́A̓t@CEpX
 *  R}hEIvV <em>-e extention</em> Ŏw肳ꂽgq
 *  t̂ɂȂ܂B</p
 * ><p>gp@ (Usage)</p><ol
 * ><li>java my_util.Conv_encoding -e extension -f from_encoding 
 *                                         [-t to_encoding] files...</li
 * ><li>java my_util.Conv_encoding -e extension -t to_encoding 
 *															files...</li
 * ></ol
 * ><ul
 * ><li>Default "from_encoding" is "JISAutoDetected"</li
 * ><li>Japanese encodings...<ul
 *		><li>ISO2022JP -- JIS X 0201, ISO 2022-0208</li
 *		><li>JIS0201   -- JIS X 0201 ANK (JIS C 6220)</li
 *		><li>JIS0208   -- JIS X 0208 Basic Kanji</li
 *		><li>JIS0212   -- JIS X 0212 Supplementary</li
 *		><li>EUC-JP    -- EUC-coded JIS X0201, 0208, 0212</li
 *		><li>SJIS      -- Shift-JIS</li
 *		><li>JISAutoDetected -- {SJIS, EUC-JP, ISO2022JP} to Unicode</li
 *		><li>Cp33722   -- IBM-EUC-JP</li
 *		><li>MS932     -- Windows Kanji extended SJIS</li
 *		><li>Cp942     -- IBM OS/2 Extended Cp932</li
 *		><li>Cp942C    -- IBM OS/2 Extended Cp942</li
 *		><li>Cp943     -- IBM OS/2 Extended Cp932 and SJIS</li
 *		><li>Cp943C    -- IBM OS/2 Extended Cp943</li
 *		></ul></li
 * ><li>UTF (Unicode Transformation Formats)...<ul
 *		><li>UTF8                  -- 8 bits UTF</li
 *		><li>UTF-16                -- 16 bits UTF Marked endian</li
 *		><li>UnicodeBig            -- 16 bits UTF Big-endian 
 *													Marked endian</li
 *		><li>UnicodeBigUnmarked    -- 16 bits UTF Big-endian</li
 *		><li>UnicodeLittle         -- 16 bits UTF Littel-endian 
 *													Marked endian</li
 *		><li>UnicodeLittleUnmarked -- 16 bits UTF Littel-endian</li
 *		></ul></li
 * ><li>American and European encodings...<ul
 *		><li>ASCII     -- American Standard Code 
 *										for Information Interchange</li
 *		><li>ISO8859_1 -- ISO 8859-1 Latin alphabet No.1</li
 *		><li>Cp1252    -- Windows Latin characters-1</li
 *		></ul></li
 * ></ul>
 *
 * @author maoyam
 * @version 1.0
 */
public class Conv_encoding {
	// Any one can't create the instance of this class!
	private Conv_encoding() {}

	/**
	 * <p>vOEGg</p
	 * ><p>G[ꍇAGR[fBO͒f܂B
	 * </p>
	 * <ol
	 * ><li>̓t@CEpX݂ȂEt@CłȂ邢
	 *      Ǎ݌Ȃꍇ</li
	 * ><li>o̓t@CEpXɑ݂ĂāAt@CłȂ
	 *      邢͏݌Ȃꍇ</li
	 * ><li>w肳ꂽϊOѕϊ̕GR[fBO
	 *      ꂩ Java ꏈnŃT|[gĂȂꍇ</li
	 * ><li>̓t@CǍ݂At@C̃N[Y I/O G[
	 *      ꍇ</li
	 * ><li>LȊO̗Oꍇ</li
	 * ></ol>
	 * @param args R}hEIvVƓ̓t@CEpXEXg
	 */
	static public void main(String[] args) {
		if (args.length < 5
		|| (args[0].compareTo("-e") != 0
				&& args[2].compareTo("-f") !=0)
				&& args[2].compareTo("-t") != 0
		) {
			System.err.println(
			  "Usage:\n"
			+ "  java my_util.Conv_encoding -e extension "
									+ "-f from_encoding "
									+ "[-t to_encoding] files...\n"
			+ "  java my_util.Conv_encoding -e extension "
									+ "-t to_encoding files...\n"
			+ "\n"
			+ "    * Default \"from_encoding\" is \"JISAutoDetected\"\n"
			+ "    * Japanese encodings...\n"
			+ "        ISO2022JP -- JIS X 0201, ISO 2022-0208\n"
			+ "        JIS0201   -- JIS X 0201 ANK (JIS C 6220)\n"
			+ "        JIS0208   -- JIS X 0208 Basic Kanji\n"
			+ "        JIS0212   -- JIS X 0212 Supplementary\n"
			+ "        EUC-JP    -- EUC-coded JIS X0201, 0208, 0212\n"
			+ "        SJIS      -- Shift-JIS\n"
			+ "        JISAutoDetected -- {SJIS, EUC-JP, ISO2022JP} "
										+ "to Unicode\n"
			+ "        Cp33722   -- IBM-EUC-JP\n"
			+ "        MS932     -- Windows Kanji extended SJIS\n"
			+ "        Cp942     -- IBM OS/2 Extended Cp932\n"
			+ "        Cp942C    -- IBM OS/2 Extended Cp942\n"
			+ "        Cp943     -- IBM OS/2 Extended Cp932 and SJIS\n"
			+ "        Cp943C    -- IBM OS/2 Extended Cp943\n"
			+ "    * UTF (Unicode Transformation Formats)...\n"
			+ "        UTF8                  -- 8 bits UTF\n"
			+ "        UTF-16                -- 16 bits UTF "
												+ "Marked endian\n"
			+ "        UnicodeBig            -- 16 bits UTF "
												+ "Big-endian "
												+ "Marked endian\n"
			+ "        UnicodeBigUnmarked    -- 16 bits UTF "
												+ "Big-endian\n"
			+ "        UnicodeLittle         -- 16 bits UTF "
												+ "Littel-endian "
												+ "Marked endian\n"
			+ "        UnicodeLittleUnmarked -- 16 bits UTF "
												+ "Littel-endian\n"
			+ "    * American and European encodings...\n"
			+ "        ASCII     -- American Standard Code for "
									+ "Information Interchange\n"
			+ "        ISO8859_1 -- ISO 8859-1 Latin alphabet No.1\n"
			+ "        Cp1252    -- Windows Latin characters-1\n"
			);
			return;
		}

		int	idx = 1;
		String[]	encoding = { null, null };
		String	extension = args[idx];
		int	encoding_cnt = 0;
		for (++idx; encoding_cnt < 2 && idx < args.length; ++idx) {
			if (args[idx].compareTo("-f") == 0) {
				if (encoding[0] == null) ++encoding_cnt;
				encoding[0]	= args[++idx];
			}
			else if (args[idx].compareTo("-t") == 0) {
				if (encoding[1]  == null) ++encoding_cnt;
				encoding[1]	= args[++idx];
			}
			else {
				break;
			}
		}

		while (idx < args.length) {
			String	line = "";

			File ff = new File(args[idx]);
			if (!ff.exists() || !ff.isFile() || !ff.canRead()) {
				System.err.println("File " + args[idx]
								+ " is non-exist, isn't file "
								+ "or can't read."
								);
				break;
			}

			String	to_file = args[idx] + "." + extension;
			File tf = new File(to_file);
			if (tf.exists() && (!tf.isFile() || !tf.canWrite())) {
				System.err.println("File " + to_file
								+ " isn't file or can't write."
								);
				break;
			}

			PrintStream	out = null;
			try {
				FileInputStream	fis = new FileInputStream(ff);
				FileOutputStream fos = new FileOutputStream(tf);

				Utillity.encoding_stream2stream(fis
												, encoding[0]
												, fos
												, encoding[1]
												, false
												);
			} catch (Exception e) {
				if (e instanceof FileNotFoundException) {
					System.err.println("File " + args[idx]
										+ " is non-exist."
										);
				}
				else if (e instanceof UnsupportedEncodingException) {
					System.err.println("Encoding "
										+ ((out == null)?
											encoding[1]: encoding[0])
										+ " is unsupported."
										);
				}
				else if (e instanceof IOException) {
					if (line == null) {
						System.err.println("File "
										+ ((out == null)?	args[idx]:
															to_file
															)
										+ " occures close error."
										);
					}
					else {
						System.err.println("File " + args[idx]
										+ " occures readLine error."
										);
					}
				}
				else {
					System.err.println("An exception occures in "
										+ "converting input-file "
										+ args[idx] + " (encoding "
										+ encoding[0]
										+ ") to output-file "
										+ to_file + " (encoding "
										+ encoding[1] + ")"
										+ e.getMessage()
										);
					e.printStackTrace();
				}
				System.err.println(e.getMessage());
				e.printStackTrace();
				break;
			}

			++idx;
		}
	}
}
