/***********************************************************************
 * File: Facility.java
 * 	(Extracted from
 *	   "$Id: Facility.java,v 1.14 2004/07/04 12:21:10 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a Facility utility class.
 *  This class is the controler of FacilityAgent instances.
 *  This can activate, dectivate and change concurrency of instances.
 *  This can change the logging system resource and log-level.
 *  (Refered Class: Facility, FacilityAgentControlClass and Log)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package	my_util;

import	java.util.Hashtable;
import	my_util.Utillity;

/**
 * <p> Facility Ƃ́A<em>AvP[V<em>\</em
 *   >@\</em>\钊ۓIȎ̂Ӗ܂B
 *    Facility ̃CX^Xꂼ́AЂƂ<em>@\</em
 *   >S܂B</p>
 * <p>Facility CX^X̒S<em>@\</em
 *   >ɑΉ鏈̏ڍׂ FacilityAgent ɂĎ܂B
 *    CX^X͕ FacilityAgent CX^XǗ邱Ƃ<em
 *   >@\</em>̕sł܂B<br
 *   >CX^X<em>@\</em>̊JnEfEIA
 *    FacilityAgent Ɋ֘AtXbhsEҋ@E~邱Ƃ
 *    ܂B܂A~ĕs̕sx𓮓IɕύX
 *    Ƃł܂B</p>
 * <p>Facility CX^Xv󂯕tvL[ƂāA
 *    Message CX^X 1 gp܂B
 *    ̃L[AǗ FacilityAgent  FacilityCommand 
 *    ǂݏoĎۂ̏i߂܂B</p>
 * <p>Facility CX^X́ÅǗɂ FacilityAgent 
 *    ̃Oo͐ƋL^̃xǗ܂B
 *    Õxɂ́AvIȏQ (FATAL)AG[ (ERROR)A
 *    x (WARN)Aoߏ (INFO)AfobO (DEBUG) 
 *    Ƃ敪܂B</p>
 * @author maoyam
 * @version 1.0
 * @see FacilityAgent
 * @see FacilityAgentControlBlock
 * @see FacilityCommand
 * @see Log
 * @see Semaphore
 * @see Message
 */
public class Facility implements Loggable, Inspectable {

	// serial number of the facility that is invoked get_summary()
	// or get_details() from the Inspector
	private int inspector_serial_number = -1;

	/**
	 * <em>@\</em>̃T} InspectionData ƂĕԂ܂B
	 * Inspector Ăяo܂B
	 *
	 * @param ser_num <em>@\</em>̒ʔ
	 * @return T}ݒ肳ꂽ InspectionData
	 */
	protected InspectionData get_summary(int ser_num) {
		InspectionData result = null;

		inspector_serial_number	= ser_num;
		result	= get_summary();
		inspector_serial_number	= -1;

		return result;
	}

	/**
	 * <em>@\</em>̃T} InspectionData ƂĕԂ܂B
	 * Inspectable C^tF[XvĂ܂B
	 *
	 * @return T}ݒ肳ꂽ InspectionData
	 */
	public InspectionData get_summary() {
		InspectionData	summary = new InspectionData();

		boolean r = false, rr = false;
		try {
			if ((r = take(true, true, 0)) == true) {
				InspectionData	id = new InspectionData();
				id.add("", name);
				id.add("FacilityAgent NX", agclass_name);
				id.add("", ((active)? "": "񊈐"));
				id.add("AvP[V", app.get_name());
				id.add("ősx"
							, Integer.toString(max_concurrency, 10));
				id.add("݂̕sx"
							, Integer.toString(concurrency, 10));
				id.add("Oo͐"
							, ((log != null)? "ς": "Ȃ"));
				id.add("Oo͐pX", log_conf.path);

				summary.add(Integer.toString(inspector_serial_number
											, 10)
							, id
							);

				rr = leave();
			}
			if (!r || !rr) {
				throw new Exception(); /* goto catch-block */
			}
		} catch (Exception e) {
			summary.add("@\"
					, "!!bN" + ((!r)? "l": "") + "Ɏs!!"
					);
		}

		return	summary;
	}

	/**
	 * <em>@\</em>̏ڍ׏ InspectionData ƂĕԂ܂B
	 * Inspector Ăяo܂B
	 *
	 * @param ser_num <em>@\</em>̒ʔ
	 * @return ڍ׏ݒ肳ꂽ InspectionData
	 */
	protected InspectionData get_details(int ser_num) {
		InspectionData result = null;

		inspector_serial_number	= ser_num;
		result	= get_details();
		inspector_serial_number	= -1;

		return result;
	}

	/**
	 * <em>@\</em>̏ڍ׏ InspectionData ƂĕԂ܂B
	 * Inspectable C^tF[XvĂ܂B
	 *
	 * @return ڍ׏ݒ肳ꂽ InspectionData
	 */
	public InspectionData get_details() {
		InspectionData	details = new InspectionData();

		boolean r = false, rr = false;
		try {
			if ((r = take(true, true, 0)) == true) {
				InspectionData	id = new InspectionData();
				id.add("", name);
				id.add("CX^X", toString());
				id.add("Z}tH", sem.toString());
				id.add("vҎ󂯃L[", req_queue.toString());
				id.add("Oo͐", log.toString());
				id.add("Oo͐pX", log_conf.path);
				id.add("Oo͕GR[fBO"
										, log_conf.encoding);
				id.add("OEx"
								, Integer.toString(log_conf.level));
				id.add("OEtB[h؂蕶", log_conf.delimit);
				id.add("FACB (ShredObjects)", facbsos.toString());
				id.add("FACB (Arrays)", facbs.toString());
				id.add("FACB ", Integer.toString(facbs.length));

				details.add(Integer.toString(inspector_serial_number
											, 10)
							, id
							);

				rr = leave();
			}
			if (!r || !rr) {
				throw new Exception(); /* goto catch-block */
			}
		} catch (Exception e) {
			details.add("@\"
					, "!!bN" + ((!r)? "l": "") + "Ɏs!!"
					);
		}

		return	details;
	}

	/**
	 * <em>G[WFg</em>̏ڍ׏ InspectionData ƂĕԂ܂B
	 * Inspector Ăяo܂B
	 *
	 * @return ڍ׏ݒ肳ꂽ InspectionData
	 */
	protected InspectionData get_agents_details() {
		InspectionData	details = new InspectionData();

		boolean r = false, rr = false;
		try {
			if ((r = take(true, true, 0)) == true) {
				for (int i = 0; i < facbsos.length; ++i) {
					InspectionData	id = new InspectionData();
					boolean rso = false, rrso = false;
					if ((rso = facbsos[i].take(true, true, 0))
															== true) {
						id.add("",	facbs[i].agname);
						id.add("ID"
							, Integer.toString(facbs[i].agent.getAid()
												, 10)
							);
						id.add("CX^X"
								, facbs[i].agent.toString());
						id.add("Xbh", facbs[i].thr.toString());
						id.add("L["
								, facbs[i].agent.resq.toString());
						id.add("L["
							, Integer.toString(facbs[i].resq_len, 10)
							);
						int status = facbs[i].status;
						id.add(""
								, ((status == facbs[i].NONE)?  "NONE":
									(status == facbs[i].IDLE)? "IDLE":
									(status == facbs[i].INIT)? "INIT":
									(status == facbs[i].SUSP)? "SUSP":
									(status == facbs[i].RESU)? "RESU":
									(status == facbs[i].WAIT)? "WAIT":
									(status == facbs[i].PROC)? "PROC":
									(status == facbs[i].TERM)? "TERM":
									(status == facbs[i].ZOMB)? "ZOMB":
									/* unknown status */	"UNKNOWN")
							);
						Object wait_channel =
										facbs[i].agent.wait_channel;
						if (wait_channel != null) {
							id.add("o̓IuWFNg"
									, wait_channel.toString()
								);
						}
						id.add("݃tO"
							, Integer.toString(facbs[i].interrupt_cause
												, 16)
							);
						id.add("Mς݋@\R}h"
							, facbs[i].agent.last_command.toString()
							);
						for (int j = 0
						; j < facbs[i].agent.fatal_exception_cnt
						; ++j) {
							id.add("~G[R[h"
								, Integer.toString(
											facbs[i].agent.halt_cause[j]
											, 10)
								);
							Exception e =
									facbs[i].agent.fatal_exception[j];
							id.add("~O"
								, ((e != null)?
										Utillity.format_exception_info(
												null, e).toString():
										"(none)")
								);
						}

						rrso = facbsos[i].leave();
					}
					if (!rso || !rrso) {
						/* goto catch-block */
						throw new Exception("G[WFg");
					}

					details.add(Integer.toString(i, 10), id);
				}

				rr = leave();
			}
			if (!r || !rr) {
				throw new Exception("@\"); /* goto catch-block */
			}
		} catch (Exception e) {
			details.add(e.getMessage()
					, "!!bN" + ((!r)? "l": "") + "Ɏs!!"
					);
		}

		return	details;
	}

	// Facility instance will be shared between 1) itself,
	// 2)FacilityAgent and 3) the application.
	static final private int MAX_SHARERS = 3;
	private Semaphore sem = null;

	// Facility name and instance dictionary
    static private Hashtable facs = new Hashtable();

	// FacilityAgentControlBlock-s are shared with the FacilityAgent-s
	private SharedObject[] facbsos = null;
	private FacilityAgentControlBlock[] facbs = null;

	// Application that owns this facility
	private Application app = null;
	/**
	 * <em>@\</em>L<em>AvP[V</em>ݒ肵܂
	 *
	 * @param application <em>AvP[V</em>
	 */
	protected void set_app(Application application) {
		synchronized (this) { app	= application; }
	}

	/**
	 * <em>@\</em>L<em>AvP[V</em>̃^C}
	 * 擾܂
	 *
	 * @return <ul><li> null -- <em>AvP[V</em>̃^C}
	 *                            Ԃ܂</li
	 *            ><li>null -- <em>AvP[V</em>^C}@\
	 *                         񋟂ĂȂꍇ</li
	 *            ></ul>
	 */
	protected Timer get_app_timer() {
		return	app.get_app_timer();
	}

	/** <em>@\</em> */
	protected String name	= null;
	/** <em>@\</em>̏ FacilityAgent NX̃NX */
	protected String agclass_name = null;
	/** ssősxB1 ȏłKv */
	protected int max_concurrency	= 0;
	/**
	 * ݂̕sxBFacilityAgent ŏQ/G[ꍇA
	 * ۂɊĂXbh́A̒l菭ȂȂ 
	 * (ĂXbh 0 ƂƂ)
	 */
	protected int concurrency	= 0;

	/** <em>@\</em>̗v҂󂯃L[BFacilityCommand n */
	protected Message req_queue	= null;

	/** <em>@\</em>̃Oo͐ */
	protected Log log	= null;

	/** <em>@\</em>̃Oo͐\ */
	protected LogConf log_conf = null;

	/** <em>@\</em>/񊈐̏Ԃ */
	protected boolean active = false;
	
	// RXgN^: CX^X𒼐ڐ邱Ƃ͂łȂB
	// create() static \bhgp */
	private Facility() {
		sem = new Semaphore(MAX_SHARERS);
	}

	// Creating FacilityAgent instances
	// @param f -- the Facility instance that controls this agent
	// @param resq_len -- the length of responce-queue
	static private void create_facs(Facility f, int resq_len)
												throws Exception {
		String	agname;
		boolean	r, rr = false;

		// create FacilityAgentControlBlock-s
		f.facbsos	= new SharedObject[f.max_concurrency];
		f.facbs	= new FacilityAgentControlBlock[f.max_concurrency];

		// create FacilityAgent-s
		for (int i = 0; i < f.max_concurrency; ++i) {
			// FacilityAgent name is "<Facility-nae>$<agent-ID>"
			agname	= f.name + "$" + i;

			// create a SharedObject for the FacilityAgent
			f.facbsos[i]	= SharedObject.create(agname
								, "my_util.FacilityAgentControlBlock"
								, resq_len
								);

			// initialize the FacilityAgent
			// lock the FacilityAgent (monopolized, infinite-blocking)
			if ((r = f.facbsos[i].take(false, true, 0)) == true) {
				f.facbs[i] = (FacilityAgentControlBlock)(
										f.facbsos[i].get_object());
				f.facbs[i].fac	= f;
				f.facbs[i].agname	= agname;
				f.facbs[i].agid	= i;
				f.facbs[i].resq_len	= resq_len;

				rr = f.facbsos[i].leave(); // unlock the FacilityAgent
			}
			if (r == false || rr == false) {
				// failed in locking or unlocking the FacilityAgent
				throw new Exception("Faclity.create_facs(" + f + ", "
									+ resq_len + "): failed in facbsos["
									+ i + "]." + ((!r)? "take": "leave")
									+ "()"
									);
			}
		}
	}

	/**
	 * <em>@\</em>̐s܂BꂽCX^X́A
	 * <em>@\</em>̂L[Ƃēo^܂B
	 *
	 * @param name <em>@\</em>
	 * @param agclass_name <em>@\</em>̏ FacilityAgent
	 *                     NX̃NX
	 * @param max_concurrency <em>@\</em>̍ősx
	 * @param concurrency <em>@\</em>̕sx̏l
	 * @param log_conf <em>@\</em>̃Oo͐\B
	 *                 my_util.LogConf QƂ̂
	 * @param resq_len FacilityAgent ւ̉L[̃L[
	 * @return <ul><li> null --  Facility CX^X</li
	 *            ><li>null -- sȈAvL[sA
	 *                         Oo͐I[vɎs̏ꍇ</li
	 *            ></ul>
	 * @exception Exception FacilityAgent ̐Ɏsꍇ
	 */
	static public Facility create(String name
								, String agclass_name
								, int max_concurrency
								, int concurrency
								, LogConf log_conf
								, int resq_len
								) throws Exception {
		Facility fac = null;

		// check parameters
		if ((((((((((((name == null)
		|| agclass_name == null)
		|| agclass_name.compareTo("") == 0)
		|| max_concurrency < 1)
		|| concurrency < 0)
		|| concurrency > max_concurrency)
		|| log_conf == null)
		|| log_conf.delimit == null)
		|| log_conf.delimit.equals(""))
		|| log_conf.level < Log.FATAL)
		|| log_conf.level > Log.DEBUG)
		|| resq_len	< 0) {
			return	fac; // return null: illegal parameter
		}

		// create reuest-queue
		Message reqq = Message.create(Thread.currentThread()
									, name
									, max_concurrency
									);
		if (reqq == null) {
			return	fac; // return null: failed in create request-queue
		}

		// create logging output steram
		Log log = new Log(log_conf);
		if (!log.open()) {
			// destroy the request-queue
			reqq.destroy(Thread.currentThread(), name);
			return	fac; // return null: failed in create logging stream
		}

		// create a new instance, or search the instance in Hashtable
		synchronized (facs) {
			fac	= (Facility)(facs.get(name));
			if (fac == null) {
				// the faility-name is new one. create instance
				fac	= new Facility();
				fac.name	= name;
				fac.agclass_name	= agclass_name;
				fac.max_concurrency	= max_concurrency;
				fac.concurrency	= concurrency;
				fac.req_queue	= reqq;
				fac.log	= log;
				fac.log_conf	= log_conf;

				try {
					// create FacilityAgentControlBlock-s
					// those are controled from this
					create_facs(fac, resq_len);
				} catch (Exception e) {
					// failed in creating FacilityAgentControlBlock-s
					// destroy the request-queue, and close the logging
					// output stream
					reqq.destroy(Thread.currentThread(), name);
					log.close();

					throw	e;
				}

				// register the faility name and instance to Hashtable
				facs.put(name, fac);
			}
		} // synchronized (facs)

		return	fac; // returns the instance
	}

	/**
	 *  {@link #create} \bhŐꂽ<em>@\</em
	 * >CX^XA<em>@\</em>̂Ō܂B
	 *
	 * @param name <em>@\</em>
	 * @return <ul><li> null -- <em>@\</em>̂ɑΉ
	 *                            CX^X</li
	 *            ><li>null -- <em>@\</em>̂ null 邢́A
	 *                         <em>@\</em>̐ĂȂꍇ</li
	 *            ></ul>
	 */
	static public Facility attach(String name) {
		Facility	fac = null;

		// check the parameters
		if (name == null) {
			return	fac; // return null: illegal parameter
		}

		synchronized (facs) {
			// search the facility by facility-name in Hashtable
			fac = (Facility)(facs.get(name));
		}

		return	fac; // returns null: not-found the instance
					 // returns not-null: the registered instance
	}

	/**
	 * <em>@\</em>CX^XbN܂B
	 * bN@\ɂ Semaphore gpĂ܂B
	 *
	 * @param sharing -- LbN̏ꍇ true A
	 *                   LbN̏ꍇ false w肵܂B
	 * @param blocking -- bNlɃubLO邩ۂw
	 *                    ܂B
	 * @param to -- ubLOꍇ̃^CAEgԂ~bx
	 *              w肵܂BɃubN΂́A0 w
	 *              ܂B
	 * @return <ul><li>true -- bNlɐꍇ</li
	 *            ><li>false -- bNlɎsꍇ</li
	 *            ></ul>
	 * @exception TimeoutException -- ^CAEgꍇ
	 * @exception Exception -- LȊO̗OBSemaphore QƂ̂ƁB
	 */
	protected boolean take(boolean sharing, boolean blocking, long to)
										throws TimeoutException
												, Exception {
		return	sem.take(Thread.currentThread(), sharing, blocking, to);
	}

	/**
	 * {@link #take} \bhŊlꂽ<em>@\</em>CX^X
	 * bNJ܂B
	 * bN@\ɂ Semaphore gpĂ܂B
	 *
	 * @return <ul><li>true -- bNJɐꍇ</li
	 *            ><li>false -- bNJɎsꍇ</li
	 *            ></ul>
	 * @exception Exception -- Semaphore QƂ̂ƁB
	 */
	protected boolean leave() {
		return	sem.leave(Thread.currentThread());
	}

	// Waiting-loop for the all agents translated status to SUSP
	private void wait_for_all_agents_suspended() throws Exception {
		int	suspended_cnt = 0;

		// wait for the all agents gone to suspend...
		while (suspended_cnt != max_concurrency) {
			Thread.sleep(100); // rest of a breath

			// count up the agents those gone to suspend...
			suspended_cnt	= 0;
			for (int i = 0; i < max_concurrency; ++i) {
				// lock the FACB (sharing, non-blocking)
				boolean	r = false, rr = false;
				if ((r = facbsos[i].take(true, false, 0)) == true) {
					// If the agent had gone to suspend,
					// increase the counter
					if (facbs[i].status == facbs[i].SUSP) {
						++suspended_cnt;
					}

					rr = facbsos[i].leave(); // unlock the FACB
				}
				if (!rr) {
					// Failed to unlock the FACB!
					throw new Exception("Facility."
								+ "wait_for_all_agents_suspended(): "
								+ "facbsos[" + i + "].leave() "
								+ "returns false."
								);
				}
				if (!r) {
					// Failed to lock the FACB,
					// ignore this case and retry locking later
					--i;
					Thread.sleep(100);
				}
			}
		}
	}

	// Waiting-loop for the FacilityAgents-s finished translate those
	// status.
	// @param min_status -- the least number of the destiny status
	// @param agent_cnt -- the number of translated thread
	private void wait_for_agents_trans_status(int min_status
											, int agent_cnt
											) throws Exception {
		int	cnt = 0;

		// check the parameters
		if (agent_cnt == 0) {
			return; // Can't wait no agents
		}

		// wait for that agents did translate those status...
		while (cnt != agent_cnt) {
			Thread.sleep(100); // rest for a breath

			// count up the agents those had translated the status...
			cnt	= 0;
			for (int i = 0; i < max_concurrency; ++i) {
				// lock the FACB (sharing, non-blocking)
				boolean	r = false, rr = false;
				if ((r = facbsos[i].take(true, false, 0)) == true) {
					// If the agent translated the status,
					// increment the counter.
					if (facbs[i].status >= min_status) {
						++cnt;
					}

					rr = facbsos[i].leave(); // unlock the FACB
				}
				if (!rr) {
					// Failed to unlock the FACB!
					throw new Exception("Facility."
								+ "wait_for_agents_trans_status("
								+ min_status + ", " + agent_cnt
								+ "): facbsos[" + i
								+ "].leave() returns false."
								);
				}
				if (!r) {
					// If locking the FACB is failed,
					// ignore this case and retry locking later...
					--i;
					Thread.sleep(100);
				}
			}
		}
	}
	
	// Waiting-loop for changing concurrency
	private void wait_changed_concurrency() throws Exception {
		if (concurrency == 0) {
			// concurreny is zero. all agents gone to SUSP...
			wait_for_all_agents_suspended();
		}
		else {
			// conurrency is greater than zero,
			// the agents gone WAIT, PROC, TERM or ZOMB...
			wait_for_agents_trans_status(FacilityAgentControlBlock.WAIT
										, concurrency
										);
		}
	}

		
	// Stop all agents
	// (set the interrupt_cause field of FACB,
	//  and then send interrupts to the thread)
	// @param forced -- 'true' to force stopping agents,
	//                  'false' to hang up process to stop all agents
	private void stop_agent_instances(boolean forced)
										throws Exception {
		boolean	r = false, rr;
		int	i;

		// loop for all agents...
		for (i = 0; i < max_concurrency; ++i) {
			try {
				// lock the FACB (monopolize, blocking infinite)
				r	= rr = false;
				if ((r = facbsos[i].take(false, true, 0)) == true) {
					// If the alive thread status is IDLE, INIT, SUSP,
					// RESU, WAIT or PROC, then set interrupt-cause to
					// TERMINATE and send an interrupt to the thread.
					if (facbs[i].thr != null) {
						if ((facbs[i].status > facbs[i].NONE)
						&& facbs[i].status < facbs[i].TERM) {
							facbs[i].interrupt_cause |=
												facbs[i].TERMINATE;
							facbs[i].thr.interrupt();
						}
					}

					rr = facbsos[i].leave(); // unlock the FACB
				}
				if (!r || !rr) {
					// Failed to lock or unlock the FACB!
					if (!forced) {
						// 'forced' isn't specified.
						// hang up the process to stop all agents!
						throw new Exception("Facility."
									+ "stop_agent_instances(" + forced
									+ "): facbsos[" + i + "]."
									+ ((!r)? "take": "leave")
									+ "returns false."
									);
					}
				}
			} catch (Exception e) {
				// Occured exception! if the FACB was locked, unlock it
				if (r) facbsos[i].leave();
				if (!forced) throw e;
			}
		}
	}

	// Join all agents those stopped
	// @param forced -- 'true' to force joining agents,
	//                  'false' to hang up process to join all agents
	// @param locked_facility -- 'true' means the facility was locked.
	//                           'false' means the facility isn't locked
	private void join_agent_instances(boolean forced
									, boolean locked_facility
									) throws Exception {
		boolean	r, rr;

		// loop for all agents...
		for (int i = 0; i < max_concurrency; ++i) {
			// lock this facility instance (it isn't locked)
			r = rr = false;
			if ((locked_facility == true)
			|| (r = take(true, true, 0)) == true) {
				// join the alive thread
				if (facbs[i].thr != null) {
					try {
						facbs[i].thr.join();
					} catch (Exception e) {
						// Occured exception!
						// if 'forced' is specified, retry to join it.
						// otherwise, throw the exception (need unlock)
						--i;
						if (!forced) {
							if (!locked_facility) {
								rr = leave();
							}
							throw e;
						}
					}
				}

				if (!locked_facility) rr = leave(); // unlock this
			}
			if (!locked_facility && (!r || !rr)) {
				// Failed to lock or unlock this facility instance!
				throw new Exception("Facility.join_agent_instances("
									+ forced + ", " + locked_facility
									+ "): " + i + "-th "
									+ ((!r)? "take": "leave") + "() "
									+ "returns false."
									);
			}
		}
	}

	// delete all FACB entries
	private void delete_agent_instances() {
		// loop for all FACB-s...
		for (int i = 0;i < max_concurrency; ++i) {
			// thread handle is set null.
			if (facbs[i].thr != null) {
				//stop() is "deprecation". // facbs[i].thr.stop();
				facbs[i].thr	= null;
			}

			// FacilityAgent is set to null,
			// and delete the responce-queue
			if (facbs[i].agent != null) {
				facbs[i].agent.destroy_resq();
				facbs[i].agent	= null;
			}

			// status is set NONE
			facbs[i].status	= facbs[i].NONE;
		}
	}

	/**
	 * <em>@\</em>܂B݂̕sx {@link #concurrency} 
	 * v FacilityAgent CX^X̃Xbh̎sJn܂B
	 *
	 * @return <ul><li>true -- ɐꍇ</li
	 *            ><li>false -- Ɏsꍇ</li
	 *            ></ul>
	 * @exception Exception -- Facility CX^X邢
	 *                         FacilityAgentControlBlock CX^X
	 *                         bNlEJɎsꍇ
	 */
	public boolean activate() throws Exception {
		boolean r = false, rr, result = false;;
		Class	agclass;
		FacilityAgent agent;
		Thread	thr;

		// lock this facility instance (monopolize, blocking infinite)
		r = rr = false;
		if ((r = take(false, true, 0)) == true) {
			if (active) {
				// already, this facility was activated.
				leave(); // unlock the facility
				return	true;
			}

			if (agclass_name == null
			&& agclass_name.compareTo("") == 0) {
				// the FacilityAgent class name isn't specified!
				leave(); // unlock the facility
				return	false;
			}
			agclass	= Class.forName(agclass_name);
			if (!FacilityAgent.class.isAssignableFrom(agclass)) {
				// the agent class isn't sub-class of FacilityAgent!
				leave(); // unlock the facility
				throw new Exception("Facility.activate(): "
									+ agclass_name
									+ " isn't sub-class of "
									+ FacilityAgent.class.getName()
									);
			}

			// create agent instances and those threads
			// (FACB status: NONE --> IDLE)
			for (int i = 0; i < max_concurrency; ++i) {
				try {
					// lock the FACB (monopolize, blocking infinite)
					r	= rr = false;
					if ((r = facbsos[i].take(false, true, 0)) == true) {
						// create instance of the agent class,
						// and initialize it
						agent = (FacilityAgent)(agclass.newInstance());
						agent.setFacname(name);
						agent.setAid(facbs[i].agid);
						agent.setAgname(facbs[i].agname);
						agent.create_resq(facbs[i].resq_len);
						facbs[i].agent	= agent;

						// create thread for running the agent
						thr	= new Thread(agent);
						facbs[i].thr	= thr;
						facbs[i].status	= facbs[i].IDLE;

						rr = facbsos[i].leave(); // unlock the FACB
					}
					if (!r || !rr) {
						// Failed to lock or unlock the FACB!
						// delete all agents those are created, yet
						delete_agent_instances();
						leave(); // unlock the facility
						throw new Exception("Facility.activate(): "
											+ "facbsos[" + i + "]."
											+ ((!r)? "take": "leave")
											+ "() returns false."
											);
					}
				} catch (Exception e) {
					// Occured exception! unlock the FACB, delete all
					// agents those are created, yet.
					if (r) facbsos[i].leave();
					delete_agent_instances();
					leave(); // unlock the facility
					throw	e;
				}
			}

			// start agent threads
			// (FACB status: IDLE --> INIT)
			for (int i = 0; i < max_concurrency; ++i) {
				try {
					// lock the FACB (monopolize, blocking infinite)
					r	= rr = false;
					if ((r = facbsos[i].take(false, true, 0)) == true) {
						// set the FACB status to INIT,
						// and start the thread
						facbs[i].status	= facbs[i].INIT;
						facbs[i].thr.start();

						rr = facbsos[i].leave(); // unlock the FACB
					}
					if (!r || !rr) {
						// Failed to lock or unlock the FACB!
						// stop, join and delete all agents those are
						// created and started, yet.
						stop_agent_instances(true);
						join_agent_instances(true, true);
						delete_agent_instances();
						leave(); // unlock the facility
						throw new Exception("Facility.activate(): "
											+ "facbsos[" + i + "]."
											+ ((!r)? "take": "leave")
											+ "() returns false."
											);
					}
				} catch (Exception e) {
					// Occured exception!
					// unlock the FACB.
					// stop, join and delete all agents those are
					// created and started, yet.
					if (r) facbsos[i].leave();
					stop_agent_instances(true);
					join_agent_instances(true, true);
					delete_agent_instances();
					leave(); // unlock the facility
					throw	e;
				}
			}

			leave(); // unlock the facility
		}
		if (r && rr) {
			// Success to create and start the agetns.
			// set the fasility status to active.
			// wait for the started agents translates status...
			active	= true;
			wait_for_agents_trans_status(FacilityAgentControlBlock.SUSP
										, max_concurrency
										);
			result	= true; // the agents gone running
		}

		return	result; // 'true' means success, otherwise, means failed
	}

	/**
	 * <em>@\</em>񊈐܂B݂̕sx {@link #concurrency}
	 * Ɉv FacilityAgent CX^X̃Xbh~邽߂
	 * ݂𑗐M܂B
	 *
	 * @return <ul><li>true -- 񊈐ɐꍇ</li
	 *            ><li>false -- 񊈐Ɏsꍇ</li
	 *            ></ul>
	 * @exception Exception -- Facility CX^X邢
	 *                         FacilityAgentControlBlock CX^X
	 *                         bNlEJɎsꍇ
	 */
	public boolean deactivate() throws Exception {
		if (active) {
			boolean r, rr;
			
			// stop all running agents...
			// lock this facility (monopolize, blocking infinite)
			r = rr = false;
			if ((r = take(false, true, 0)) == true) {
				stop_agent_instances(false);

				rr = leave(); // unlock this facility
			}
			if (!r || !rr) {
				// Failed to lock or unlock this facility!
				throw new Exception("Facility.deactivate(): "
									+ ((!r)? "take": "leave") + "() "
									+ "to invoke stop_agent_instances()"
									+ " , returns false."
									);
			}

			// join all the stopped agents...
			join_agent_instances(false, false);

			// delete all agents...
			// lock this facility (monopolize, blocking infinite)
			r = rr = false;
			if ((r = take(false, true, 0)) == true) {
				delete_agent_instances();

				rr = leave(); // unlock this facility
			}
			if (!r || !rr) {
				// Failed to lock or unlock this facility!
				throw new Exception("Facility.deactivate(): "
									+ ((!r)? "take": "leave") + "() "
									+ "to invoke delete_agent_inctances"
									+ "(), returns false."
									);
			}

			// set this facility status to inactive
			active	= false;
		}

		return	true; // success to deactivate this facility
	}

	/**
	 * <em>@\</em>̕sx {@link #concurrency} ύX܂B
	 * sxꍇ̓TXyh݂A
	 * sxグꍇ̓W[݂
	 * K؂Ȑ FacilityAgen Xbh֑M܂B
	 *
	 * @param concurrency -- ύX̕sx
	 * @return <ul><li>true -- sx̕ύXɐꍇ</li
	 *            ><li>false -- sx̕ύXɎsꍇ</li
	 *            ></ul>
	 * @exception Exception -- Facility CX^X邢
	 *                         FacilityAgentControlBlock CX^X
	 *                         bNlEJɎsꍇ
	 */
	public boolean change_concurrency(int concurrency)
											throws Exception {
		boolean	result	= false, r, rr;

		// lock this facility (monopolize, blocking infinite)
		r = rr = false;
		if ((r = take(false, true, 0)) == true) {
			// check parameters
			if (concurrency < 0 || concurrency > max_concurrency) {
				rr	= leave(); // unlock this facility
				return	 false; // return false: illegal parameter
			}

			// check this facility status and current concurrency
			// if status was inactive or the concurrency is such as
			// the current it, this method works nothing.
			if ((!active)
			|| this.concurrency == concurrency) {
				this.concurrency	= concurrency;
				rr	= leave(); // ulock this facility
				return	true; // success: worked nothing
			}

			// get the number of interrupting agents,
			// decide to send the suspend/resume interrupt,
			// and set the curent concurrency to specified degree
			int	change_cnt = concurrency - this.concurrency;
			boolean	do_suspend = (change_cnt < 0);
			this.concurrency	= concurrency;

			// loop for interrupting agents...
			for (int i = 0
			; i < max_concurrency && change_cnt != 0
			; ++i) {
				try {
					// lock the FACB (monopolize, blocking infinite)
					r	= rr = false;
					if ((r = facbsos[i].take(false, true, 0)) == true) {
						if (do_suspend) {
							// if the agent status is IDLE, WAIT or
							// PROC, set SUSPEND-bit interrupt-cause
							// and send the interrupt to the IDLE thread
							if (facbs[i].status == facbs[i].IDLE
							|| facbs[i].status == facbs[i].WAIT
							|| facbs[i].status == facbs[i].PROC) {
								facbs[i].interrupt_cause
													|= facbs[i].SUSPEND;
								if (facbs[i].status != facbs[i].IDLE) {
									facbs[i].thr.interrupt();
								}
								++change_cnt;
							}
						}
						else {
							// if the agent status is SUSP,
							// set RESUME-bit interrupt-cause
							// and send the interrupt to the SUSP thread
							if (facbs[i].status == facbs[i].SUSP) {
								facbs[i].interrupt_cause
													|= facbs[i].RESUME;
								facbs[i].thr.interrupt();
							}
							--change_cnt;
						}

						rr = facbsos[i].leave(); // unlock the FACB
					}
					if (!r || !rr) {
						// Failed to lock or unlock the FACB!
						rr	= leave(); // unlock this facility
						throw new Exception("Facility."
											+ "change_concurrency("
											+ concurrency + "): "
											+ "facbsos[" + i + "]."
											+ ((!r)? "take" : "leave")
											+ "() returns false."
											);
					}
				} catch (Exception e) {
					// Occured exception!
					if (r) facbsos[i].leave(); // unlock the FACB
					rr	= leave(); // unlock this facility
					throw e;
				}
			}

			rr = leave(); // unlock this facility
		}
		if (r && rr) {
			// Success to send the SUSPEND/RESUME interrupt to threads!
			// wait for the agents translates status to SUSP or RESU...
			wait_changed_concurrency();
			result	= true; // success changing the current concurency
		}

		return	result; // 'true' means sucesss, otherwise, means failed
	}

	/**
	 * <em>@\</em>̃OL^܂B
	 * Loggable C^tF[Xv܂B
	 */
	 public void log(int log_class, String msg) {
		boolean	r, rr, do_logging = false;

		try {
			// lock this facility (sharing, blocking infinite)
			r = rr = false;
			if ((r = take(true, true, 0)) == true) {
				if (log != null && log_class <= log_conf.level) {
					do_logging	= true;
				}

				rr = leave(); // unlock this facility
			}
			if (r && rr && do_logging) {
				log.print(log_class, msg);
			}
		} catch (Exception e) {
			; // NEVER THROW EXCEPTION
		}
	}

	/**
	 * Oo͂̃tB[h؂蕶擾܂B
	 * @return <ul><li> null -- ؂蕶</li
	 *            ><li>null -- ؂蕶ݒ肳ĂȂꍇ</li
	 * @exception Exception <em>@\</em>̃bNlɎsꍇ
	 */
	protected String get_log_delimit() throws Exception {
		String	delimit = null;

		// lock this facility (sharing, blocking infinite)
		if (take(true, true, 0)) {
			delimit	= log_conf.delimit;

			leave();
		}
		
		return	delimit;
	}

	/**
	 * <em>@\</em>̃OL^o͐EGR[fBOE
	 * OExύX܂BLoggable C^tF[Xv
	 * ܂B
	 *
	 * @param log_conf Oo͐\Bmy_util.LogConf QƂ̂
	 * @return <ul><li>true -- Oo͐Ȃǂ̕ύXɐꍇ</li
	 *            ><li>false -- ύXɎsꍇ</li
	 *            ></ul>
	 * @exception Exception Facility CX^X̃bNlɎs
	 *                      ꍇ
	 */
	public boolean change_log(LogConf log_conf) throws Exception {
		boolean	r, rr, rrr, result = false;

		// lock this facility (monopolize, blocking infinite)
		r = rr = false;
		if ((r = take(false, true, 0)) == true) {
			// change the path and level and...etc
			rrr = true;
			if (log == null) {
				log = new Log(log_conf);
			}
			else {
				rrr = log.change(log_conf);
			}
			if (log != null && rrr == true) {
				// set the fields of this facility
				this.log_conf	= log_conf;
				result	= true; // success changing the logging destiny
			}

			rr = leave(); // unlock this facility
		}
		if (!r || !rr) {
			// Failed to lock or unlock this facility!
			throw new Exception("Facility.change_log(" + log_conf
									+ "): this."
									+ ((!r)? "take": "leave")
									+ "() returns false."
									);
		}

		return	result; // 'true' means success, otherwise, mewns failed
	}

	/**
	 * <em>@\</em>ɗv𑗂邽߂ɗvMĂяo܂B
	 *
	 * @param cmd veƉeێ FacilityCommand 
	 *            CX^X
	 * @param timeout ^CAEgԂ~bxŎw肵܂
	 * @param insert_head cmd <em>@\</em>̗vL[̐擪ɑ}
	 *                    邩ۂw肵܂B
	 * @return <ul><li>true -- v̑Mꍇ</li
	 *            ><li>false -- v̑Msꍇ</li
	 *            ></ul>
	 * @exception InactiveFacilityException <em>@\</em>񊈐̏ꍇ
	 * @exception TimeoutException ^CAEgꍇ
	 * @exception Exception LȊO̗O
	 */
	public boolean put_request(Object cmd
							, long timeout
							, boolean insert_head
							) throws InactiveFacilityException
	 									, TimeoutException
	 									, Exception {
		long start_millis = System.currentTimeMillis(), tto = timeout;
		boolean r, rr,result = false;
		boolean	this_is_active = false;
		FacilityCommand	req	= (FacilityCommand)cmd;

		// lock this facility (sharing, blocking until timeout)
		r = rr = false;
		tto = Utillity.check_timeout(start_millis, timeout);
		if ((r = take(true, true, tto)) == true) {
			// get the current active status
			this_is_active = active;

			rr = leave(); // unlock this facility
		}
		if (!r || !rr) {
			// Failed to lock or unlock this facility!
			throw new Exception("Facility.put_request("
								+ cmd + ", " + timeout + ", "
								+ insert_head + "): this."
								+ ((!r)? "take": "leave")
								+ "() returns false."
								);
		}

		// check this facility is active?
		if (!this_is_active) {
			// The destiny afcility isn't activated!
			throw new InactiveFacilityException(
								"Facility.put_request("
								+ cmd + ", " + timeout + ", "
								+ insert_head + "): this facility "
								+ "is inactive."
								);
		}

		// try to put the commad to this facility request queue...
		tto	= Utillity.check_timeout(start_millis, timeout);
		result = req_queue.put(cmd, insert_head, true, tto);

		// if this facility puts the command completelly,
		// set the request command status to IN_QUEUE
		if (result) {
			req.setStatus(req.IN_QUEUE);
		}

		return	result; // returns true: sent the command to the desyiny
						// returns false: failed to send the command
	}
}
