/***********************************************************************
 * File: FacilityCommand.java
 * 	(Extracted from
 *	   "$Id: FacilityCommand.java,v 1.6 2004/06/20 10:50:21 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a FacilityCommand utility abstract
 * class.  This class is a command for requesting and responcing to the
 * "Facility" class.
 *  (Refered Class: Facility, FacilityAgent)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package	my_util;

import java.io.Serializable;

/**
 * <p>Facility ԂőMR}hƂ̉`邽߂
 *    ۃNXłBFacilityAgent ͗vR}h process() \bh
 *    ʂĎMȀʂ̃R}h̉
 *    tB[hɐݒ肵 responce() \bhɂ著M܂B</p>
 *
 * @author maoyam
 * @version 1.0
 * @see Facility
 * @see FacilityAgent
 */
public abstract class FacilityCommand implements Serializable {

	/** R}h: ŗvL[ɑMĂȂ */
	static final public int INIT				= 0;
	/** R}h: vL[ɑMꂽ */
	static final public int IN_QUEUE			= 1;
	/**
	 * R}h:
	 * FacilityAgent ɂMꏈĂ
	 */
	static final public int PROCESSING			= 2;
	/**
	 * R}h:
	 * FacilityAgent ɂ菈ʉMꂽ
	 */
	static final public int RESPONCED			= 3;
	/**
	 * R}h:
	 * FacilityAgent ̃R}hŗOjꂽ
	 */
	static final public int OCCURED_EXCEPTION	= 4;
	/**
	 * R}h:
	 * FacilityAgent ̃R}hُIjꂽ
	 */
	static final public int ABNORMAL_ENDED		= 5;
	// output: status of processing this command
	private int status	= INIT;
	/**
	 * R}h status 擾
	 * @return R}h󋵂Ԃ
	 */
	public int getStatus() { synchronized (this) { return status; } }
	/**
	 * R}h status ݒ肷
	 * @param status ݒ肷R}h
	 */
	public void setStatus(int status) {
		synchronized (this) { this.status = status; }
	}

	// output: the name of the agent that processes this command
	private String processing_agent = null;
	/**
	 * vM FacilityAgent ̂擾
	 * @return vMĂ FacilityAgent ̖
	 */
	public String getProcessing_agent() {
		synchronized (this) { return processing_agent; }
	}
	/**
	 * vM FacilityAgent ̂ݒ肷
	 * @param agent_name FacilitiAgent ̖
	 */
	public void setProcessing_agent(String agent_name) {
		synchronized (this) { processing_agent	= agent_name; }
	}

	// output: the class name of the exception that
	//        the processing agent thrown
	private String exception_class_name = null;
	/**
	 * vM FacilityAgent  throw ÕNX
	 * 擾
	 * @return j錴ƂȂÕNX
	 */
	public String getException_class_name() {
		synchronized (this) { return exception_class_name; }
	}
	/**
	 * vM FacilityAgent  throw ÕNX
	 * ݒ肷
	 * @param exception_class_name j錴ƂȂO
	 *                             NX
	 */
	public void setException_class_name(String exception_class_name) {
		synchronized (this) {
			this.exception_class_name	= exception_class_name;
		}
	}

	// output: if it's a positive number or zero:
	//            the return value of the FacilityAgent.process()
	//         if it's a negative number:
	//            the internal error code of the FacilityAgent framework
	private int		halt_cause = 0; // See Also: FacilityAgent
	/**
	 * M FacilityAgent ُIۂ̃^[ER[h
	 * 擾
	 * @return ُIۂ FacilityAgent.process() ̖߂l
	 */
	public int getHalt_cause() {
		synchronized (this) { return halt_cause; }
	}
	/**
	 * M FacilityAgent ُIۂ̃^[ER[h
	 * ݒ肷
	 * @param halt_cause ُIۂ FacilityAgent.process()
	 *                   ̖߂l
	 */
	public void setHalt_cause(int halt_cause) {
		synchronized (this) { this.halt_cause	= halt_cause; }
	}

	// input: the name of the FacilityAgent that sent thhis command
    private String	fac_name = null;
	/**
	 * vR}h𑗐M FacilityAgent ̑<em>@\</em>̂
	 * 擾
	 * @return vR}h𑗐M FacilityAgent <em>@\</em>
	 */
    public String getFac_name() {
		synchronized (this) { return fac_name; }
	}
	/**
	 * vR}h𑗐M FacilityAgent ̑<em>@\</em>̂
	 * ݒ肷
	 * @param name vR}h𑗐M FacilityAgent <em>@\</em>
	 *             
	 */
    public void setFac_name(String name) {
		synchronized (this) { fac_name = name; }
	}

	// input: the id number of the FacilityAgent that sent this command
    private int	aid = -1;
	/**
	 * vR}h𑗐M FacilityAgent <em>G[WFg</em> ID
	 * 擾
	 * @return vR}h𑗐M FacilityAgent 
	 *         <em>G[WFg</em> ID
	 */
    public int getAid() { synchronized (this) { return aid; } }
	/**
	 * vR}h𑗐M FacilityAgent <em>G[WFg</em> ID
	 * ݒ肷
	 * @param id vR}h𑗐M FacilityAgent 
	 *           <em>G[WFg</em> ID
	 */
    public void setAid(int id) { synchronized (this) { aid = id; } }

	// input: if it's true:
	//           the processing FacilityAgent must send to the responce
	//          queue owned by the "requester" agent
	//        otherwise:
	//           the processing FacilityAgent must send to the queue
	//          owned by the non-agent requester (maybe Application)
	private boolean resp_to_agent = false;
	/**
	 * vR}h̑M FacilityAgent ۂtO擾
	 * @return <ul><li>true -- M FacilityAgent ł邱Ƃ
	 *                                                              </li
	 *            ><li>false -- M FacilityAgent łȂƂ
	 *                                                              </li
	 *            ></ul>
	 */
	public boolean getResp_to_agent() {
		synchronized (this) { return resp_to_agent; }
	}
	/**
	 * vR}h̑M FacilityAgent ۂtOݒ肷
	 * @param resp_to_agent <ul><li>true -- M FacilityAgent 
	 *                                      ł邱Ƃݒ肷</li
	 *                         ><li>false -- M FacilityAgent
	 *                                       łȂƂݒ肷</li
	 *                         ></ul>
	 */
	public void setResp_to_agent(boolean resp_to_agent) {
		synchronized (this) { this.resp_to_agent	= resp_to_agent; }
	}

	// input: the name of the queue that reuester waits for responces
    private String	resp_queue_name = null;
	/**
	 * vML[̖̂擾
	 * @return vML[̖
	 */
    public String getResp_queue_name() {
		synchronized (this) { return resp_queue_name; }
	}
	/**
	 * vML[̖̂ݒ肷
	 * @param resp_queue_name vML[̖
	 */
    public void setResp_queue_name(String resp_queue_name) {
		synchronized (this) { this.resp_queue_name = resp_queue_name; }
	}
}
