/***********************************************************************
 * File: Inspector.java
 * 	(Extracted from
 *	   "$Id: Inspector.java,v 1.8 2004/07/10 16:06:37 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a Inspector utility class.
 *  (Refered Class: InspectionData, Application)
 *  (Refered Interface: Inspectable)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

import java.io.*;
import java.nio.*;
import java.nio.charset.*;
import java.nio.channels.*;
import java.nio.channels.spi.*;
import java.net.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

import my_util.Log;
import my_util.LogConf;
import my_util.Utillity;

/**
 * <p>s<em>AvP[V</em>̓ԂA
 * HTTP ŐڑĂOVXe UTF-8 GR[fBOꂽ
 * HTML `ŉ<em>CXyNV</em>@\łB</p>
 * <p>!!!! TeraTerm Őڑ邱Ƃ\łB̍ۂ́ATeraTerm 
 * j[o[ [Setup]>>[Terminal] ƑI\ꂽ_CAO
 * [Transmit]  CR+LF IA[Local echo]  ON ɂ܂B<br
 * >̐ڑ@ƂꍇA<em>CXyNV</em>@\̉
 * <em>AvP[V</em>sĂVXẽftHgE
 * GR[fBO̒PeLXgƂȂ܂B</p>
 *
 * @author maoyam
 * @version 1.0
 * @see InspectionData
 * @see Inspectable
 * @see Application
 */
public class Inspector implements Runnable, Loggable {

	// the application that this inspector is watching
	private Application application = null;

	// status of this inspector
	static final protected int NOT_PROVIDED	= 0; // port 0 is never-used
	static final protected int ACCEPTABLE	= 1;
	private int	status = NOT_PROVIDED;

	// set status of this inspector
	// @param status status to set
	private void set_status(int status) {
		synchronized (this) {
			this.status	= status;
		} // synchronized (this)
	}

	// get status of this inspector
	// @return urrent status of this inspector
	private int get_status() {
		int	result;
		synchronized (this) {
			result = this.status;
		} // synchronized (this)

		return	result;
	}

	// port number of this inspector is accepting connecting requests
	private int	port = NOT_PROVIDED;

	// log instance of this inspector
	private Log		log = null;
	/**
	 * Oo͐Ɋ蓖Ă Log ̕\擾܂
	 * @return Log ̕\
	 */
	protected String get_log() {
		synchronized (this) { return log.toString(); }
	}

	// the logging configuration
	private LogConf	log_conf = null;

	/**
	 * Oo͐pX擾܂
	 * @return Oo͐pX
	 */
	protected String get_log_path() {
		synchronized (this) { return log_conf.path; }
	}

	/**
	 * Oo͐敶GR[fBO擾܂
	 * @return Oo͐敶GR[fBO
	 */
	protected String get_log_encoding() {
		synchronized (this) { return log_conf.encoding; }
	}

	/**
	 * OEtB[h؂蕶擾܂
	 * @return OEtB[h؂蕶
	 */
	protected String get_log_delimit() {
		synchronized (this) { return log_conf.delimit; }
	}

	/**
	 * OEx擾܂
	 * @return OEx
	 */
	protected int get_log_level() {
		synchronized (this) { return log_conf.level; }
	}

	// error codes of this inspector logs
	static final protected int ERR_START_NOT_PROVIDED_INSPECTOR = 1;
	static final protected int OCCURED_EXCEPTION = 2;

	/**
	 * RXgN^: <em>CXyN^</em>@\𐶐܂B
	 *
	 * @param  application ԂĎ<em>AvP[V</em>
	 * @param  port <em>CXyN^</em>OVXe TCP/IP 
	 *              ڑv󂯕t|[gԍ
	 * @param  log_conf <em>CXyN^</em>OVXeƂ̒ʐMe
	 *                  ȂǂL^郍Oo͐̍\B
	 *                  my_util.LogConf QƂ̂ƁB
	 */
	protected Inspector(Application application
						, int port
						, LogConf log_conf
						) {
		// construct the log instance, and open it
		log	= new Log(log_conf);
		log.open();

		// set this application, port-number and logging configuration
		this.application	= application;
		this.port			= port;
		this.log_conf		= log_conf;
	}

	/**
	 * <em>CXyN^</em>@\~邽߂<em>AvP[V</em
	 * >Ăяo܂B
	 */
	protected void stop() {
		set_status(NOT_PROVIDED);
	}

	/**
	 * OVXeƂ̒ʐMeȂǂL^郍Oo͐
	 * pXEtB[h؂蕶EO񃌃xE
	 * o͕GR[fBO؂ւ܂B<br
	 * >Loggable C^tF[XvĂ郁\bhłB
	 *
	 * @param  log_conf <em>CXyN^</em>OVXeƂ̒ʐMe
	 *                  ȂǂL^郍Oo͐̍\B
	 *                  my_util.LogConf QƂ̂ƁB
	 * @return <ul><li>true -- Oo͐̐؂ւɐꍇ</li
	 *            ><li>false -- ؂ւɎsꍇ</li
	 *            ></ul>
	 */
	public boolean change_log(LogConf log_conf) throws Exception {
		boolean r = true, result = false;;

		// check the log instance was constructed?
		if (log != null) {
			// change the logging configuration
			r = log.change(log_conf);
		}
		else {
			// construct log instance, and open it
			log	= new Log(log_conf);
			log.open();
		}

		// check the log instance is exist?
		if (log != null && r == true) {
			// set this logging configurations
			this.log_conf	= log_conf;

			result	= true; // success!
		}

		return	result; // returns true: success to change logging conf.
						// returns false: failed to change logging conf.
	}

	/**
	 * OL^܂B<br
	 * >Loggable C^tF[XvĂ郁\bhłB
	 *
	 * @param level -- ÕxBLog NXQƂ邱
	 * @param msg -- OL^郁bZ[W
	 */
	public void log(int level, String msg) {
		// check the log instance is exist?
		if (log != null) {
			log.print(level, msg); // log this message
		}
	}

	// logging the fatal message caused an occured exception
	// @param errcode error code
	// @param msg the fatal message
	// @param e the exception that caused this logging action
	private void log_fatal_exception(int errcode
									, String msg
									, Exception e
									) {
		// edit the logging message on a StringBuffer
		StringBuffer sb = new StringBuffer(4096);
		sb.append("Inspector").append(log_conf.delimit).append(errcode);
		sb.append(log_conf.delimit).append(msg).append(" {");
		Utillity.format_exception_info(sb, e);
		sb.append("}");

		log(Log.FATAL, sb.toString()); // log the fatal message
	}

	// log the error message
	// @param errcode error code
	// @param msg the fatal message
	private void log_err(int errcode, String msg) {
		log(Log.ERROR
			, "Inspector" + log_conf.delimit + errcode
							+ log_conf.delimit + msg
			);
	}

	// log the infomation message
	// @param errcode error code
	// @param msg the fatal message
	private void log_info(int errcode, String msg) {
		log(Log.INFO
			, "Inspector" + log_conf.delimit + errcode
							+ log_conf.delimit + msg
			);
	}

	// flag that channel is reached EOF or not
	private boolean reached_EOF = false;

	// CharsetDecoder for decoding to UTF-8 charset
	private CharsetDecoder http_cd =
								Charset.forName("UTF-8").newDecoder();

	// convert hex-decimal character code to integer value
	// @param b hex-decimal character code (byte)
	// @return <ul><li>0 to 15 -- b is '0' to '9', 'a' to 'f'
	//                            or 'A' to 'F'</li
	//            ><li>-1 -- illegal character code</li
	//            ></ul>
	private int hex2int(byte b) {
		int d = (new Byte(b)).intValue();
		return	('0' <= d && d <= '9')?	d - '0':
				('a' <= d && d <= 'f')?	d - 'a' + 10:
				('A' <= d && d <= 'F')?	d - 'A' + 10:
				/** Not hexa-dicimal code **/	-1;
	}

	// decoding the URI formats.
	// convert '+' to SPACE, convert "%[0-9a-fA-F][0-9a-fA-F]" to
	// the character code, and convert "\r\n" to '\n'
	// @param bb ByteBuffer that stores the HTTP request
	private void decode_URI(ByteBuffer bb) {
		int limit = bb.position(); // get size of stored bytes

		// loop for the each bytes in the ByteBuffer...
		byte[]	ba = bb.array(); // get byte array
		int	p = 0;
		for (int i = 0; i < limit;) {
			// now, the EOL sequence?
			if (i + 1 < ba.length
			&& ba[i] == '\r'
			&& ba[i + 1] == '\n') {
				// replace "\r\n" to "\n"
				ba[p]	= ba[i + 1];
				++p; i += 2;
			}
			else if (ba[i] == '+') {
				// replace '+' to SPACE code
				ba[p]	= ' ';
				++p; ++i;
			}
			else if (i + 2 < ba.length && ba[i] == '%') {
				// convert "%[0-9a-fA-F]" sequence to the character code
				ba[p]	= (byte)((hex2int(ba[i + 1]) << 4)
										| hex2int(ba[i + 2]));
				++p; i += 3;
			}
			else {
				// not-converting bytes
				ba[p++]	= ba[i++];
			}
		}

		// replace the ByteBuffer to the converted byte array
		bb.clear();			// clear the ByteBuffer
		bb.put(ba, 0, p);	// put the converted byte array
	}

	// request from the clients
	private StringBuffer request = null;

	/**
	 * OVXe̎Mf[^̍őTCY 16 LEoCgɐ
	 * Ă܂B
	 */
	static final private int REQUEST_BUFFER_SIZE = 4096;
	static final protected int MAX_REQUEST_BUFFER_SIZE =
												4 * REQUEST_BUFFER_SIZE;

	// ByteBuffer that recieves the requests from clients
	private ByteBuffer rcv_bb =
							ByteBuffer.allocate(REQUEST_BUFFER_SIZE);

	// read the request from clients
	// @param sc SocketChannel that has "read" event
	// @return <ul><li>true -- this request is HTTP protocol.</li
	//            ><li>false -- this request isn't HTTP protocol</li
	//            ></ul>
	// @exception Exception See SocketChannel exceptions
	private boolean read_request(SocketChannel sc) throws Exception {
		// loop until reached to EOF or read blank-line from requests...
		boolean result = false, checked_1st_line = false;
		rcv_bb.clear(); // clear the recieve buffer
		reached_EOF	= false;
		while (true) {
			// read the request (IOException is ignored)
			int	n = -1;
			try { n = sc.read(rcv_bb); } catch (IOException ie) { ; }
			if (n < 0) {
				// reached EOF of the request
				reached_EOF = true;
				break; // return!
			}

			// The read size must be greater than 3bytes
			// for deciding the protocol
			if (!checked_1st_line && rcv_bb.position() < 4) {
				continue; // read more
			}

			// loop for deciding the protocol (HTTP or not)
			// and checking read the blank-line...
			for (int i = 0; i <= rcv_bb.position() - 4;) {
				// get the 4 character codes
				int	c0 = (new Byte(rcv_bb.get(i))).intValue()
					, c1 = (new Byte(rcv_bb.get(i + 1))).intValue()
					, c2 = (new Byte(rcv_bb.get(i + 2))).intValue()
					, c3 = (new Byte(rcv_bb.get(i + 3))).intValue();

				// decide the protocol
				// 1) the 1st line starts "GET " or "POST" means HTTP
				// 2) otherwise, means non-HTTP
				if (!checked_1st_line
				&& ((c0 == 'G' && c1 == 'E' && c2 == 'T'
										&& (c3 == ' ' || c3 == '\t'))
					|| (c0 == 'P' && c1 == 'O' && c2 == 'S'
														&& c3 == 'T')
					)) {
					result	= true; // this request protocol is HTTP
				}
				checked_1st_line = true; // decided the protocol

				// seaech the blank-line in requests...
				if (c0 != '\r'
				|| c1 != '\n') {
					++i; continue;
				}
				if (result && (c2 != '\r' || c3 != '\n')) {
					i += 3; continue;
				}
				// read the blank-line!

				// check the protocol is HTTP or not?
				if (result) { // this is the HTTP reuest
					decode_URI(rcv_bb); // decode the URI formats

					// decode the request that is encoded UTF-8
					rcv_bb.flip(); // set the position to zero
					CharBuffer cb = null;
					try {
						cb = http_cd.decode(rcv_bb);
					} catch (CharacterCodingException cce) {
						// the request encoding is illegal!
						log_info(0
								, "the request encoding isn't UTF-8.");
						reached_EOF = true;
						return result; // return from the inner loop
					}
					request = new StringBuffer(cb.toString());
				}
				else { // this is the non-HTTP reuest
					// decode the request to the system default encoding
					request = new StringBuffer(new String(rcv_bb.array()
													, 0
													, rcv_bb.position()
												));
				}

				return	result; // return from the inner loop
			}

			// check the ByteBuffer size is overflow?
			if (rcv_bb.capacity() >= MAX_REQUEST_BUFFER_SIZE) {
				log_info(0, "the request buffer overflow ("
							+ MAX_REQUEST_BUFFER_SIZE + ")"
						);

				break; // return as "Reached EOF"!
			}

			// check the ByteBuffer is full?
			if (rcv_bb.remaining() == 0) {
				// extend the ByteBuffer size
				ByteBuffer bbb =
						ByteBuffer.allocate(rcv_bb.capacity()
												+ REQUEST_BUFFER_SIZE);
				bbb.put(rcv_bb.array());
				rcv_bb	= bbb;
			}
		}

		// reached the EOF of the request or read the blank-line
		reached_EOF = true;
		return	result; // returns true: the request protocol is HTTP
						// returns false: the protocol isn't HTTP
	}

	// query ID-s that recieved request means
	static final private int QID_UNKNOWN		= 0xffffffff;
	static final private int QID_ALL			= 0x00000000;
	static final private int QID_APPLICATION	= 0x00000001;
	static final private int QID_FACILITIES		= 0x00000002;
	static final private int QID_AGENTS			= 0x00000004;
	static final private int QID_WHOLE_INFO		= 0x40000000;
	private int query_id = QID_ALL;

	// parse the recieved request, and set query ID
	// @param is_HTTP_request true: the client uses HTTP protocol
	//                        false: the client uses another
	private void parse_request(boolean is_HTTP_request) {
		query_id	= QID_ALL;

		// get the query string, or decide the illegal request
		int end = 0;
		String query = null;
		if (is_HTTP_request) {
			if (request.indexOf("GET") == 0
			&& (request.indexOf(" /") == 3
					|| request.indexOf("\t/") == 3)
			&& ((end = request.indexOf(" ", 5)) != -1
					|| (end = request.indexOf("\t", 5)) != -1)) {
				if (end > 5) {
					query	= request.substring(5, end);
				}
			}
			else if(request.indexOf("POST") == 0
			&& (request.indexOf(" / ") == 4
					|| request.indexOf("\t/ ") == 4
					|| request.indexOf(" /\t") == 4
					|| request.indexOf("\t/\t") == 4)) {
				if ((end = request.lastIndexOf("\n"))
											< request.length() -3) {
					if (request.indexOf("q=", end) != end + 1) {
						query_id	= QID_UNKNOWN;
					}
					else {
						query	= request.substring(end+ 3);
					}
				}
			}
			else {
				query_id	= QID_UNKNOWN;
			}
		}
		else {
			if ((end = request.indexOf("\r")) != -1
			|| (end = request.indexOf("\n")) != -1
			|| (end = request.indexOf(" ")) != -1
			|| (end = request.indexOf("\t")) != -1) {
				query	= request.substring(0, end);
			}
		}

		// check the request isn't illegal?
		if (query_id != QID_UNKNOWN) {
			// query is null ==> QID_ALL
			// sub-string of query is "Appliaction" ==> QID_APPLICATION
			//                     is "Facilities"  ==> QID_FACILITIES
			//                     is "Agents"		==> QID_AGENTS
			//                     is "Full"        ==> QID_WHOLE_INFO
			// anythig else	 ==> QID_UNKNOWN
			if (query == null) {
				query_id = QID_ALL;
			}
			else {
				// parse "query-key(,query-key...)" sequence
				StringBuffer q = new StringBuffer(query);
				int	p = 0, qid = 0, length = q.length();
				while (p < length) {
					if ((end = q.indexOf(",", p)) == -1) {
						query = q.substring(p);
					}
					else if (p == end) {
						++p;
						continue;
					}
					else {
						query = q.substring(p, end);
						p = end + 1;
					}

					query = query.trim();
					qid = ((query.compareTo("Application") == 0)?
												QID_APPLICATION:
							(query.compareTo("Facilities") == 0)?
												QID_FACILITIES:
							(query.compareTo("Agents") == 0)?
												QID_AGENTS:
							(query.compareTo("Full") == 0)?
												QID_WHOLE_INFO:
							QID_UNKNOWN /* anythig else */ );
					if (qid == QID_UNKNOWN) {
						query_id = QID_UNKNOWN;
						break;
					}
					query_id	|= qid;

					if (end == -1) break;
				}
			}
		}
	}

	// !!!! HTTP ]f[^ UTF-8 ZbgŃGR[hĂ
	//        Ƃ肵Ă܂
	// encoder for the HTTP responce in UTF-8 charset
	private CharsetEncoder ce = Charset.forName("UTF-8").newEncoder();
	private String encoding			= "UTF8";
	private String charset_attr		= "charset=utf-8";

	// the responce StringBuffer
	private StringBuffer res_sb = new StringBuffer(4096);

	// make the TR-tags from the InspectionData
	// @param sb StringBuffer that is appended TR-tags
	// @param id InspectionData
	// @return StringBuffer taht was appended the TR-tags
	private StringBuffer make_TR_tags(StringBuffer sb
									, InspectionData id
									) {
		if (sb == null) sb = new StringBuffer(4096);
		if (id == null) return sb;
		for (int i = 0; i < id.entries.length; ++i) {
			sb.append("><tr\r\n");
			if (id.entries[i].values != null) {
				for (int j = 0
				; j < id.entries[i].values.entries.length
				; ++j) {
					if (j != 0) {
						sb.append("><tr\r\n");
					}
					else {
						sb.append("><td rowspan=\"")
							.append(id.entries[i].values.entries.length)
							.append("\">").append(id.entries[i].name)
							.append("</td\r\n");
					}
					sb.append("><td>")
						.append(id.entries[i].values.entries[j].name)
						.append("</td><td>")
						.append(id.entries[i].values.entries[j].value)
						.append("</td\r\n")
						.append("></tr\r\n");
				}
			}
			else {
				sb.append("><td>").append(id.entries[i].name)
					.append("</td\r\n")
					.append("><td>")
					.append(((id.entries[i].value != null)?
												id.entries[i].value:
												"N/A"))
					.append("</td\r\n")
					.append("></tr\r\n");
			}
		}

		return	sb; // returns StringBuffer taht was appended TR-tags
	}

	// make the text lines of InspectionData entries
	// @param sb StringBuffer that is appended the lines
	// @param id InspectionData
	// @return StringBuffer that was appended the lines
	private StringBuffer make_TEXT_lines(StringBuffer sb
										, InspectionData id
										) {
		if (sb == null) sb = new StringBuffer(4096);
		if (id == null) return sb;
		for (int i = 0; i < id.entries.length; ++i) {
			sb.append("\t").append(id.entries[i].name).append(" {");
			if (id.entries[i].values != null) {
				for (int j = 0
				; j < id.entries[i].values.entries.length
				; ++j) {
					sb.append(id.entries[i].values.entries[j].name)
						.append(" {")
						.append(
							id.entries[i].values.entries[j].value)
						.append("} ");
				}
			}
			else {
				sb.append(((id.entries[i].value != null)?
											id.entries[i].value:
											"N/A"));
			}
			sb.append("}\r\n");
		}

		return sb; // returns StringBuffer that was appended TEXT-lines
	}

	// GMT date-time formatter for HTTP headers
	static private DateFormat df_GMT =
				new SimpleDateFormat("EEE, dd MMM yyyy HH:MM:SS 'GMT'");

	// responce the HTTP request (encoded UTF-8 charset)
	// @param sc SocketChannel that connects to the client
	private void responce_HTTP(SocketChannel sc) throws Exception {
		// make the HTTP responce in UTF-8 charset
		StringBuffer sb = res_sb.delete(0, res_sb.length());

		// get the date-time of responce
		long now = System.currentTimeMillis();

		// make the HTTP responce contents in HTML
		sb.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD ")
										.append("HTML 4.01//EN\"\r\n")
			.append("><html lang=\"ja\"><head\r\n")
			.append("><meta http-equiv=\"CONTENT-TYPE\" ")
								.append("content=\"text/html; ")
								.append(charset_attr).append("\"\r\n")
			.append("><title>Responce of Inspector 1.0</title\r\n")
			.append("></head><body\r\n")
			.append("><h1>CXyN^ 1.0 ̉</h1\r\n")
			.append("><p>--- ").append(new Date(now).toString())
												.append(" ---</p\r\n")
			.append("><hr\r\n");

		if (query_id == QID_UNKNOWN) {
			sb.append("><strong>NG͂ĂB")
				.append("</strong\r\n");
		}
		else if (application == null) {
			sb.append("><strong>AvP[VُԂłB")
				.append("</strong\r\n");
		}
		else {
			// make the summary informations of this application
			sb.append("><h2>AvP[VT}</h2\r\n")
					.append("><table border=\"1\" ")
					.append("summary=\"Application Summary Info\"\r\n")
					.append("><tbody\r\n");
			make_TR_tags(sb, application.get_summary())
					.append("></tbody></table\r\n");
			sb.append("><hr\r\n");

			// make the detail informations of this application
			if ((query_id & QID_WHOLE_INFO) != 0
			|| (query_id & QID_APPLICATION) != 0) {
				sb.append("><h2>AvP[Vڍ׏</h2\r\n")
						.append("><table border=\"1\" ")
						.append("summary=\"Application Details\"\r\n")
						.append("><tbody\r\n");
				make_TR_tags(sb, application.get_details())
						.append("></tbody></table\r\n");
				sb.append("><hr\r\n");
			}

			// make the summary of the facilities
			FacilityConf[] fcs = application.get_fac_confs();
			if (fcs == null) {
				sb.append("><strong>AvP[Vɋ@\\")
					.append("ݒ肳Ă܂B</strong\r\n");
			}
			else {
				if (query_id == QID_ALL
				|| (query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_FACILITIES) != 0) {
					sb.append("><h2>@\T}</h2\r\n")
						.append("><table border=\"1\" ")
						.append("summary=\"Facilities Summary Info\"")
						.append("\r\n")
						.append("><tbody\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						if (fac != null) {
							make_TR_tags(sb, fac.get_summary(i + 1));
						}
						else {
							sb.append("><tr><td>")
							.append(Integer.toString(i + 1, 10))
							.append("</td><td colspan=\"2\"")
							.append(">!!@\ \"")
							.append(fcs[i].getName())
							.append("\" ̌Ɏs!!</td></tr\r\n");
						}
					}
					sb.append("></tbody></table\r\n")
						.append("><hr\r\n");
				}

				if ((query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_FACILITIES) != 0) {
					sb.append("><h2>@\ڍ׏</h2\r\n")
						.append("><table border=\"1\" ")
						.append("summary=\"Facilities Details\"\r\n")
						.append("><tbody\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						if (fac != null) {
							make_TR_tags(sb, fac.get_details(i + 1));
						}
						else {
							sb.append("><tr><td>")
							.append(Integer.toString(i + 1, 10))
							.append("</td><td colspan=\"2\"")
							.append(">!!@\ \"")
							.append(fcs[i].getName())
							.append("\" ̌Ɏs!!</td></tr\r\n");
						}
					}
					sb.append("></tbody></table\r\n")
						.append("><hr\r\n");
				}

				if ((query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_AGENTS) != 0) {
					sb.append("><h2>@\G[WFg</h2\r\n")
						.append("><table border=\"1\" ")
						.append("summary=\"Facility Agents Details")
						.append("\"\r\n><tbody\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						sb.append("><tr><td colspan=\"3\">")
							.append(fcs[i].getName())
							.append("</td\r\n");
						if (fac != null) {
							make_TR_tags(sb
										, fac.get_agents_details()
										);
						}
						else {
							sb.append("><td colspan=\"3\">")
							.append(">!!@\ \"")
							.append(fcs[i].getName())
							.append("\" ̌Ɏs!!</td><tr\r\n");
						}
					}
					sb.append("></tbody></table\r\n")
						.append("><hr\r\n");
				}
			}
		}

		// this FORM-tag provides the query service
		sb.append("><form name=\"f\" method=\"POST\" ")
							.append("action=\"/\"\r\n")
		.append("><p\r\n")
		.append(">\̃L[[h͂Ă<br\r\n")
		.append(">(Full, Application, Facilities, Agents)<br\r\n")
		.append("><input maxLength=\"256\" size=\"55\" name=\"q\" ")
								.append("value=\"\"\r\n")
		.append("><input type=\"submit\"\r\n")
		.append("></p></form\r\n")
		.append("><hr\r\n");

		// make the HTML string
		String html = sb.toString() + "></body></html>\r\n";

		// encode the HTTP responce contents to UTF-8 charset
		CharBuffer cb = CharBuffer.allocate(html.length());
		ByteBuffer bb = ce.encode((CharBuffer)(cb.put(html).flip()));

		// make the HTTP responce header in MIME
		String head =
				"HTTP/1.1 200 OK\r\n"
				+ "Date: "
					+ df_GMT.format(new Date(now - 9 * 3600 * 1000))
					+ "\r\n"
				+ "Server: Inspector/1.0\r\n"
				+ "Cache-Control: no-cache\r\n"
				+ "Content-Length: " + bb.limit() + "\r\n"
				+ "Content-Type: text/html; " + charset_attr + "\r\n"
				+ "Connection: close\r\n"
				+ "\r\n";

		// encode the HTTP responce header to UTF-8 charset
		cb	= CharBuffer.allocate(head.length());
		ByteBuffer bb_head =
						ce.encode((CharBuffer)(cb.put(head).flip()));

		// send the HTTP responce header and contents
		try { sc.write(bb_head); sc.write(bb);
		} catch (IOException ie) { ; /* IOException is igbored */ }

		// set the "Reached EOF" flag
		reached_EOF = true;
	}

	// !!!!  HTTP ]f[^̓VXe̕GR[fBO
	//        GR[hĂ邱Ƃ肵Ă܂
	// responce the non-HTTP request
	// @param sc SocketChannel connects to the client
	// @exception Exception See SocketChannel exceptions
	private void responce_non_HTTP(SocketChannel sc) throws Exception {
		// make the responce texts in the system default encoding
		StringBuffer sb = res_sb.delete(0, res_sb.length());

		// make the header
		sb.append("=====================================\r\n");
		sb.append("CXyN^ 1.0 ̉\r\n")
			.append(new Date().toString()).append("\r\n");

		//check the application is exist?
		if (query_id == QID_UNKNOWN) {
			sb.append("***Not Correctly Query*** ")
				.append("Type the correctly query, please\r\n")
				.append("(Full, Application, Facilities, Agents)\r\n");
		}
		else if (application == null) {
			sb.append("!!!FATAL!!! The application is lost!\r\n");
		}
		else {
			// make the summary informations
			sb.append("--- Application summary --------------\r\n");
			make_TEXT_lines(sb, application.get_summary());

			// make the details informations
			if ((query_id & QID_WHOLE_INFO) != 0
			|| (query_id & QID_APPLICATION) != 0) {
				sb.append("--- Application details --------------\r\n");
				make_TEXT_lines(sb, application.get_details());
			}

			// make the summary of the facilities
			FacilityConf[] fcs = application.get_fac_confs();
			if (fcs == null) {
				sb.append("!!!FATAL!!! ")
					.append("The application specified ")
					.append("no facility configurations.\r\n");
			}
			else {
				if (query_id == QID_ALL
				|| (query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_FACILITIES) != 0) {
					sb.append("--- Facilities summary ")
										.append("---------------\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						if (fac != null) {
							make_TEXT_lines(sb, fac.get_summary(i + 1));
						}
						else {
							sb.append(Integer.toString(i + 1, 10))
								.append(" !!@\ \"")
								.append(fcs[i].getName())
								.append("\" ̌Ɏs!!\r\n");
						}
					}
				}

				if ((query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_FACILITIES) != 0) {
					sb.append("--- Facilities details ")
										.append("---------------\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						if (fac != null) {
							make_TEXT_lines(sb, fac.get_details(i + 1));
						}
						else {
							sb.append(Integer.toString(i + 1, 10))
								.append(" !!@\ \"")
								.append(fcs[i].getName())
								.append("\" ̌Ɏs!!\r\n");
						}
					}
				}

				if ((query_id & QID_WHOLE_INFO) != 0
				|| (query_id & QID_AGENTS) != 0) {
					sb.append("--- Facility Agenst details ")
										.append("---------------\r\n");
					for (int i = 0; i < fcs.length; ++i) {
						Facility fac =
									Facility.attach(fcs[i].getName());
						sb.append("<").append(fcs[i].getName())
										.append(">\r\n");
						if (fac != null) {
							make_TEXT_lines(sb
											, fac.get_agents_details()
											);
						}
						else {
							sb.append(Integer.toString(i + 1, 10))
								.append(" !!@\ \"")
								.append(fcs[i].getName())
								.append("\" ̌Ɏs!!\r\n");
						}
					}
				}
			}
		}
		sb.append("=====================================\r\n");

		// encode the responce text to the system default encoding,
		// and then, send the encoded text to the non-HTTP client
		byte[] resp = sb.toString().getBytes();
		ByteBuffer bb = ByteBuffer.allocate(resp.length);
		try {
			sc.write((ByteBuffer)(bb.put(resp).flip()));
		} catch (IOException ie) {
			// Occured IOException!
			Socket so = sc.socket();
			log_info(0
				, "occured IOException to send the non-HTTP responce "
					+ "to " + so.getInetAddress().toString()
					+ " (port#: " + sc.socket().getPort() + ")"
				);

			reached_EOF = true; //set the "Reached EOF" flag
		}
	}

	/**
	 * <em>AvP[V</em>Ap̃Xbh
	 * <em>CXyN^</em>NƌĂяo܂B<br
	 * Java 1.4 œꂽ nio (New I/O) gpāAmubLO
	 * ȃ\Pbg TCP/IP ʐMs܂B<br
	 * >Runnable C^tF[XvĂ郁\bhłB
	 */
	public void run() {
		// check the inspector is provided?
		if (port == NOT_PROVIDED) {
			// The inspector isn't provided by this application!
			log_err(ERR_START_NOT_PROVIDED_INSPECTOR
				, "port is " + port + ". inspector isn't provided."
				);
			return; // error!
		}
		set_status(ACCEPTABLE); // set the status to ACCEPTABLE
		log_info(0, "application inspector " + this + " starts...");

		try {
			// construct the server socket to accept connecting
			// from clients
			Socket so = null;
			SocketChannel sc = null;
			InetAddress lh = InetAddress.getLocalHost();
			Selector sel = SelectorProvider.provider().openSelector();
			ServerSocketChannel ssc = ServerSocketChannel.open();
			ssc.configureBlocking(false);
			ssc.socket().bind(new InetSocketAddress(lh, port));
			SelectionKey accept_sk = ssc.register(
											  sel
											, SelectionKey.OP_ACCEPT
											);
			log_info(0, "started to accept connecting to "
						+ lh.toString() + " (port: " + port + ")"
					);

			// loop for accepting the connecting request
			// and recieving the requests from clients...
			int n = 0;
			while ((n = sel.select(100)) >= 0) {
				// check the selected key is exist?
				if (n > 0) {
					// loop for the each selected keys...
					Iterator ite = sel.selectedKeys().iterator();
					while (ite.hasNext()) {
						// get a selected key
						SelectionKey sk = (SelectionKey)(ite.next());
						ite.remove();

						// check the selected key is relayed to
						// the server socket?
						if (sk.isAcceptable()) {
							// get the new socket and its channel
							so = ((ServerSocketChannel)(sk.channel()))
													.accept().socket();
							sc = so.getChannel();

							// set the channel NON-BLOCKING mode
							sc.configureBlocking(false);

							// register the channel to selector
							sc.register(sel, SelectionKey.OP_READ);

							// log the connected information
							log_info(0, "accepted connecting from "
										+ so.getInetAddress().toString()
										+ " (port: " + so.getPort()
										+ ")"
									);

							continue; // wait for the selection...
						}

						// get the established socket and its channel
						sc = (SocketChannel)(sk.channel());
						so = sc.socket();

						// read the request and decide the protocol
						reached_EOF = false;
						boolean is_HTTP_request = read_request(sc);

						// check the connection is alive?
						if (!reached_EOF) {
							// log the recieved requests
							log_info(0, "request from "
										+ so.getInetAddress().toString()
										+ " (port: " + so.getPort()
										+ ") ["
										+ ((is_HTTP_request)?
															"HTTP":
															"non-HTTP")
										+ "] {" + request.toString()
										+ "}"
									);

							// parse the recieved request
							parse_request(is_HTTP_request);

							// check the protocol is HTTP or not?
							if (is_HTTP_request) {
								// responce th the HTTP client
								responce_HTTP(sc);
							}
							else {
								// responce to the non-HTTP client
								responce_non_HTTP(sc);
							}
						}

						// check the connection is closed from client?
						if (reached_EOF) {
							// cancel the select key, close the socket
							// and close the chaneel
							sk.cancel();
							so = sc.socket();
							so.close();
							sc.close();

							// log the closed connection message
							log_info(0, "closed connection from "
										+ so.getInetAddress().toString()
										+ " (port: " + so.getPort()
										+ ")"
									);
						}
					}
				}

				// check this inspector is stopped by the application?
				if (get_status() != ACCEPTABLE) {
					break; // halt this inspector!
				}
			}

			// cancel the server select key, close the server socket
			// and close the server channel
			accept_sk.cancel();
			ssc.socket().close();
			ssc.close();

			// log the stopped accepting message
			log_info(0, "stopped to accept connecting to "
						+ lh.toString() + " (port: " + port + ")"
					);
		} catch (Exception e) {
			// Occured the fatal exception!
			log_fatal_exception(OCCURED_EXCEPTION
				, "occured exception!"
				, e
				);
		}

		// set the status to NOT-PROVIDED
		set_status(NOT_PROVIDED);

		// log the stoppped the inspector message
		log_info(0, "application inspector " + this + " finished.");
	}
}
