/***********************************************************************
 * File: Log.java
 * 	(Extracted from
 *	   "$Id: Log.java,v 1.7 2004/07/04 12:32:45 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a logging utility class.
 *  (Refered Class: LogResource)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package	my_util;

import java.io.*;
import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import my_util.Utillity;

/**
 * <p>OL^@\񋟂郆[eBeBENXłB</p>
 * @author maoyam
 * @version 1.0
 * @see my_util.LogResource
 */
public class Log {

	/** ftHg̃tB[h؂蕶 */
	static public final String DEFAULT_DELIMIT = ":";

	static private String[] names
						= { "FATAL", "ERROR", "WARN", "INFO", "DEBUG" };
	static final private int NONE = -1;
	/** OENX̎ʎqBvIȎۂ */
	static final public int FATAL = 0;
	/** OENX̎ʎqBʓIȃG[ */
	static final public int ERROR = 1;
	/** OENX̎ʎqBx */
	static final public int WARN = 2;
	/** OENX̎ʎqBʏ */
	static final public int INFO = 3;
	/** OENX̎ʎqBfobO */
	static final public int DEBUG = 4;
	static final private int ALL = 5;

	/** Oo͂~邱Ƃ */
	static public final String NO_LOG = null;
	/** OWo System.out ɏo͂邱Ƃ */
	static public final String STDOUT = "";

	// configuration of logging
	private LogConf conf = new LogConf(null
										, NO_LOG
										, null
										, DEFAULT_DELIMIT
										, NONE);

	// output stream of logging
	private PrintStream out = null;

	/**
	 * RXgN^B
	 * O̓tEtH[}bgEp^[AOo͂
	 * VXẽpXƋ؂蕶AGR[fBO
	 * o͂郍OENXw肵 my_util.LogConf w肵܂B
	 * @param conf Oo͂Ɋւ\B
	 *             null ̏ꍇAOo͂͂ȂB
	 */
	public Log(LogConf conf) {
		if (conf != null) {
			this.conf	= conf;
		}
	}

	// open the output stream
	// @return <ul><li>true -- success; the path is opend.</li
	//             ><li>false -- failed to open the path.</li
	//             ></ul>
	private boolean _open() {
		boolean result = true;

		if (conf.level <= NONE || ALL <= conf.level) {
			result	= false; // illegal logging level
		}
		else if (conf.path == NO_LOG) {
			out	= null; // no output stream
		}
		else if (conf.path.compareTo(STDOUT) == 0) {
			out	= System.out; // logging to standard output stream
		}
		else {
			try {
				// search the LogResource
				LogResource	lr = LogResource.create(conf.path
													, conf.encoding
													);
				if (lr == null) {
					result	= false; // new logging path
				}
				else {
					out	= lr.out; // the path was opend, yet
				}
			} catch (Exception e) {
				// Occured Exception!
				result = false;
			}
		}

		return result;	// returns true; success. the path is opened.
						//  returns false; failed to open the path
	}

	/**
	 * Oۊǐ̃VXeI[vAOo͂s悤
	 * ܂B
	 * @return <ul><li>true -- VXeւ̃Oo͂\ɂȂ
	 *                         Ƃ</li
	 *            ><li>false -- VXe̊lɎsꍇ</li
	 *            ></ul>
	 */
	public boolean open() {
		boolean result	= false;

		synchronized (this) {
			result	= _open();
		} // synchronized (this)

		return	result;
	}

	/**
	 * OۊǐɁAOENXw肵bZ[Wo͂܂B
	 * ftHg̃OEbZ[W̃tH[}bg͎̒ʂ<br
	 * >"yyyy/mm/dd HH:MM:SS.sss<strong>:</strong
	 * ><em>level</em><strong>:</strong
	 * ><em>msg</em>"<ul
	 *   ><li><strong>:</strong> -- tB[h؂蕶</li
	 *   ><li><em>level</em> --  level ɑΉ郍OENX (
	 *                          FATAL, ERROR, WARN, INFO ̂ꂩ)</li
	 *   ><li><em>msg</em> --  msg Ŏw肳ꂽ</li
	 * ></ul>
	 */
	public void print(int level, String msg) {
		synchronized (this) {
			// check the logging level is legal?
			if (NONE < level && level <= conf.level) {
				// get the current time from the system epoch
				long now = System.currentTimeMillis();
				StringBuffer buf = null;

				// check the date-time formatter wasn't specified?
				if (conf.df == null) {
					// formatter wasn't specified
					buf = Utillity.format_milli_sec(null, now);
				}
				else {
					// use the specified SimpleDateFormatter pattern
					buf	= new StringBuffer();
					buf.append(conf.df.format(new Date(now)));
				}
				buf.append(conf.delimit);
				buf.append(names[level]);
				buf.append(conf.delimit);
				buf.append(msg);

				out.println(buf.toString());
			}
		} // synchronized (this)
	}

	// close the logging output stream
	private void _close() {
		if (out != null && out != System.out) {
			LogResource.destroy(conf.path);
			out				= null;
			conf.df			= null;
			conf.path		= NO_LOG;
			conf.encoding	= null;
			conf.delimit	= DEFAULT_DELIMIT;
			conf.level		= NONE;
		}
	}

	/**
	 * Oۊǐ̃VXeAȌ̃OL^͍sȂ悤
	 * ܂B
	 */
	public void close() {
		synchronized (this) {
			_close();
		} // synchronized (this)
	}

	/**
	 * Oۊǐ̃VXe؂ւ܂B
	 * ؂ւAɃOL^s悤ɃVXe
	 * I[v܂B
	 * ؂ւ郍Oo͐ my_util.LogConf Ŏw肵܂B
	 * @param conf Oo͐̍\Bmy_util.LogConf QƂ̂ƁB
	 * @return <ul><li>true -- VXe̐؂ւɐꍇ</li
	 *            ><li>false -- VXe̐؂ւɎsꍇ</li
	 *            ></ul>
	 */
	public boolean change(LogConf conf) {
		boolean result;

		synchronized (this) {
			_close();
			this.conf	= conf;
			result	= ((conf != null)? _open(): true);
		} // synchronized (this)

		return result;
	}
}
