/***********************************************************************
 * File: Message.java
 * 	(Extracted from
 *	   "$Id: Message.java,v 1.10 2004/06/13 06:46:46 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a message utility class.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

import java.util.Hashtable;

/**
 * <p>XbhԂŃIuWFNg𑗎M邽߂̃bZ[WEL[łB
 * </p>
 * @author maoyam
 * @version 1.0
 */
public class Message {
	/**
	 * bZ[WEL[ɑؗ\ȃIuWFNg̊l (= 100) 
	 * łB
	 */
	static public final int DEFAULT_MESSAGE_MAX	= 100;

	static private Hashtable	messages = new Hashtable();

	int		owner_hash	= -1;
	int	message_max	= DEFAULT_MESSAGE_MAX;

	private int message_cnt	= 0;
	private Object[]	buckets = new Object[DEFAULT_MESSAGE_MAX];
	private int			bucket_head	= 0;
	private int			bucket_tail	= 0;
	private boolean	destroyed	= false;

	/**
	 * RXgN^𒼐ڌĂяoƂ͂ł܂B<br>
	 * {@link #create} ŃbZ[WEL[쐬܂B
	 * @param me bZ[WEL[쐬XbhB
	 * @param length ؗłIuWFNg̍őlB
	 */
	private Message(Thread me, int length) {
		owner_hash	= me.hashCode();
		message_max	= length;
		if (message_max != buckets.length) {
			buckets	= new Object[message_max];
		}
		for (int i = 0; i < buckets.length; ++i) {
			buckets[i]	= null;
		}
	}

	/**
	 * XbhԂŎgp郁bZ[WEL[쐬NXE\bh
	 * łBbZ[WEL[ɌŗL̖Oč쐬܂B
	 * Xbh̖͂OŃbZ[WEL[ɐڑ ({@link #attach})
	 * ܂B
	 * @param me bZ[WEL[쐬XbhBnull w肷
	 * 			bZ[WL[͍쐬ꂸAnull Ԃ܂B
	 * @param name bZ[WEL[ɌŗL̖OB̃Xbhڑ
	 * 			鎞ɂ̖OŐڑ邱ƂɂȂ܂B
	 * @param length ؗIuWFNg̍őlB
	 * @return <ul><li> null - 쐬ꂽbZ[WEL[B
	 * 							邢́Aɍ쐬ꂽbZ[WE
	 * 							L[ (Alength ̃L[
	 * 							쐬Ɏw肳ꂽ length Ɠlł
	 * 							ꍇɌ܂)</li
	 *			><li>null - Ɍ肪ꍇɕԂ܂B
	 * 						bZ[WEL[̍쐬 length 
	 * 						̃\bh length vȂꍇ
	 * 						Ԃ܂B</li
	 *			></ul>
	 */
	static public Message create(Thread me, String name, int length) {
		Message	msg = null;

		if (((me != null) && name != null) && length > 0) {
			synchronized (messages) {
				msg	= (Message)(messages.get((Object)name));
				if (msg == null) {
					msg	= new Message(me, length);
					messages.put(name, msg);
				}
				else if (msg.message_max != length) {
					msg	= null;
				}
			}
		}
		return	msg;
	}

	/**
	 * Ow肵Ċɍ쐬Ă郁bZ[WEL[擾
	 * NXE\bhłB
	 * @param name bZ[WEL[쐬ꂽɂꂽŗL
	 * 			O
	 * @return <ul><li> null - bZ[WEL[</li
	 * 				><li>null - OɑΉ郁bZ[WEL[
	 * 							쐬ĂȂƂ܂B</li
	 * 				></ul>
	 */
	static public Message attach(String name) {
		Message	msg;

		synchronized (messages) {
			msg	= (Message)(messages.get((Object)name));
		}
		return	msg;
	}

	/**
	 * bZ[WEL[jNXE\bhłB
	 * bZ[WEL[쐬Xbhjł܂B
	 * @param me bZ[WEL[jXbhBnull 
	 * 			bZ[WEL[쐬XbhłȂꍇA
	 * 			bZ[WL[͔j܂B
	 * @param name bZ[WEL[쐬ɕtꂽOB
	 * 				null ̏ꍇAbZ[WEL[͔j܂B
	 * @return <ul><li>true - bZ[WEL[jꂽƂ
	 * 						܂B</li
	 * 			><li>false - ł邩AɃbZ[WEL[
	 * 						jĂ邱Ƃ܂B</li
	 * 			></ul>
	 */
	static public boolean destroy(Thread me, String name) {
		boolean	result = false;

		if ((me != null) && name != null) {
			synchronized (messages) {
				Message	msg = (Message)(messages.get((Object)name));

				if (msg != null) {
					if (me.hashCode() == msg.owner_hash) {
						messages.remove(name);
						msg.set_destroyed();
						result	= true;
					}
				}
			}
		}
		return result;
	}

	/**
	 * bZ[WEL[ɑؗĂIuWFNg̐擾
	 * \bhłB
	 * @return	bZ[WEL[ɑؗĂIuWFNgԂ܂
	 */
	public int	get_message_cnt() {
		synchronized (this) {
			return	message_cnt;
		} // synchronized (this)
	}

	/**
	 * bZ[WEL[jꂽƂtOZbg
	 * veNebhE\bhłB
	 * {@link #destroy} Ăяo܂B
	 */
	void set_destroyed() {
		synchronized (this) {
			destroyed	= true;
			notifyAll();
		}
		return;
	}

	/**
	 * bZ[WEL[̐擪IuWFNg擾܂B
	 * bZ[WEL[̏ꍇubN邩ȂAɁA
	 * ubNꍇ^CAEg܂ł̎Ԃ~bxŎw
	 * ł܂B
	 * @param head bZ[WEL[̐擪 (= true) 邢
	 * 			 (= false) ǂ̈ʒuIuWFNg
	 * 			w肷邩ɂĎw肵܂B
	 * @param blocking bZ[WEL[̏ꍇɏubN
	 * 				(= true) Ȃ (= false) ɂĎw肵܂B
	 * @param timeout ubNꍇA^CAEg܂ł̎Ԃ
	 * 				~bxŎw肵܂B
	 * 				0 ȉ̏ꍇ^CAEg͍s킸A
	 * 				ubN܂B
	 * @return <ul><li> null - 擾IuWFNgB</li
	 *			><li>null - bZ[WEL[ł邱Ƃ܂B
	 * 				></li
	 * 			></ul>
	 * @exception TimeoutException ^CAEgꍇB
	 * 				ObZ[W 
	 * 				"Message.get(): Timeout (^CAEgE~b)"
	 * @exception InterruptedException Xbh interrupt()
	 * 				\bhĂяoŊ荞܂ꂽꍇB
	 */
	public Object get(boolean head
						, boolean blocking
						, long timeout
						) throws Exception, InterruptedException {
		Object	result	= null;
		long	to, t;

		if (blocking && timeout > 0) {
			to	= timeout + System.currentTimeMillis();
		}
		else {
			to	= 0;
		}
		while (true) {
			synchronized (this) {
				if (destroyed) {
					notifyAll();
					break;
				}
				if (message_cnt > 0) {
					if (head) {
						result	= buckets[bucket_head];
						buckets[bucket_head]	= null;
						bucket_head	= (bucket_head + 1) % message_max;
					}
					else {
						--bucket_tail;
						if (bucket_tail < 0) {
							bucket_tail	= message_max - 1;
						}
						result	= buckets[bucket_tail];
						buckets[bucket_tail]	= null;
					}
					--message_cnt;
					notifyAll();
					break;
				}

				if (!blocking)	break;

				try {
					if (to > 0) {
						t	= to - System.currentTimeMillis();
						if (t <= 0) {
							notifyAll();
							throw new TimeoutException(
												"Message.get(" + head
												+ ", " + blocking + ", "
												+ timeout + ")"
											, timeout
											);
						}
					}
					else {
						t	= 0;
					}
					wait(t);
					if ((to > 0) && to < System.currentTimeMillis()) {
						notifyAll();
						throw new TimeoutException("Message.get(" + head
													+ ", " + blocking
													+ ", " + timeout
													+ ")"
												, timeout
												);
					}
				} catch (InterruptedException e) {
					if ((to > 0) && to < System.currentTimeMillis()) {
						notifyAll();
						throw new TimeoutException("Message.get(" + head
													+ ", " + blocking
													+ ", " + timeout
													+ ")"
												, timeout
												);
					}
				}
			} // synchronized (this);
		}
		return result;
	}

	/**
	 * bZ[WEL[̐擪ɃIuWFNg}܂B
	 * bZ[WEL[t̏ꍇubN邩ȂAɁA
	 * ubNꍇ^CAEg܂ł̎Ԃ~bxŎw
	 * ł܂B
	 * @param head bZ[WEL[̐擪 (= true) 邢
	 * 			 (= false) ǂ̈ʒuɃIuWFNg
	 * 			}邩ɂĎw肵܂B
	 * @param blocking bZ[WEL[t̏ꍇɏubN
	 * 				 (= true) Ȃ (= false) ɂĎw
	 * 				܂B
	 * @param timeout ubNꍇA^CAEg܂ł̎Ԃ
	 * 				~bxŎw肵܂B
	 * 				0 ȉ̏ꍇ^CAEg͍s킸A
	 * 				ubN܂B
	 * @return <ul><li>true - IuWFNg̑}ɐƂ
	 * 						܂B</li
	 *			><li>false - bZ[WEL[tł邱Ƃ
	 * 						܂B></li
	 * 			></ul>
	 * @exception TimeoutException ^CAEgꍇ
	 *				ObZ[W 
	 * 				"Message.put(): Timeout (^CAEgE~b)"
	 * @exception InterruptedException Xbh interrupt()
	 * 						\bhĂяoŊ荞܂ꂽꍇB
	 */
	public boolean put(Object o
						, boolean head
						, boolean blocking
						, long timeout
						) throws Exception, InterruptedException {
		boolean	result	= false;
		long	to, t;

		if (o == null) {
			return	result;
		}
		if (blocking && timeout > 0) {
			to	= timeout + System.currentTimeMillis();
		}
		else {
			to	= 0;
		}
		while (true) {
			synchronized (this) {
				if (destroyed) {
					notifyAll();
					break;
				}
				if (message_cnt < message_max) {
					if (head) {
						bucket_head	= ((bucket_head == 0)?
											message_max - 1:
											bucket_head - 1
										);
						buckets[bucket_head]	= o;

					}
					else {
						buckets[bucket_tail]	= o;
						bucket_tail	= ((bucket_tail == message_max - 1)?
											0:
											bucket_tail + 1
											);
					}
					++message_cnt;
					notifyAll();
					result	= true;
					break;
				}

				if (!blocking)	break;

				try {
					if (to > 0) {
						t	= to - System.currentTimeMillis();
						if (t <= 0) {
							notifyAll();
							throw new TimeoutException(
												"Message.put(" + o
												+ ", " + head + ", "
												+ blocking + ", "
												+ timeout + ")"
											, timeout
											);
						}
					}
					else {
						t	= 0;
					}
					wait(t);
					if ((to > 0) && to < System.currentTimeMillis()) {
						notifyAll();
						throw new TimeoutException("Message.put(" + o
											+ ", " + head + ", "
											+ blocking + ", "
											+ timeout + ")"
										, timeout
										);
					}
				} catch (InterruptedException e) {
					if ((to > 0) && to < System.currentTimeMillis()) {
						notifyAll();
						throw new TimeoutException("Message.put(" + o
											+ ", " + head + ", "
											+ blocking + ", "
											+ timeout + ")"
										, timeout
										);
					}
				}
			} // synchronized (this);
		}

		return	result;
	}
}
