/***********************************************************************
 * File: Semaphore.java
 * 	(Extracted from
 *	   "$Id: Semaphore.java,v 1.8 2004/06/13 06:47:33 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a semaphore utility class.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

/**
 * <p>ɕ̃XbhpIɊlłZ}tHłB
 * ܂A̃XbhLIɊl邱Ƃł܂B
 * XbhLIɊlĂԂ́ÃXbh
 * Z}tHlł (ҋ@w肳Ăꍇ) ^CAEg
 * ꂽҋ@s܂B</p
 * ><p
 * >RXgN^ɃXbhw肷邱ƂŁA炩ߊlς݂̏
 * ɂȂĂZ}tH쐬邱Ƃł܂B
 * ̍ۂɋpIɊl邩LIɊl̂ɂĂ
 * wł܂B</p
 * ><p
 * >XbhԂŋLIuWFNg (LIuWFNg) 
 * RXgN^Ɏw肷邱ƂŃZ}tHɊ֘At邱Ƃł܂B</p
 * >
 * @author maoyam
 * @version 1.0
 * @see my_util.BinSemaphore
 */
public class Semaphore {
	/** ɃZ}tHlłXbh̊l (= 16) łB */
	static public final int	DEFAULT_CNT_OF_SHARING_OWNERS = 16;
	/** ɃZ}tHlłXbh̍ŏl (= 1) łB */
	static public final int	MIN_CNT_OF_OWNERS	= 1;
	/**
	 * Z}tHl̂߂ɖɑ҂ۂ̃^CAEgl (= 0) łB
	 */
	static public final int	INFINITE	= 0; // for `timeout' of take()

	static private final int	MONOPOLIZE = -1, NONE_TAKES = 0;
	static private final int	OWNER_NONE_HASHCODE = -1;

	private int owner_cnt				= NONE_TAKES;
	private Object	exclusive_obj		= null;
	private int	max_owners				= DEFAULT_CNT_OF_SHARING_OWNERS;
	private int [] owner_hashCodes		=
						new int [DEFAULT_CNT_OF_SHARING_OWNERS + 1];

	/**
	 *  me Ŏw肳ꂽXbh炠炩ߊlꂽԂ
	 * Z}tH쐬܂B
	 *  sharing ̐^U (true/false) ɂAZ}tH̋pIȊl
	 * (true )ƐLIȊl (false ) ̂ꂩs܂B
	 *  o Ŏw肳ꂽIuWFNg̃Z}tHƊ֘AtA
	 * ̃XbhԂ̖IȋLIuWFNgƂȂ܂B
	 *  max_sharers Ɏw肳ꂽlɃZ}tHpIɊl
	 * łXbhɂȂ܂B
	 * @param	me	Z}tH炩ߊlXbhB
	 *				null w肳ꂽꍇAǂ̃Xbhl
	 *				ȂZ}tH쐬܂
	 * @param	sharing	me XbhZ}tH炩ߊlۂ
	 *				[hw肵܂Btrue ȂΑ̃Xbh
	 *				l\ȋpIȊlsAfalse Ȃ
	 *				Xbh͓ɊlłȂLIȊl
	 *				s܂
	 * @param	o	Z}tHɖIɊ֘At
	 *				XbhԋLIuWFNgB
	 *				null w肳ꂽꍇALIuWFNgƊ֘At
	 *				ĂȂZ}tH쐬܂
	 * @param	max_sharers	pIȊl𓯎ɍsXbh
	 *				w肵܂B
	 *				{@link #MIN_CNT_OF_OWNERS MIN_CNT_OF_OWNERS}
	 *				Ȓlw肵Ă͂܂B
	 */
	public Semaphore(Thread me
					, boolean sharing
					, Object o
					, int max_sharers
					) {
		if (max_sharers < MIN_CNT_OF_OWNERS) {
			max_owners	= max_sharers;
			return;
		}
		if (max_sharers != owner_hashCodes.length) {
			max_owners	= max_sharers;
			owner_hashCodes	= new int [max_owners + 1];
		}

		for (int i = 0; i < owner_hashCodes.length; ++i) {
			owner_hashCodes[i]	= OWNER_NONE_HASHCODE;
		}

		if (me != null) {
			owner_hashCodes[0]	= me.hashCode();
			owner_cnt	= (sharing)? 1: MONOPOLIZE;
		}

		exclusive_obj	= o;
	}

	/**
	 * ɃZ}tHpIɊlłXbh
	 * {@link #DEFAULT_CNT_OF_SHARING_OWNERS
	 * DEFAULT_CNT_OF_SHARING_OWNERS} ŊƂlŁA
	 * me XbhLIɊlA
	 * IȃXbhԋLIuWFNgw肳ꂽZ}tH
	 * 쐬܂B
	 * new Semaphore(me, sharing, o, DEFAULT_CNT_OF_SHARING_OWNERS);
	 * ƓłB
	 */
	public Semaphore(Thread me, boolean sharing, Object o) {
		this(me, sharing, o, DEFAULT_CNT_OF_SHARING_OWNERS);
	}

	/**
	 * IȃXbhԋLIuWFNg
	 * pIȃZ}tHl𓯎ɍsXbhw肳A
	 * me XbhLIɊlꂽZ}tH쐬܂B
	 * new Semaphore(me, false, o, max_sharers);
	 * ƓłB
	 */
	public Semaphore(Thread me, Object o, int max_sharers) {
		this(me, false, o, max_sharers);
	}

	/**
	 * IȃXbhԋLIuWFNgw肳A
	 * me XbhLIɊlꂽZ}tH쐬܂B
	 * new Semaphore(me, false, o); ƓłB
	 */
	public Semaphore(Thread me, Object o) {
		this(me, false, o);
	}

	/**
	 * IȃXbhԋLIuWFNg
	 * pIȊl𓯎ɍsXbhw肳A
	 * ǂ̃XbhlĂȂZ}tH쐬܂B
	 * new Semaphore(null, o, max_sharers); ƓłB
	 */
	public Semaphore(Object o, int max_sharers) {
		this(null, o, max_sharers);
	}

	/**
	 * IȃXbhԋLIuWFNgw肳A
	 * ǂ̃XbhlĂȂZ}tH쐬܂B
	 * new Semaphore(null, o); ƓłB
	 */
	public Semaphore(Object o) {
		this(null, o);
	}

	/**
	 * pIȊl𓯎ɍsXbhw肳A
	 * IȃXbhԋLIuWFNgw肳ĂȂA
	 * ǂ̃XbhlĂȂZ}tH쐬܂B
	 * new Semaphore(null, max_sharers); ƓłB
	 */
	public Semaphore(int max_sharers) {
		this(null, max_sharers);
	}

	/**
	 * IȃXbhԋLIuWFNgw肳ĂȂA
	 * ǂ̃XbhlĂȂZ}tH쐬܂B
	 * new Semaphore(null); ƓłB
	 */
	public Semaphore() {
		this(null);
	}

	/*
	 * hashCodes[] z hashCode ƈvvfB
	 * vvfȂꍇA -1 Ԃ
	 */
	private int search_owner_index(int hashCode) {
		int limit = (owner_cnt < 0)? -owner_cnt: owner_cnt;

		for (int i = 0; i < limit; ++i) {
			if (hashCode == owner_hashCodes[i]) {
				return	i; // Found
			}
		}

		return	-1; // Not found
	}

	/**
	 * me Xbh̃Z}tHl݂B
	 * sharing  true Ȃ΋pIȊlA
	 * false ȂΐLIȊlsB
	 * blocking  true ŃZ}tHlłȂꍇɑҋ@ԂɂȂB
	 * ҋ@ԂsۂɁAtimeout Ŏw肳ꂽ~b
	 * ^CAEgsB
	 * ^CAEgꍇƑXbh犄荞܂ꂽꍇ
	 * O𔭐B
	 * @param	me	XbhlsXbhB
	 * @param	sharing	pIl (true) ƐLIl (false)
	 *			w肷
	 * @param	blocking	lłȂꍇɑҋ@Ԃɓ (true)
	 *					ۂ (false) w肷
	 * @return	<ul><li>true - Z}tH̊lłƂ</li
	 *			><li>false - ubLOɃZ}tH̊l
	 *					łȂA
	 *					邢́Aȉ̂ꂩ̏ꍇB<ol
	 *					><li>me XbhɃZ}tHlĂ</li
	 *					><li>me Xbh null w肳Ă</li
	 *					><li>RXgN^ max_sharers 
	 *					{@link #MIN_CNT_OF_OWNERS MIN_CNT_OF_OWNERS}
	 *					̒lݒ肵Ă</li
	 *					></ol></li
	 *			></ul>
	 * @param	timeout	lłȂꍇ̑ҋ@^CAEgԂ~b
	 *					Ŏw肷B
	 *					1 ̒lw肵ꍇA^CAEg
	 *					sȂB
	 * @exception	TimeoutException	ҋ@^CAEgꍇB
	 *					ObZ[W
	 *					 "Semaphore.take(): Timeout (~b)"
	 * @exception	InterruptedException me Xbh̃Xbh
	 * 					interrupt() \bhĂяoŊ荞܂ꂽꍇB
	 */
	public boolean take(Thread me
						, boolean sharing
						, boolean blocking
						, long timeout
						) throws Exception, TimeoutException {
		boolean	result = false;
		long	to, wait_time;

		if (me == null) {
			return	false; // `me' thread is null (illegal argument)
		}

		to	= (timeout > 0)? timeout + System.currentTimeMillis(): 0;
		while (true) {
			synchronized (this) {
				if (max_owners < MIN_CNT_OF_OWNERS) {
					break;	// Constructor had taken the illegal
							// `max_shares' argument, yet.
				}

				int	me_idx = search_owner_index(me.hashCode());

				if (me_idx >= 0) {
					break;	// `owner' had taken this semaphore, yet.
				}
				if (!sharing) {
					if (owner_cnt == NONE_TAKES) {
						// 'owner' has taken this semaphore .
						// This thread monoplizes it.
						owner_hashCodes[NONE_TAKES]	= me.hashCode();
						owner_cnt	= MONOPOLIZE;
						result	= true;
						break; // `owner' has taken this semaphore.
					}
				}
				else if ((owner_cnt >= NONE_TAKES)
				&& owner_cnt < max_owners) {
					// 'owner' has taken this semaphore.
					// This thread will share it with other threads
					owner_hashCodes[owner_cnt] = me.hashCode();
					++owner_cnt;
					result	= true;
					break;
				}

				if (!blocking) {
					// Another thread had taken this semaphore.
					// This thread can't take it, now.
					break;
				}
				else {
					//   This thread waits for this semaphore,
					// until other threads invoke notifyAll()
					try {
						if (to != 0) {
							// Get the msecs for time-out.
							wait_time = to - System.currentTimeMillis();
							if (wait_time <= 0) {
								// Time out! Semaphore throws Exception.
								throw new TimeoutException(
												"Semaphore.take(" + me
												+ ", " + sharing + ", "
												+ blocking + ", "
												+ timeout + ")"
											, timeout
											);
							}
						}
						else {
							wait_time	= 0; // Wait for infinite.
						}

						wait(wait_time);

						if ((to != 0)
						&& to < System.currentTimeMillis()) {
							// Time out! Semaphore throws Exception.
							throw new TimeoutException(
												"Semaphore.take(" + me
												+ ", " + sharing + ", "
												+ blocking + ", "
												+ timeout + ")"
											, timeout
											);
						}
					} catch (InterruptedException e) {
						if ((to != 0)
						&& to < System.currentTimeMillis()) {
							// Time out! Semaphore throws Exception.
							throw new TimeoutException(
												"Semaphore.take(" + me
												+ ", " + sharing + ", "
												+ blocking + ", "
												+ timeout + ")"
											, timeout
											);
						}
					}
				}
			} // synchronized(this)
		}

		return	result;
	}

	/**
	 * ^CAEgs킸OȂ悤ɁA
	 * LI܂͋pIȃZ}tHlsB
	 * take(me, sharing, blocking, INFINITE);
	 * ƓB
	 */
	public boolean take(Thread me, boolean sharing, boolean blocking) {
		boolean	result;

		while (true) {
			try {
				result	= take(me, sharing, blocking, INFINITE);
				break;
			} catch (Exception e) {
				; // Ignore InterruptException
			}
		}

		return	result;
	}

	/**
	 * ^CAEgs킸OȂ悤ɁA
	 * LIȃZ}tHlsB
	 * take(me, false, blocking);
	 * ƓB
	 */
	public boolean take(Thread me, boolean blocking) {
		return	take(me, false, blocking);
	}

	/**
	 * Z}tHɊ֘AtꂽXbhԋLIuWFNg擾B
	 * @param	me	XbhԋLIuWFNg擾Xbh
	 * @return	<ul><li>null - XbhԋLIuWFNg֘At
	 *				ĂȂꍇ</li
	 *			><li> null - XbhԋLIuWFNg</li
	 *			></ul>
	 * @exception	Exception	me XbhZ}tHlĂȂ
	 *						ꍇBObZ[W
	 *						"Semaphore: This thread has not taken this
	 *						semaphore."
	 * @exception	Exception	RXgN^ŋpIȊl𓯎
	 *						sXbh max_sharers 
	 *						{@link #MIN_CNT_OF_OWNERS MIN_CNT_OF_OWNERS}
	 *						Ɏw肵ĂꍇBObZ[W
	 * 						"Semaphore: Constructor had taken the
	 *						illegal `max_sharers' argument, yet"
	 */
	public Object get_exclusive_object(Thread me) throws Exception {
		synchronized (this) {
			if (max_owners < MIN_CNT_OF_OWNERS) {
				throw new Exception("Semaphore: Constructor had taken "
									+ "the illegal `max_sharers' "
									+ "argument, yet"
									);
			}

			int	me_idx = search_owner_index(me.hashCode());

			if (me_idx < 0) {
				throw new Exception("Semaphore: This thread has not "
									+ "taken this semaphore."
									); // This thread had not taken it.
			}
		} // synchronized (this)

		return	exclusive_obj;
	}

	/**
	 * lZ}tHBۂɁAZ}tHlҋ@Ԃ
	 * XbhꍇAnotifyAll() ŃZ}tHl\ɂȂ
	 * ƂʒmB
	 * @param	me	Z}tHlĂXbh
	 * @return	<ul><li>true - Z}tHꂽƂB</li
	 *			><li>false - ȉ̂ꂩ̏ꍇB<ol
	 *					><li>me XbhZ}tHlĂȂ</li
	 *					><li>RXgN^ max_sharers 
	 *					{@link #MIN_CNT_OF_OWNERS MIN_CNT_OF_OWNERS}
	 *					̒lݒ肵Ă</li
	 *					></ol></li
	 *			></ul>
	 */
	public boolean leave(Thread me) {
		boolean	result	= false;

		synchronized (this) {
			if (max_owners < MIN_CNT_OF_OWNERS) {
				return	result;	// Constructor had taken the illegal
								// `max_shares' argument, yet.
			}

			int i = search_owner_index(me.hashCode());

			if (i >= 0) {
				while (i < owner_cnt) {
					owner_hashCodes[i] = owner_hashCodes[i + 1];
					++i;
				}
				owner_cnt += (owner_cnt != MONOPOLIZE)? -1: 1;
				result	= true;

				notifyAll();
			}
		} // synchronized (this)

		return	result;
	}
}
