/***********************************************************************
 * File: SharedObject.java
 * 	(Extracted from
 *	   "$Id: SharedObject.java,v 1.3 2004/06/02 13:35:46 maoyam Exp maoyam $
 *
 * Abstracts:
 *   This Java source code implements the shared-object utility class.
 *   (Refered interface: "Sharable")
 *   (Refered class: "Semaphore")
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package	my_util;

import	java.util.Hashtable;

/**
 * <p>Sharable C^tF[XIuWFNgXbhԂŋL
 * 邽߂̃bN@\񋟂郆[eBeBENXłB</p>
 * @author maoyam
 * @version 1.0
 * @see my_util.Sharable
 * @see my_util.Semaphore
 */
public class SharedObject {
	static private Hashtable sos = new Hashtable();

	private Semaphore sem = null;

	private SharedObject(Object obj, int max_sharers) {
		sem	= new Semaphore(obj, max_sharers);
	}

	/**
	 * XbhԂŋLIuWFNgp̃bN쐬܂B
	 * XbhԂŖObNƂ SharedObject gpꍇ́A
	 * LIuWFNg̃NX sharable_class_name  null w
	 * ܂B
	 * @param key bN̖O
	 * @param sharable_class_name Sharable C^tF[X
	 *                            NX̃NXBObN
	 *                            @\gpꍇ́Anull w
	 * @param max_sharers bNpőXbh
	 * @return <ul><li>null -- ObN̍쐬Ɏsꍇ</li
	 *            ><li> null -- 쐬ꂽObN</li
	 *            ></ul>
	 * @exception Exception sharable_class_name Ŏw肳ꂽNX
	 *                      ݂Ȃ邢́ÃNX
	 *                      CX^XɎsꍇ
	 */
	static public SharedObject create(String key
									, String sharable_class_name
									, int max_sharers
									) throws Exception {
		return	((key == null || key.compareTo("") == 0)?
					null:
					create(key
						, ((sharable_class_name == null)?
								null:
								Class.forName(sharable_class_name)
							)
						, max_sharers
						)
				);
	}

	/**
	 * XbhԂŋLIuWFNgp̃bN쐬܂B
	 * XbhԂŖObNƂ SharedObject gpꍇ́A
	 * LIuWFNgENX sharable_class  null w肵܂B
	 * @param key bN̖O
	 * @param sharable_class Sharable C^tF[X
	 *                            NX̃NXBObN
	 *                            @\gpꍇ́Anull w
	 * @param max_sharers bNpőXbh
	 * @return <ul><li>null -- ObN̍쐬Ɏsꍇ</li
	 *            ><li> null -- 쐬ꂽObN</li
	 *            ></ul>
	 * @exception Exception sharable_class Ŏw肳ꂽNX
	 *                      CX^XɎsꍇ
	 */
	static public SharedObject create(String key
									, Class	sharable_class
									, int max_sharers
									) throws Exception {
		SharedObject so	= null;
		Sharable	obj	= null;

		if ((key != null)
		&& (sharable_class == null
			|| Sharable.class.isAssignableFrom(sharable_class)
			)
		&& max_sharers >= Semaphore.MIN_CNT_OF_OWNERS) {
			synchronized (sos) {
				so	= 	(SharedObject)(sos.get(key));
				if (so == null) {
					if (sharable_class != null) {
						obj	= (Sharable)(sharable_class.newInstance());
					}
					else {
						obj	= null;
					}
					so	= new SharedObject(obj, max_sharers);
					sos.put(key, so);
				}
			} // synchronized (sos)
		}

		return	so;
	}

	/**
	 * ɍ쐬ꂽObN擾܂B
	 * @param key bN̖O
	 * @return <ul><li>null -- key ɑΉ閼ObN쐬
	 *                         ĂȂꍇ</li
	 *            ><li> null -- key ɑΉtꂽObN</li
	 *            ></ul>
	 */
	static public SharedObject attach(String key) {
		SharedObject	so = null;

		if (key != null) {
			synchronized (sos) {
				so	= (SharedObject)(sos.get((Object)key));
			} // synchronized (sos)
		}

		return	so;
	}

	/**
	 * XbhԂŋLIuWFNg̏s܂B
	 * l͕zŎw肵܂B
	 * IuWFNg̏ Sharable C^tF[X initialize
	 * \bhgpĎs܂B
	 * @param vals LIuWFNg̏lw肷镶z
	 * @return <ul><li>true -- ɐꍇ</li
	 *            ><li>false -- Ɏsꍇ</li
	 *            ></ul>
	 * @exception Exception ObN̐LlŃG[Nꍇ
	 *                      邢͋LIuWFNg̎擾Ɏsꍇ
	 */
	public boolean initialize(String[] vals) throws Exception {
		boolean	result = false;

		if (take(false // monopolize
					, true // blocking
					, 0
					)) {
			Sharable	obj = get_object();

			if (obj != null) {
				obj.initialize(vals);
			}

			result	= leave();
		}
		else {
			// Can't take semaphore of this SharedObjcet
			throw new Exception("SharingObject: "
								+ "Can't take exclusive semaphore"
								);
		}

		return	result;
	}

	/**
	 * ObN̐L/LbNs܂B
	 * r̎s Semaphore NX take() \bhgp
	 * Ă܂B
	 * @param sharing LbNsꍇ true A
	 *                LbNsꍇ false w肵܂
	 * @param blocking bNl܂ŏubNꍇ true A
	 *                 ubNɂɃ^[ꍇ false
	 *                 w肵܂
	 * @param time_out blocking  true ̏ꍇAubLO
	 *                 ő~bw肵܂BɃbNlҋ@
	 *                 ꍇ 0 w肵܂
	 * @return <ul><li>true -- bNlꂽꍇ</li
	 *            ><li>false -- bNlsꍇ</li
	 *            ></ul>
	 * @exception Exception Semaphore.take() \bhQƂĂ
	 */
	public boolean take(boolean sharing
						, boolean blocking
						, long time_out
						) throws Exception {
		return	sem.take(Thread.currentThread()
						, sharing
						, blocking
						, time_out
						);
	}

	/**
	 * take() ŊɊlꂽObNɊ֘AtĂ
	 * LIuWFNg擾܂B
	 * @return <ul><li>null -- ֘AtꂽIuWFNg݂Ȃ
	 *                         ꍇ</li
	 *            ><li> null -- ֘AtꂽIuWFNg</li
	 *            ></ul>
	 * @exception Exception Semaphore.get_exclusive_object() \bh
	 *                      QƂĂ
	 */
	public Sharable	get_object() throws Exception {
		return	(Sharable)(sem.get_exclusive_object(
										Thread.currentThread()
										)
							);
	}

	/**
	 * take() ŊlꂽObN܂B
	 * @return <ul><li>true -- bNꂽꍇ</li
	 *            ><li>false -- bNɎsꍇ</li
	 *            ></ul>
	 */
	public boolean leave() {
		return	sem.leave(Thread.currentThread());
	}
}
