/***********************************************************************
 * File: Timer.java
 * 	(Extracted from
 *	   "$Id: Timer.java,v 1.7 2004/07/04 12:37:08 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a Timer utility class.
 *  This class runs on the thread to dispatch TimerCommand-s to agents.
 *  (Refer Class: TimerCommand)
 *  (Refer Interface: TimerUser)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

import java.util.*;

/**
 * <p>Facility ɕbx̃^C}@\񋟂邽߂ɁA<em
 *   >AvP[V</em>p̃XbhŎs܂B</p>
 * <p>^C}gp<em>@\</em> FacilityAgent  TimerUser
 *    C^tF[XKv܂B^CAEg邢
 *    ^C}jꂽƂʒm邽߂ɁATimer NX́A
 *    TimerComand  resp_queue_name tB[hŎ Message L[
 *     TimerCommand 𑗐M܂B̍ہA TimerCommand ̓L[̐擪
 *    ɑ}܂B</p>
 *
 * @author maoyam
 * @version 1.0
 * @see TimerCommand
 * @see TimerUser
 * @see Facility
 * @see Loggable
 * @see Log
 */
public class Timer implements Runnable, Loggable {
	static final private long TIMER_RESOLUTION = 1000; // 1sec
	static final private long SEND_TIMEOUT = 100; // 100msecs

	// the array of the registered TimerCommand-s
	private TimerCommand[] users = { null };
	// the count of the registered TimerCommand-s
	private int users_cnt = 0;
	// the minimum expire-time
	private long min_exp_time = Long.MAX_VALUE;

	// flag to halt this timer (set true, this timer halts)
	private boolean halt_timer	= false;

	// logging configurations
	private LogConf log_conf = null;

	// logging the timer errors
	private Log log = null;
	/**
	 * Oo͐ Log ̕\擾܂B
	 * @return Log ̕\
	 */
	protected String get_log() {
		synchronized (this) {
			return ((log != null)? log.toString(): "null");
		} // synchronized (this)
	}

	/**
	 * Oo͐pX擾܂B
	 * @return Oo͐pX
	 */
	protected String get_log_path() {
		synchronized (this) {
			return ((log_conf != null)? log_conf.path: "null");
		} // synchronized (this)
	}

	/**
	 * Oo͕GR[fBO擾܂B
	 * @return Oo͕GR[fBO
	 */
	protected String get_log_encoding() {
		synchronized (this) {
			return ((log_conf != null)? log_conf.encoding: "null");
		} // synchronized (this)
	}

	/**
	 * OEx擾܂B
	 * @return OEx
	 */
	protected int get_log_level() {
		synchronized (this) {
			return ((log_conf != null)? log_conf.level: -1);
		} // synchronized (this)
	}

	/**
	 * OEtB[h؂蕶擾܂B
	 * @return OEtB[h؂蕶
	 */
	protected String get_log_delimit() {
		synchronized (this) {
			return ((log_conf != null)? log_conf.delimit: "null");
		} // synchronized (this)
	}


	/**
	 * <p>RXgN^: Thread Ŏs Timer CX^X𐶐
	 *    ܂B</p>
	 * @param log_conf Oo͐̍\B
	 *                 my_util.LogConf QƂ̂
	 */
	public Timer(LogConf log_conf) {
		this.log_conf	= log_conf;
		this.log	= new Log(log_conf);
		if (this.log != null) {
			try {
				this.log.open();
			} catch (Exception e) {
				this.log.close();
				this.log = null;
			}
		}
	}

	/**
	 * ^C}@\̏Iwݒ肷邽߂ɌĂяo܂B
	 * ̃\bhĂяoÃCX^XsĂ Thread
	 *  join() Kv܂B
	 */
	public void halt() {
		synchronized (this) {
			// set the falg to halt this timer
			halt_timer	= true;
			notifyAll(); // interrupt to waiters of this timer
		} // synchronized (this)
	}

	// adjust the minimum expire time (msec)
	private void adjust_min_exp_time(long exp_time) {
		if (exp_time < min_exp_time) {
			min_exp_time	= exp_time;
		}
	}

	// append the new TimerCommand to the users array
	// @param cmd the new TimerCommand
	private void append(TimerCommand cmd) {
		// chec the users is full?
		if (users.length == users_cnt) {
			// expand the users array
			TimerCommand[] tc = new TimerCommand[users_cnt + 1];
			for (int i = 0; i < users_cnt; ++i) {
				tc[i]	= users[i];
			}
			users	= tc;
		}

		// adjust the minimum expire time
		adjust_min_exp_time(cmd.getExpireTimeMillis());

		// put the new one at the last element of users array
		users[users_cnt++]	= cmd;
	}

	/**
	 * ^C}@\gp Facility ^CAEgݒo^邽߂
	 * Ăяo܂B^CAEg܂ł̃C^[oԂ 1 b
	 * Ă͂܂B܂A^CAEgʒm̂߂ TimerCommand
	 *  null łĂ͂܂B
	 *
	 * @param intervals ^CAEg܂ł̃C^[oԂbx
	 *                  w肵܂ (sec)
	 * @param periodic true ̏ꍇ͎IɃ^CAEgA
	 *                 false ̏ꍇ͈x^CAEg܂B
	 * @param cmd ^CAEg Facility ̗vL[ɑM
	 *            TimerCommand
	 * @return <ul><li> null -- ^CAEgݒ̓o^ꍇ
	 *                            ŁAcmd Ԃ܂B</li
	 *            ><li>null -- ^CAEgݒ肪sȏꍇ</li
	 *            ></ul>
	 */
	public TimerCommand add(long intervals
									, boolean periodic
									, TimerCommand cmd
									) {
		// get the current system time (msec)
		long now = System.currentTimeMillis();

		// check the parameters
		if (cmd == null || intervals <= 0) {
			// The parameter is illegal!
			return null; // returns null
		}

		// change intervals seconds to milli-seconds
		intervals	*= TIMER_RESOLUTION;

		synchronized (this) {
			// save the minimum expire time
			long saved_min_exp_time = min_exp_time;

			// check this timer is running?
			if (!halt_timer) {
				// check this command isn't registered, yet?
				int i;
				for (i = 0; i < users_cnt; ++i) {
					if (users[i].equals(cmd)) break; // This exists!
				}
				if (i < users_cnt) {
					// This command was registerd, yet!
					cmd	= null; // returns null: add the same command
				}
				else {
					// set the expiration system time (msec)
					cmd.setIntervals(intervals);
					cmd.setExpireTimeMillis(now + intervals);
					cmd.setPeriodic(periodic); // set periodic/once flag
					append(cmd); // append the TimerCommand to users
				}
			}
			else {
				// This timer halted!
				cmd	= null; // returns null
			}

			// if the minimum expire time is changed...
			if (min_exp_time != saved_min_exp_time) {
				notifyAll(); // interrupt to waiters of this users
			}
		} // synchronized (this)

		return	cmd; // returns not-null: finished to register command
					 // return null: failed to register
	}

	/**
	 * o^^CAEgݒ߂ Facility Ăяo
	 * ܂B
	 *
	 * @param cmd add() \bhœo^ TimerCommand
	 * @return <ul><li> null -- ^CAEgݒ̎
	 *                            ꍇ cmd Ԃ܂B</li
	 *            ><li>null -- ݒ̎Ɏsꍇ</li
	 *            ></ul>
	 */
	public TimerCommand cancel(TimerCommand cmd) {
		// check the parameters
		if (cmd == null) {
			return	null; // returns null: The parameter is illegal!
		}

		synchronized (this) {
			// save the minimum expire time
			long saved_min_exp_time = min_exp_time;

			// check this timer is running?
			if (!halt_timer) {
				// check the users is exist?
				if (users_cnt == 0) {
					// No user exists!
					return null; // Failed: not found the cmd
				}

				// search the canceled TimerCommand in the users...
				// and adjust the minimum expire time
				min_exp_time	= Long.MAX_VALUE;
				boolean found = false;
				for (int i = 0; i < users_cnt; ++i) {
					if (users[i].equals(cmd)) {
						// find the canceled TimerCommand
						found	= true;

						// move the rest part to ahead in users
						--users_cnt;
						for (int j = i; j < users_cnt; ++j) {
							users[j]	= users[j + 1];

							// adjust the minimum expire time
							adjust_min_exp_time(
										users[j].getExpireTimeMillis());
						}
						// clear the last element of users
						users[users_cnt]	= null;

						break; // canceled this command
					}

					// adjust the minimum expire time
					adjust_min_exp_time(users[i].getExpireTimeMillis());
				}

				// check the expiration isn't found?
				if (!found) {
					cmd	= null; // Failed: un-found the cmd
				}
			}
			else {
				// This timer halted!
				cmd	= null; // returns null
			}

			// if the minimum expire time is changed...
			if (min_exp_time != saved_min_exp_time) {
				notifyAll(); // interrupt to waiters of users
			}
		} // synchronized (this)

		return	cmd;
	}

	/**
	 * OEbZ[Wo͂܂B
	 * o̓tH[}bg yyyy/mm/dd HH:MM:SS.sss:ERROR:0:<em>msg</em>
	 * łB
	 *
	 * @param log_class O擾x
	 * @param msg OEbZ[W
	 */
	public void log(int log_class, String msg) {
		synchronized (this) {
			if (log != null && log_conf.level >= log_class) {
				log.print(log_class, msg);
			}
		}
	}

	// logging the error
	// @param msg error message
	private void log(String msg) {
		StringBuffer sb = new StringBuffer(4096);
		sb.append("Timer").append(log_conf.delimit);
		sb.append("0").append(log_conf.delimit).append(msg);

		log(Log.ERROR, sb.toString());
	}

	// logging the information
	// @param msg information message
	private void log_info(String msg) {
		StringBuffer sb = new StringBuffer(4096);
		sb.append("Timer").append(log_conf.delimit);
		sb.append("0").append(log_conf.delimit).append(msg);

		log(Log.INFO, sb.toString());
	}

	/**
	 * Ȍo͐EGR[fBOE擾xE
	 * tB[h؂蕶ύX܂B
	 *
	 * @param log_conf Oo͐̍\B
	 *                 my_util.LogConf QƂ̂ƁB
	 * @return <ul><li>true -- O̐؂ւɐꍇ</li
	 *            ><li>false -- O̐؂ւɎsꍇ</li
	 *            ></ul>
	 * @exception Exception O̐؂ւɎsꍇ
	 */
	public boolean change_log(LogConf log_conf) throws Exception {
		boolean r = true, result = false;;

		synchronized (this) {
			if (log != null) {
				r = log.change(log_conf);
			}
			else {
				log	= new Log(log_conf);
				log.open();
			}
			if (log != null && r == true) {
				this.log_conf	= log_conf;
				result	= true; // success!
			}
		}

		return	result;
	}

	// send the TimerCommand to the user's facility.
	// the command means "timer-expired" or "timer-halted".
	// @param cmd the TimerCommand to send to user
	private void send_expiration(TimerCommand cmd) {
		// TimerCommand fac_name field is the facility name.
		Facility fac = Facility.attach(cmd.getFac_name());

		// check the facility is exist?
		if (fac != null) {
			// try to send the TimerCommand to the user's facility
			try {
				if (!fac.put_request(cmd, SEND_TIMEOUT, true)) {
					log("Can't put the \""
						+ ((cmd.getHalt_this_timer())?
									"timer halt": "timer expired")
						+ "\" event to \"" + cmd.getFac_name()
						+ "\" facility (intervals = "
						+ cmd.getIntervals() + " ("
						+ ((cmd.getPeriodic())? "periodic": "once")
						+ "))"
						);
				}
			} catch (Exception e) {
				log("Occures exception to put \""
					+ ((cmd.getHalt_this_timer())?
								"timer halt": "timer expired")
					+ "\" event to \"" + cmd.getFac_name()
					+ "\" facility. "
					+ "Exception-MSG: \"" + e.getMessage()
					+ "\" (intervals = " + cmd.getIntervals()
					+ " (" + ((cmd.getPeriodic())? "periodic": "once")
					+ "))"
					);
			}
		}
		else {
			log("\"" + cmd.getFac_name() + "\" facility isn't exist. "
				+ "(intervals = " + cmd.getIntervals() + " ("
				+ ((cmd.getPeriodic())? "periodic": "once") + "))"
				);
		}
			
	}

	// dispatch the TimerComand to all facilities,
	// for notifing "Timer Halts" event.
	// @param now the current system time (msec)
	private void notify_timer_halts(long now) {
		// close the log stream
		if (log != null) {
			log_info("application timer " + this + " finished.");
			log.close();
			log	= null;
		}

		// check is there users?
		if (users_cnt == 0) {
			return; // No processing!
		}

		// loop for the each TimerCommand-s...
		for (int i = 0; i < users_cnt; ++i) {
			// set the current system time
			users[i].setExpiredTime(now);

			// set the timer-halted flag
			users[i].setHalt_this_timer(true);

			// send the TimerCommand to queue
			send_expiration(users[i]);

			// clear the element of the user array
			users[i]	= null;
		}
		users_cnt	= 0; // be zero the counter of the users array
	}

	// dispatch the TimerCommand to expired facilities
	// @param now the current system time (msec)
	private void notify_expired_facility(long now) {
		// loop for the each TimerCommand-s...
		// adjust the minimum expire time
		min_exp_time = Long.MAX_VALUE;
		long exp_time = 0;
		int p = 0, i = 0;
		while (i < users_cnt) {
			// search the non-checked element...
			while (users[p] == null) {
				if (++i >= users_cnt) {
					break; // all elements are checkd!
				}
				users[p]	= users[i];
				users[i]	= null;
			}
			if (i >= users_cnt) {
				break; // all elements are checkd!
			}

			// check this TimerComannd expires?
			if ((exp_time = users[p].getExpireTimeMillis()) <= now) {
				// Expire!
				users[p].setExpiredTime(now); // set the current time
				users[p].setLastExpireTimeMillis(exp_time);
											// set the last expire time
				send_expiration(users[p]); // send this command to queue

				// check this TimerCommand is periodic?
				if (users[p].getPeriodic()) {
					// this TimerCommand is periodic
					// set the next expiration time (msec)
					users[p].setExpireTimeMillis(exp_time
											+ users[p].getIntervals());

					// adjust the minimum expire time
					adjust_min_exp_time(users[p].getExpireTimeMillis());

					++p; ++i;
				}
				else {
					// this TimerCommand is once expiration
					users[p]	= null;
				}
			}
			else { // Not expire
				// adjust the minimum expire time
				adjust_min_exp_time(users[p].getExpireTimeMillis());

				++p; ++i;
			}
		}
		users_cnt	= p; // p is the count of elements in users
	}

	/**
	 * Runnable C^tF[Xɂ邱ƂKvȃ\bhB
	 * ̃\bhŁATimer ̓^CAEǧo<em>@\</em>ւ
	 * TimerCommand ʒmsB
	 */
	public void run() {
		long	idle_msec, now;

		log_info("application timer " + this + " starts...");
		synchronized (this) {
			while (true) {
				// get the current system time
				now	= System.currentTimeMillis();

				// check this timer is ordered to halt?
				if (halt_timer) {
					// dispatch the TimerComand to all facilities,
					// for notifing "This timer halts" event.
					notify_timer_halts(now);

					break; // Halt this timer!
				}

				// check there is the users?
				if (users_cnt > 0) {
					// dispatch the TimerCommand to expired facilities
					notify_expired_facility(now);

					// sleeping for TICK times...
					idle_msec = min_exp_time
										- System.currentTimeMillis();
					if (idle_msec <= 0) {
						idle_msec	= 1;
					}
				}
				else {
					// no user exists
					idle_msec	= 0; // sleeping infinite...
				}

				// sleeps slight times...
				try {
					wait(idle_msec);
				} catch (InterruptedException ie) {
					; // ignore interruption
				}
			}
		} // synchronized (this)
	}
}
