/***********************************************************************
 * File: TimerCommand.java
 * 	(Extracted from
 *	   "$Id: TimerCommand.java,v 1.3 2004/06/15 13:36:38 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a TimerCommand utility class.
 *  This class is a command for regitration and canceling the
 * "Timer" expiration.
 *  (Refered Class: Timer, TimerUser, Loggable)
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package	my_util;

/**
 * <p>Timer NXɃ^CAEgʒm˗邽߂̃R}hłB</p>
 *
 * @author maoyam
 * @version 1.0
 * @see Timer
 * @see TimerUser
 * @see Loggable
 * @see FacilityCommand
 */
public class TimerCommand extends FacilityCommand {
	// input: expiration interval (msec)
	private long	intervals = 0;
	/**
	 * ^CAEg܂ł̃C^[oԊu~bxŎ擾܂
	 * @return C^[oԊu (msec)
	 */
	public long getIntervals() {
		synchronized (this) { return intervals; }
	}
	/**
	 * ^CAEg܂ł̃C^[oԊu~bxŐݒ肵܂
	 * @param intervals C^[oԊu (msec)
	 */
	public void setIntervals(long intervals) {
		synchronized (this) { this.intervals = intervals; }
	}

	// input: expiration system time (msec)
	private long expireTimeMillis = 0;
	/**
	 * ^CAEgɓBVXe~bxŎ擾܂
	 * @return ^CAEgɓBVXe (msec)
	 */
	public long getExpireTimeMillis() {
		synchronized (this) { return expireTimeMillis; }
	}
	/**
	 * ^CAEgɓBVXe~bxŐݒ肵܂
	 * @param expireTimeMillis ^CAEgɓBVXe (msec)
	 */
	public void setExpireTimeMillis(long expireTimeMillis) {
		synchronized (this) {
			this.expireTimeMillis = expireTimeMillis;
		}
	}

	// output: the expiration time (msec) just in expired
	private long lastExpireTimeMillis = -1;
	/**
	 * ^CAEgBoƂ̃^CAEgBݒl
	 * ~bxŎ擾܂
	 * @return ^CAEgBõ^CAEgBݒl (msec)
	 */
	public long getLastExpireTimeMillis() {
		synchronized (this) { return lastExpireTimeMillis; }
	}
	/**
	 * ^CAEgBoƂ̃^CAEgBݒl
	 * ~bxŐݒ肵܂
	 * @param lastExpireTimeMillis ^CAEgBo
	 *                             ^CAEgBݒl (msec)
	 */
	public void setLastExpireTimeMillis(long lastExpireTimeMillis) {
		synchronized (this) {
			this.lastExpireTimeMillis = lastExpireTimeMillis;
		}
	}

	// input: flag periodic/once-expiration
	private boolean periodic = false;
	/**
	 * Iȃ^C} periodic sۂɂĎ擾܂
	 * @return <ul><li>true -- I^C} periodic sꍇ</li
	 *            ><li>false -- ̃^C}sꍇ</li
	 *            ></ul>
	 */
	public boolean getPeriodic() {
		synchronized (this) { return periodic; }
	}
	/**
	 * Iȃ^C} periodic sۂɂĐݒ肵܂
	 * @param periodic I^C} periodic ɐݒ肷ɂ true A
	 *                 ̃^C}ɐݒ肷ɂ false w肵܂
	 */
	public void setPeriodic(boolean periodic) {
		synchronized (this) { this.periodic = periodic; }
	}

	// input: Logging interface implementation
	private Loggable logger = null;
	/**
	 * ^C}˗w肷郍O@\CX^X擾܂B
	 * @return <ul><li> null -- Loggable CX^X̎Q</li
	 *            ><li>null -- Loggable CX^Xw肳ĂȂ
	 *                         ꍇ</li
	 *            ></ul>
	 */
	public Loggable getLogger() {
		synchronized (this) { return logger; }
	}
	/**
	 * ^C}˗w肷郍O@\CX^Xݒ肵܂B
	 * @param logger Loggable CX^XB
	 */
	public void setLogger(Loggable logger) {
		synchronized (this) { this.logger = logger; }
	}

	// output: expired system time (msec)
	private long expiredTime = 0;
	/**
	 * ^CAEgoꂽVXe (msec) 擾܂
	 * @return ^CAEgoVXe (msec)
	 */
	public long getExpiredTime() {
		synchronized (this) { return expiredTime; }
	}
	/**
	 * ^CAEgoꂽVXe (msec) ݒ肵܂
	 * @param expiredTime ^CAEgoVXe (msec)
	 */
	public void setExpiredTime(long expiredTime) {
		synchronized (this) { this.expiredTime = expiredTime; }
	}

	// output: this timer halts or not?
	private boolean halt_this_timer = false;
	/**
	 * ^C}jꂽۂɂĎ擾܂
	 * @return <ul><li>true -- ^C}jꂽꍇ</li
	 *            ><li>false -- ^C}삵Ăꍇ</li
	 *            ></ul>
	 */
	public boolean getHalt_this_timer() {
		synchronized (this) { return halt_this_timer; }
	}
	/**
	 * ^C}jꂽۂɂĐݒ肵܂
	 * @param halt_this_timer ^C}jꂽꍇ true A
	 *                        삵Ăꍇ false w肵܂
	 */
	public void setHalt_this_timer(boolean halt_this_timer) {
		synchronized (this) { this.halt_this_timer = halt_this_timer; }
	}
}
