/***********************************************************************
 * File: Wc.java
 * 	(Extracted from
 *	   "$Id: Wc.java,v 1.5 2004/06/28 15:01:09 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This Java source code implements a Wc utility tool.
 *  This tool counts lines and words of files.
 *
 * Copyright (C) 2004 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

package my_util;

import java.io.*;
import java.util.StringTokenizer;

/**
 * <p>t@C̍sƌꐔvc[łB</p
 * <p>java my_util.Wc [file...]</p>
 * <p>t@CƂ̍sEꐔesɏo͂ASt@C̍v
 *    o͂܂Bt@Cwȗꂽꍇ́AW͂̍sƌꐔ
 *    v܂B</p>
 */
public class Wc {
	static long wcnt = 0, lcnt = 0, f_size = -1;

	// Any one can't create the instance of this class!
	private Wc() {}

	static private void word_count(BufferedReader in) throws Exception {
		String line;
		while ((line = in.readLine()) != null) {
			++lcnt;
			StringTokenizer st = new StringTokenizer(line);
			while (st.hasMoreTokens()) {
				st.nextToken();
				++wcnt;
			}
		}
	}

	/**
	 * vOGg
	 * @param args t@CEpX̕z
	 */
	static public void main(String[] args) {
		try {
			if (args.length == 0) {
				word_count(new BufferedReader(
									new InputStreamReader(System.in)
									));
			}
			else {
				long	l = 0, w = 0, sz = 0;
				for (int i = 0; i < args.length; ++i) {
					File	f = new File(args[i]);
					if (!f.exists() || !f.isFile() || !f.canRead()) {
						System.err.print("File " + args[i]
										+ " is non-exist, isn't a file "
										+ "or can't read.\n"
										);
						break;
					}

					BufferedReader br = new BufferedReader(
											new InputStreamReader(
												new FileInputStream(f)
											));
					wcnt = lcnt = 0; f_size = f.length();
					word_count(br);
					l += lcnt; w += wcnt; sz += f_size;
					if (args.length > 1) {
						System.out.println("" + lcnt
											+ "\t" + wcnt
											+ "\t" + f_size
											+ "\t" + args[i]
											);
					}
					br.close();
				}
				lcnt	= l; wcnt	= w; f_size = sz;
			}

			System.out.println("" + lcnt + "\t" + wcnt
								+ ((f_size > -1)? ("\t" + f_size): "")
								);
		} catch (Exception e) {
			System.err.print(e.getMessage());
			e.printStackTrace();
		}
	}
}
